@@ -32,24 +32,19 @@
 #include <kapplication.h>
 #include <kcmdlineargs.h>
 #include <klocale.h>
-#include <kxmlguiwindow.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
 #include <ksplashscreen.h>
+#include <kmessagebox.h>
 #include <ktexteditor/cursor.h>
-#include <KMessageBox>
-#include <KProcess>
 
 #include <QFileInfo>
-#include <QPixmap>
-#include <QTimer>
 #include <QDir>
+#include <QProcess>
 #include <QSessionManager>
-#include <QThread>
-#include <QDBusConnection>
+#include <QTextStream>
 #include <QDBusInterface>
 #include <QDBusReply>
-#include <QTextStream>
 
 #include <shell/core.h>
 #include <shell/mainwindow.h>
@@ -62,15 +57,24 @@
 #include <shell/session.h>
 #include <interfaces/ilauncher.h>
 #include <interfaces/iproject.h>
+#include <interfaces/launchconfigurationtype.h>
 
 #include "kdevideextension.h"
 
 #include <iostream>
 
 #include "welcomepage/welcomepageview.h"
+#include "splash.h"
+
+#ifdef Q_OS_MAC
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 
 using KDevelop::Core;
 
+// Represents a file to be opened, consisting of its URL and the linenumber to jump to
+typedef QPair<QString, int> File;
+
 class KDevelopApplication: public KApplication {
 public:
     explicit KDevelopApplication(bool GUIenabled = true): KApplication(GUIenabled) {}
@@ -90,7 +94,7 @@ public:
 /// Tries to find a session identified by @p data in @p sessions.
 /// The @p data may be either a session's name or a string-representation of its UUID.
 /// @return pointer to the session or NULL if nothing appropriate has been found
-const KDevelop::SessionInfo* findSessionInList( QList<KDevelop::SessionInfo>& sessions, const QString& data )
+static const KDevelop::SessionInfo* findSessionInList( QList<KDevelop::SessionInfo>& sessions, const QString& data )
 {
     // We won't search a session without input data, since that could lead to false-positives
     // with unnamed sessions
@@ -106,11 +110,96 @@ const KDevelop::SessionInfo* findSessionInList( QList<KDevelop::SessionInfo>& se
     return 0;
 }
 
+/// Parses a filename given as an argument by determining its line number and full path
+static File parseFilename(QString argument)
+{
+    if ( KUrl::isRelativeUrl(argument) && ! argument.startsWith('/') ) {
+        argument = QDir::currentPath() + "/" + argument;
+    }
+    //Allow opening specific lines in documents, like mydoc.cpp:10
+    int lineNumberOffset = argument.lastIndexOf(':');
+    int line = -1;
+    if( lineNumberOffset != -1 )
+    {
+        bool isInt;
+        int lineNr = argument.mid(lineNumberOffset+1).toInt(&isInt);
+        if (isInt)
+        {
+            argument = argument.left(lineNumberOffset);
+            line = lineNr;
+        }
+    }
+    return File(argument, line);
+}
+
+/// Performs a DBus call to open the given @p files in the running kdev instance identified by @p pid
+/// Returns the exit status
+static int openFilesInRunningInstance(const QVector<File>& files, int pid)
+{
+    const QString service = QString("org.kdevelop.kdevelop-%1").arg(pid);
+    QDBusInterface iface(service, "/org/kdevelop/DocumentController", "org.kdevelop.DocumentController");
+
+    QStringList urls;
+    bool errors_occured = false;
+    foreach ( const File& file, files ) {
+        QDBusReply<bool> result = iface.call("openDocumentSimple", file.first);
+        if ( ! result.value() ) {
+            QTextStream err(stderr);
+            err << i18n("Could not open file %1.", file.first) << "\n";
+            errors_occured = true;
+        }
+    }
+    // make the window visible
+    QDBusMessage makeVisible = QDBusMessage::createMethodCall( service, "/kdevelop/MainWindow", "org.kdevelop.MainWindow",
+                                                               "ensureVisible" );
+    QDBusConnection::sessionBus().asyncCall( makeVisible );
+    return errors_occured ? 1 : 0;
+}
+
+/// Gets the PID of a running KDevelop instance, eventually asking the user if there is more than one.
+/// Returns -1 in case there are no running sessions.
+static int getRunningSessionPid()
+{
+    QList<KDevelop::SessionInfo> candidates;
+    foreach( const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo() ) {
+        if( KDevelop::SessionController::isSessionRunning(si.uuid.toString()) ) {
+            candidates << si;
+        }
+    }
+    if ( candidates.isEmpty() ) {
+        return -1;
+    }
+
+    QString sessionUuid;
+    if ( candidates.size() == 1 ) {
+        sessionUuid = candidates.first().uuid.toString();
+    }
+    else {
+        const QString title = i18n("Select the session to open the document in");
+        sessionUuid = KDevelop::SessionController::showSessionChooserDialog(title, true);
+    }
+    return KDevelop::SessionController::sessionRunInfo(sessionUuid).holderPid;
+}
+
 int main( int argc, char *argv[] )
 {
+#ifdef Q_OS_MAC
+    CFBundleRef mainBundle = CFBundleGetMainBundle();
+    if (mainBundle) {
+        // get the application's Info Dictionary. For app bundles this would live in the bundle's Info.plist,
+        // for regular executables it is obtained in another way.
+        CFMutableDictionaryRef infoDict = (CFMutableDictionaryRef) CFBundleGetInfoDictionary(mainBundle);
+        if (infoDict) {
+            // Try to prevent App Nap on OS X. This can be tricky in practice, at least in 10.9 .
+            CFDictionarySetValue(infoDict, CFSTR("NSAppSleepDisabled"), kCFBooleanTrue);
+            CFDictionarySetValue(infoDict, CFSTR("NSSupportsAutomaticTermination"), kCFBooleanFalse);
+        }
+    }
+#endif
+
     static const char description[] = I18N_NOOP( "The KDevelop Integrated Development Environment" );
     KAboutData aboutData( "kdevelop", 0, ki18n( "KDevelop" ), QByteArray(VERSION), ki18n(description), KAboutData::License_GPL,
-                          ki18n( "Copyright 1999-2012, The KDevelop developers" ), KLocalizedString(), "http://www.kdevelop.org/" );
+                          ki18n( "Copyright 1999-2014, The KDevelop developers" ), KLocalizedString(), "http://www.kdevelop.org/" );
     aboutData.addAuthor( ki18n("Andreas Pakulat"), ki18n( "Architecture, VCS Support, Project Management Support, QMake Projectmanager" ), "apaku@gmx.de" );
     aboutData.addAuthor( ki18n("Alexander Dymo"), ki18n( "Architecture, Sublime UI, Ruby support" ), "adymo@kdevelop.org" );
     aboutData.addAuthor( ki18n("David Nolden"), ki18n( "Definition-Use Chain, C++ Support, Code Navigation, Code Completion, Coding Assistance, Refactoring" ), "david.nolden.kdevelop@art-master.de" );
@@ -120,6 +209,7 @@ int main( int argc, char *argv[] )
     aboutData.addAuthor( ki18n("Amilcar do Carmo Lucas"), ki18n( "Website admin, API documentation, Doxygen and autoproject patches" ), "amilcar@kdevelop.org" );
     aboutData.addAuthor( ki18n("Niko Sams"), ki18n( "GDB integration, Webdevelopment Plugins" ), "niko.sams@gmail.com" );
     aboutData.addAuthor( ki18n("Milian Wolff"), ki18n( "Co-Maintainer, Generic manager, Webdevelopment Plugins, Snippets, Performance" ), "mail@milianw.de" );
+    aboutData.addAuthor( ki18n("Sven Brauch"), ki18n( "Python Support, User Interface improvements" ), "svenbrauch@gmail.com" );
 
     aboutData.addCredit( ki18n("Matt Rogers"), KLocalizedString(), "mattr@kde.org");
     aboutData.addCredit( ki18n("Cédric Pasteur"), ki18n("astyle and indent support"), "cedric.pasteur@free.fr" );
@@ -169,6 +259,7 @@ int main( int argc, char *argv[] )
     aboutData.addCredit( ki18n("Sascha Cunz") , ki18n( "Cleanup and bugfixes for qEditor, AutoMake and much other stuff" ), "mail@sacu.de" );
     aboutData.addCredit( ki18n("Zoran Karavla"), ki18n( "Artwork for the ruby language" ), "webmaster@the-error.net", "http://the-error.net" );
 
+
     //we can't use KCmdLineArgs as it doesn't allow arguments for the debugee
     //so lookup the --debug switch and eat everything behind by decrementing argc
     //debugArgs is filled with args after --debug <debuger>
@@ -215,7 +306,7 @@ int main( int argc, char *argv[] )
                        "Example: kdevelop --debug gdb myapp --foo bar"));
 
     options.add("pid");
-           
+
     options.add("+files", ki18n( "Files to load" ));
 
     options.add(":", ki18n("Deprecated options:"));
@@ -241,7 +332,7 @@ int main( int argc, char *argv[] )
             }
             qout << si.uuid.toString() << '\t' << si.description;
 
-            if(!KDevelop::SessionController::tryLockSession(si.uuid.toString()))
+            if(KDevelop::SessionController::isSessionRunning(si.uuid.toString()))
                 qout << "     " << i18n("[running]");
 
             qout << endl;
@@ -249,16 +340,30 @@ int main( int argc, char *argv[] )
         return 0;
     }
 
+    // Handle extra arguments, which stand for files to open
+    QVector<File> initialFiles;
+    for ( int i = 0; i < args->count(); i++ ) {
+        initialFiles.append(parseFilename(args->arg(i)));
+    }
+    if ( ! initialFiles.isEmpty() && ! args->isSet("open-session") && ! args->isSet("new-session") )
+    {
+        int pid = getRunningSessionPid();
+        if ( pid > 0 ) {
+            return openFilesInRunningInstance(initialFiles, pid);
+        }
+        // else there are no running sessions, and the generated list of files will be opened below.
+    }
+
     // if empty, restart kdevelop with last active session, see SessionController::defaultSessionId
     QString session;
-    
+
     if(args->isSet("pss"))
     {
         QTextStream qerr(stderr);
         QList<KDevelop::SessionInfo> candidates;
         foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
             if( (!si.name.isEmpty() || !si.projects.isEmpty() || args->isSet("pid")) &&
-                (!args->isSet("pid") || !KDevelop::SessionController::tryLockSession(si.uuid.toString())))
+                (!args->isSet("pid") || KDevelop::SessionController::isSessionRunning(si.uuid.toString())))
                 candidates << si;
         
         if(candidates.size() == 0)
@@ -356,46 +461,25 @@ int main( int argc, char *argv[] )
             return 5;
         }
 
-        KDevelop::SessionController::LockSessionState state = KDevelop::SessionController::tryLockSession( sessionData->uuid.toString() );
-        if(state.success) {
+        KDevelop::SessionRunInfo sessionInfo = KDevelop::SessionController::sessionRunInfo( sessionData->uuid.toString() );
+        if (!sessionInfo.isRunning) {
             kError() << session << sessionData->name << "is not running";
             return 5;
         } else {
             // Print the PID and we're ready
-            std::cout << state.holderPid << std::endl;
+            std::cout << sessionInfo.holderPid << std::endl;
             return 0;
         }
     }
 
-    // If the session to load is known by this point,
-    // try to do the same iterative lock-check as in SessionController::loadDefaultSession().
-    KDevelop::SessionController::LockSessionState state;
-    if( sessionData ) {
-        do {
-            state = KDevelop::SessionController::tryLockSession( sessionData->uuid.toString() );
-            if( state ) {
-                break;
-            } else {
-                session = KDevelop::SessionController::handleLockedSession( sessionData->name, sessionData->uuid.toString(), state );
-                sessionData = findSessionInList( sessions, session );
-            }
-        } while( sessionData );
-        if( !state ) {
-            return 1;
-        }
-    }
-
     KDevIDEExtension::init();
 
-    KSplashScreen* splash = 0;
-    QString splashFile = KStandardDirs::locate( "appdata", "pics/kdevelop-splash.png" );
-    if( !splashFile.isEmpty() )
-    {
-        QPixmap pm;
-        pm.load( splashFile );
-        splash = new KSplashScreen( pm );
+    KDevSplashScreen* splash = 0;
+    if ( !QProcessEnvironment::systemEnvironment().contains("KDEV_DISABLE_SPLASH") ) {
+        splash = new KDevSplashScreen;
         splash->show();
         splash->repaint();
+        app.processEvents();
     }
 
     if(!Core::initialize(splash, Core::Default, session))
@@ -482,35 +566,11 @@ int main( int argc, char *argv[] )
 
         core->runControllerInternal()->execute("debug", launch);
     } else {
-        int count=args->count();
-        for(int i=0; i<count; ++i)
-        {
-            QString file=args->arg(i);
-            //Allow opening specific lines in documents, like mydoc.cpp:10
-            int lineNumberOffset = file.lastIndexOf(':');
-            KTextEditor::Cursor line;
-            if( lineNumberOffset != -1 )
-            {
-                bool isInt;
-                int lineNr = file.mid(lineNumberOffset+1).toInt(&isInt);
-                if (isInt)
-                {
-                    file = file.left(lineNumberOffset);
-                    line = KTextEditor::Cursor(lineNr, 0);
-                }
+        foreach ( const File& file, initialFiles ) {
+            if(!core->documentController()->openDocument(file.first, KTextEditor::Cursor(file.second, 0))) {
+                kWarning() << i18n("Could not open %1", file.first);
             }
-
-            KUrl f;
-            if( QFileInfo(file).isRelative() ) {
-                f = KUrl( QDir::currentPath(), file );
-            } else {
-                f = file;
-            }
-
-            if(!core->documentController()->openDocument(f, line))
-                kWarning() << i18n("Could not open %1", args->arg(i));
         }
-        args->clear();
     }
 
 #ifdef WITH_WELCOMEPAGE
