@@ -45,9 +45,23 @@ using namespace GDBDebugger;
 
 GDB::GDB(QObject* parent)
 : QObject(parent), process_(0), sawPrompt_(false), currentCmd_(0), receivedReply_(false), isRunning_(false)
+#if defined(Q_OS_MAC)
+, childPid_(0)
+#endif
 {
 }
 
+GDB::~GDB()
+{
+    // prevent Qt warning: QProcess: Destroyed while process is still running.
+    if (process_ && process_->state() == QProcess::Running) {
+        disconnect(process_, SIGNAL(error(QProcess::ProcessError)),
+                    this, SLOT(processErrored(QProcess::ProcessError)));
+        process_->kill();
+        process_->waitForFinished(10);
+    }
+}
+
 void GDB::start(KConfigGroup& config)
 {
     // FIXME: verify that default value leads to something sensible
@@ -74,12 +88,11 @@ void GDB::start(KConfigGroup& config)
     QStringList arguments;
     arguments << "--interpreter=mi2" << "-quiet";
 
-    QString shell = config.readEntry(GDBDebugger::debuggerShellEntry);
+    KUrl shell = config.readEntry(GDBDebugger::debuggerShellEntry, KUrl());
     if( !shell.isEmpty() )
     {
-        kDebug(9012) << "have shell\n";
-        shell = shell.simplified();
-        QString shell_without_args = shell.split(QChar(' ')).first();
+        kDebug(9012) << "have shell" << shell;
+        QString shell_without_args = shell.toLocalFile().split(QChar(' ')).first();
 
         QFileInfo info( shell_without_args );
         /*if( info.isRelative() )
@@ -98,7 +111,7 @@ void GDB::start(KConfigGroup& config)
         }
 
         arguments.insert(0, gdbBinary_);
-        arguments.insert(0, shell);
+        arguments.insert(0, shell.toLocalFile());
         process_->setShellCommand( KShell::joinArgs( arguments ) );
     }
     else
@@ -109,7 +122,7 @@ void GDB::start(KConfigGroup& config)
     process_->start();
 
     kDebug(9012) << "STARTING GDB\n";
-    emit userCommandOutput(shell + ' ' + gdbBinary_
+    emit userCommandOutput(shell.toLocalFile() + ' ' + gdbBinary_
                            + " --interpreter=mi2 -quiet\n" );
 }
 
@@ -124,8 +137,9 @@ void GDB::execute(GDBCommand* command)
     isRunning_ = false;
     receivedReply_ = false;
 
-    process_->write(commandText.toLatin1(),
-                    commandText.length());
+    QByteArray commandUtf8 = commandText.toUtf8();
+
+    process_->write(commandUtf8, commandUtf8.length());
 
     QString prettyCmd = currentCmd_->cmdToSend();
     prettyCmd.remove( QRegExp("set prompt \032.\n") );
@@ -146,8 +160,16 @@ void GDB::interrupt()
 {
     //TODO:win32 Porting needed
     int pid = process_->pid();
-    if (pid != 0)
+
+    #if defined(Q_OS_MAC)
+    if (childPid_) {
+        pid = childPid_;
+    }
+    #endif
+
+    if (pid) {
         ::kill(pid, SIGINT);
+    }
 }
 
 GDBCommand* GDB::currentCommand() const
@@ -197,9 +219,9 @@ void GDB::processLine(const QByteArray& line)
     FileSymbol file;
     file.contents = line;
 
-    std::auto_ptr<GDBMI::Record> r(mi_parser_.parse(&file));
+    std::unique_ptr<GDBMI::Record> r(mi_parser_.parse(&file));
 
-    if (r.get() == 0)
+    if (!r)
     {
         // FIXME: Issue an error!
         kDebug(9012) << "Invalid MI message:" << line;
@@ -239,7 +261,27 @@ void GDB::processLine(const QByteArray& line)
                GDBMI::ResultRecord& result = static_cast<GDBMI::ResultRecord&>(*r);
 
                emit internalCommandOutput(QString::fromUtf8(line) + '\n');
-               
+
+               #if defined(Q_OS_MAC)
+               if (result.reason == "thread-group-started") {
+                   if (currentCmd_->cmdToSend().contains("exec-run"))
+                       receivedReply_ = true;
+                   //     (gdb) -exec-run
+                   //     =thread-group-started,id="i1",pid="16768"
+                   if (line.contains("pid=\"")) {
+                       QList<QByteArray> splitLine = line.split(',');
+                       if (splitLine.size() > 2) {
+                           QByteArray pidStr = splitLine[2];
+                           pidStr.chop(1);
+                           childPid_ = pidStr.mid(5).toULong();
+                           if (childPid_ == 0) {
+                               kDebug() << "line=" << line << "pidStr=" << pidStr << pidStr.mid(5) << "pid=" << childPid_;
+                           }
+                       }
+                   }
+               }
+               #endif
+
                // FIXME: the code below should be reviewed to consider result record
                // subtype when doing all decisions.
                
