@@ -21,21 +21,19 @@ namespace GDBDebugger
 {
 
 GDBCommand::GDBCommand(GDBMI::CommandType type, const QString &command)
-: type_(type), command_(command), commandHandler_(0),
-  run(false), stateReloading_(false), m_thread(-1), m_frame(-1)
+: type_(type), command_(command), handler_method(0), commandHandler_(0),
+  run(false), stateReloading_(false), handlesError_(false), m_thread(-1), m_frame(-1)
 {
-    handlesError_ = false;
 }
 
 GDBCommand::GDBCommand(GDBMI::CommandType type, int index)
-: type_(type), command_(QString::number(index)), commandHandler_(0),
-  run(false), stateReloading_(false), m_thread(-1), m_frame(-1)
+: type_(type), command_(QString::number(index)), handler_method(0), commandHandler_(0),
+  run(false), stateReloading_(false), handlesError_(false), m_thread(-1), m_frame(-1)
 {
-    handlesError_ = false;
 }
 
 GDBCommand::GDBCommand(CommandType type, const QString& arguments, GDBCommandHandler* handler)
-: type_(type), command_(arguments), commandHandler_(handler),
+: type_(type), command_(arguments), handler_method(0), commandHandler_(handler),
   run(false), stateReloading_(false), m_thread(-1), m_frame(-1)
 {
     handlesError_ = handler->handlesError();
