@@ -44,7 +44,10 @@ private Q_SLOTS:
     void testQSetInt();
     void testQSetString();
     void testQChar();
+    void testQListPOD();
     void testQUuid();
+    void testKTextEditorTypes();
+    void testKDevelopTypes();
 };
 
 }
