@@ -42,8 +42,6 @@ class QtHelpPlugin : public KDevelop::IPlugin, public KDevelop::IDocumentationPr
         virtual QList<KDevelop::IDocumentationProvider*> providers();
         QList<QtHelpProvider*> qtHelpProviderLoaded();
         bool qtHelpQtDocLoaded();
-        void writeConfig(QStringList iconList, QStringList nameList, QStringList pathList, bool loadQtDoc);
-
 
     public slots:
         void readConfig();
@@ -51,6 +49,7 @@ class QtHelpPlugin : public KDevelop::IPlugin, public KDevelop::IDocumentationPr
         void changedProvidersList() const;
     private:
         void loadQtDocumentation(bool loadQtDoc);
+        void searchHelpDirectory(QStringList& pathList, QStringList& nameList, QStringList& iconList, const QString& searchDir);
         void loadQtHelpProvider(QStringList pathList, QStringList nameList, QStringList iconList);
 
         static QtHelpPlugin *s_plugin;
