@@ -1,10 +1,5 @@
 {% load kdev_filters %}
-{% block license_header %}
-/*
-
- {{ license|lines_prepend:" * " }}
- */
-{% endblock license_header %}
+{% include "license_header_cpp.txt" %}
 
 
 {% block includes %}
@@ -27,15 +22,13 @@
 
 {% block function_definitions %}
 {% for method in functions %}
-{% with method.arguments as arguments %}
-
-{% if method.type %}{{ method.type }} {% endif %}{{ name }}::{{ method.name }}({% include "arguments_types_names.txt" %})
+{% include "method_definition_cpp.txt" %}
 {
 
 
 }
 
-{% endwith %}
+
 {% endfor %}
 
 {% endblock function_definitions %}
