@@ -29,6 +29,7 @@
 #include <KProcess>
 #include <interfaces/icore.h>
 #include <interfaces/isourceformattercontroller.h>
+#include <interfaces/isourceformatter.h>
 #include <memory>
 #include <QDir>
 #include <util/formattinghelpers.h>
@@ -104,18 +105,12 @@ QString CustomScriptPlugin::description()
 				"can be easily shared by all team members, independent of their preferred IDE.");
 }
 
-QString CustomScriptPlugin::highlightModeForMime(const KMimeType::Ptr &mime)
-{
-	Q_UNUSED(mime);
-	return "C++";
-}
-
 QString CustomScriptPlugin::formatSourceWithStyle(SourceFormatterStyle style, const QString& text, const KUrl& url, const KMimeType::Ptr& /*mime*/, const QString& leftContext, const QString& rightContext)
 {
 	KProcess proc;
 	QTextStream ios(&proc);
 	
-	std::auto_ptr<QTemporaryFile> tmpFile;
+	std::unique_ptr<QTemporaryFile> tmpFile;
 
 	if (style.content().isEmpty())
 	{
@@ -142,7 +137,7 @@ QString CustomScriptPlugin::formatSourceWithStyle(SourceFormatterStyle style, co
 	
 	if(command.contains("$TMPFILE"))
 	{
-		tmpFile = std::auto_ptr<QTemporaryFile>(new QTemporaryFile(QDir::tempPath() + "/code"));
+		tmpFile.reset(new QTemporaryFile(QDir::tempPath() + "/code"));
 		tmpFile->setAutoRemove(false);
 		if(tmpFile->open())
 		{
@@ -220,8 +215,27 @@ QString CustomScriptPlugin::formatSource(const QString& text, const KUrl& url, c
 	return formatSourceWithStyle( KDevelop::ICore::self()->sourceFormatterController()->styleForMimeType( mime ), text, url, mime, leftContext, rightContext );
 }
 
+static QList<SourceFormatterStyle> stylesFromLanguagePlugins() {
+	QList<KDevelop::SourceFormatterStyle> styles;
+	for ( ILanguage* lang: ICore::self()->languageController()->loadedLanguages() ) {
+		SourceFormatterItemList languageStyles = lang->languageSupport()->sourceFormatterItems();
+		for ( const SourceFormatterStyleItem& item: languageStyles ) {
+			if ( item.engine == "customscript" ) {
+				styles << item.style;
+			}
+		}
+	}
+	return styles;
+};
+
 KDevelop::SourceFormatterStyle CustomScriptPlugin::predefinedStyle(const QString& name)
 {
+	for ( auto langStyle: stylesFromLanguagePlugins() ) {
+		qDebug() << "looking at style from language with custom sample" << langStyle.description() << langStyle.overrideSample();
+		if ( langStyle.name() == name ) {
+			return langStyle;
+		}
+	}
 	SourceFormatterStyle result(name);
 	if (name == "GNU_indent_GNU")
 	{
@@ -233,12 +247,12 @@ KDevelop::SourceFormatterStyle CustomScriptPlugin::predefinedStyle(const QString
 	} else if (name == "GNU_indent_orig") {
 		result.setCaption(i18n("Gnu Indent: Original Berkeley indent style"));
 		result.setContent("indent -orig");
-	} else if(name == "kdev_format_source.sh") {
-		result.setCaption("KDevelop: kdev_format_source.sh");
-		result.setContent("kdev_format_source.sh $FILE $TMPFILE");
+	} else if(name == "kdev_format_source") {
+		result.setCaption("KDevelop: kdev_format_source");
+		result.setContent("kdev_format_source $FILE $TMPFILE");
 		result.setUsePreview(false);
 		result.setDescription(i18n( "Description:<br />"
-									"<b>kdev_format_source.sh</b> is a script bundled with KDevelop "
+									"<b>kdev_format_source</b> is a script bundled with KDevelop "
 			                        "which allows using fine-grained formatting rules by placing "   
 									"meta-files called <b>format_sources</b> into the file-system.<br /><br />"
 									"Each line of the <b>format_sources</b> files defines a list of wildcards "
@@ -246,21 +260,26 @@ KDevelop::SourceFormatterStyle CustomScriptPlugin::predefinedStyle(const QString
 									"The formatting-command should use <b>$TMPFILE</b> to reference the "
 									"temporary file to reformat.<br /><br />"
 									"Example:<br />"
-									"<b>*.cpp *.h : myformatter.sh $TMPFILE</b><br />"
+									"<b>*.cpp *.h : myformatter $TMPFILE</b><br />"
 									"This will reformat all files ending with <b>.cpp</b> or <b>.h</b> using "
-									"the custom formatting script <b>myformatter.sh</b>.<br /><br />"
+									"the custom formatting script <b>myformatter</b>.<br /><br />"
 									"Example: <br />"
 									"<b>subdir/* : uncrustify -l CPP -f $TMPFILE -c uncrustify.config -o $TMPFILE</b> <br />"
 									"This will reformat all files in subdirectory <b>subdir</b> using the <b>uncrustify</b> "
 									"tool with the config-file <b>uncrustify.config</b>." ));
 	}
+	result.setMimeTypes({
+		{"text/x-c++src", "C++"}, {"text/x-chdr", "C"},
+		{"text/x-c++hdr", "C++"}, {"text/x-csrc", "C"},
+		{"text/x-java", "Java"}, {"text/x-csharp", "C#"}
+	});
 	return result;
 }
 
 QList<KDevelop::SourceFormatterStyle> CustomScriptPlugin::predefinedStyles()
 {
-    QList<KDevelop::SourceFormatterStyle> styles;
-	styles << predefinedStyle("kdev_format_source.sh");
+	QList<KDevelop::SourceFormatterStyle> styles = stylesFromLanguagePlugins();
+	styles << predefinedStyle("kdev_format_source");
 	styles << predefinedStyle("GNU_indent_GNU");
 	styles << predefinedStyle("GNU_indent_KR");
 	styles << predefinedStyle("GNU_indent_orig");
@@ -363,8 +382,11 @@ static QString indentingSample()
     "\tbarArg3);\n";
 }
 
-QString CustomScriptPlugin::previewText(const KMimeType::Ptr &)
+QString CustomScriptPlugin::previewText(const SourceFormatterStyle& style, const KMimeType::Ptr& mime)
 {
+	if ( ! style.overrideSample().isEmpty() ) {
+		return style.overrideSample();
+	}
 	return formattingSample() + "\n\n" + indentingSample();
 }
 
@@ -459,7 +481,8 @@ CustomScriptPlugin::Indentation CustomScriptPlugin::indentation( const KUrl& url
 
 void CustomScriptPreferences::updateTimeout()
 {
-    QString formatted = indentPluginSingleton.data()->formatSourceWithStyle ( m_style, indentPluginSingleton.data()->previewText ( KMimeType::Ptr() ), KUrl(), KMimeType::Ptr() );
+	const QString& text = indentPluginSingleton.data()->previewText ( m_style, KMimeType::Ptr() );
+    QString formatted = indentPluginSingleton.data()->formatSourceWithStyle ( m_style, text, KUrl(), KMimeType::Ptr() );
     emit previewTextChanged ( formatted );
 }
 
