@@ -19,13 +19,9 @@
 #ifndef CODECOMPLETIONCONTEXT_H
 #define CODECOMPLETIONCONTEXT_H
 
-#include <ktexteditor/cursor.h>
 #include <ksharedptr.h>
 #include <language/duchain/duchainpointer.h>
 #include "../cppduchain/typeconversion.h"
-#include "../cppduchain/expressionparser.h"
-#include "../cppduchain/viablefunctions.h"
-#include "../cppduchain/overloadresolutionhelper.h"
 #include "../cppduchain/expressionevaluationresult.h"
 #include <language/util/includeitem.h>
 #include "item.h"
@@ -42,6 +38,7 @@ namespace KDevelop {
 
   class CompletionTreeItem;
   typedef KSharedPtr<CompletionTreeItem> CompletionTreeItemPointer;
+  typedef QPair<Declaration*, bool> DeclAccessPair;
 }
 
 namespace Cpp {
@@ -52,6 +49,8 @@ namespace Cpp {
     int expressionBefore( const QString& _text, int index );
   #endif
 
+  typedef QPair<Declaration*, int> DeclarationDepthPair;
+
   /**
    * This class is responsible for finding out what kind of completion is needed, what expression should be evaluated for the container-class of the completion, what conversion will be applied to the result of the completion, etc.
    * */
@@ -88,6 +87,7 @@ namespace Cpp {
         TemplateAccess,       /// "bla<."
         ReturnAccess,         /// "return " -- Takes into account return type
         CaseAccess,           /// "case " -- Takes into account switch expression type
+        NamespaceAccess
       };
 
       /**
@@ -108,6 +108,8 @@ namespace Cpp {
 
       ///@return the used access-operation
       AccessType accessType() const;
+      ///@return the list of IndexedTypes that this CodeCompletionContext expects
+      QList<IndexedType> matchTypes();
 
       /**
        * When the access-operation is a MemberAccess or ArrowMemberAccess, this
@@ -153,8 +155,7 @@ namespace Cpp {
       * */
       QList<KDevelop::IncludeItem> includeItems() const;
 
-      ///*DUChain must be locked*
-      KDevelop::IndexedType applyPointerConversionForMatching(KDevelop::IndexedType type, bool fromLValue) const;
+      int pointerConversions() const;
       
       QString followingText() const;
       
@@ -173,6 +174,7 @@ namespace Cpp {
 #endif
       enum OnlyShow {
         ShowAll,
+        ShowIntegralConstants, // compile-time integral constants and related items (scopes, ...)
         ShowTypes,
         ShowSignals,
         ShowSlots,
@@ -241,7 +243,7 @@ namespace Cpp {
       QList<CompletionTreeItemPointer> templateAccessCompletionItems();
       QList<CompletionTreeItemPointer> functionAccessCompletionItems(bool fullCompletion);
       QList<CompletionTreeItemPointer> binaryFunctionAccessCompletionItems(bool fullCompletion);
-      ///*DUChain must be locked*
+      ///*DUChain must _not_ be locked*
       QList<CompletionTreeItemPointer> commonFunctionAccessCompletionItems(bool fullCompletion);
       QList<CompletionTreeItemPointer> includeListAccessCompletionItems(const bool& shouldAbort);
       QList<CompletionTreeItemPointer> signalSlotAccessCompletionItems();
@@ -250,23 +252,38 @@ namespace Cpp {
       QList<CompletionTreeItemPointer> getImplementationHelpers();
       QList<CompletionTreeItemPointer> getImplementationHelpersInternal(const QualifiedIdentifier& minimumScope, DUContext* context);
 
+      ///If @param forDecl is an instance of a class, find declarations in that class which match @param matchTypes
+      ///@returns the list of matching declarations and whether or not you need the arrow operator (->) to access them
+      QList<DeclAccessPair> getLookaheadMatches(Declaration* forDecl, const QList<IndexedType>& matchTypes) const;
+      void addLookaheadMatches(const QList<CompletionTreeItemPointer> items);
+      ///For a given @param container, find members which may potentially be used for lookahead matching
+      ///@param isPointer specifies whether the container should be accessed with operator->
+      ///@returns a list of declarations paired with whether or not they use "operator->"
+      ///Note that a non-pointer container may declare an operator-> (ie, smart pointer)
+      QList<DeclAccessPair> containedDeclarationsForLookahead(Declaration* decl, TopDUContext* top, bool isPointer) const;
+
       ///*DUChain must be locked*
-      bool  filterDeclaration(Declaration* decl, DUContext* declarationContext = 0, bool dynamic = true);
+      bool  filterDeclaration(Declaration* decl, DUContext* declarationContext = 0, bool dynamic = true) const;
       ///*DUChain must be locked*
-      bool  filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext = 0);
-      ///Replaces the member-access type at the current cursor position from "from" to "new", for example from "->" to "."
-      ///*DUChain must be locked*
-      void replaceCurrentAccess(QString old, QString _new);
+      bool  filterDeclaration(ClassMemberDeclaration* decl, DUContext* declarationContext = 0) const;
+      /// Request a replace of the member-access type at the current cursor position from "oldAccess"
+      /// to "newAccess", for example from "->" to "."
+      /// The actual replacement is delayed into the foreground thread.
+      void replaceCurrentAccess(const QString& oldAccess, const QString& newAccess);
 
       ///Creates the group and adds it to m_storedUngroupedItems if items is not empty
       void eventuallyAddGroup(QString name, int priority, QList< KSharedPtr< KDevelop::CompletionTreeItem > > items);
       
       ///@param type The type of the argument the items are matched to.
       ///*DUChain must be locked*
-      QList<CompletionTreeItemPointer> specialItemsForArgumentType(AbstractType::Ptr type);
+      void addSpecialItemsForArgumentType(AbstractType::Ptr type);
       
       ///Returns whether the declaration is directly visible from within the current context
-      bool visibleFromWithin(Declaration* decl, DUContext* currentContext);
+      bool visibleFromWithin(Declaration* decl, DUContext* currentContext) const;
+
+      ///Returns whether the declaration can be considered an integral constant
+      ///@param acceptHelperItems whether we check for filtering (true) or for marking match quality (false)
+      bool isIntegralConstant(Declaration* decl, bool acceptHelperItems) const;
       
       ///Returns whether the end of m_text is a valid completion-position
       bool isValidPosition();
@@ -293,6 +310,9 @@ namespace Cpp {
       ///Handle code-completion for constructor-initializers, \returns true if valid
       bool doConstructorCompletion();
 
+      QList<DeclarationDepthPair> namespaceItems(KDevelop::DUContext* duContext, const KDevelop::CursorInRevision& position, bool global,
+                                                 const QSet<KDevelop::DUContext*>& skipContexts = QSet<KDevelop::DUContext*>()) const;
+
       AccessType m_accessType;
       QString m_expression;
       QString m_followingText;
@@ -328,6 +348,12 @@ namespace Cpp {
       bool m_doAccessFiltering;
       DUContextPointer m_localClass;
 
+      QList<IndexedType> m_cachedMatchTypes;
+      /// Maps container declaration to cached return value of getLookaheadMatches
+      /// This is useful, as otherwise we'd repeat the same stuff for every variable
+      /// with the same type accessible from the current scope
+      mutable QHash<Declaration*, QList<DeclAccessPair> > m_lookaheadMatchesCache;
+
       friend class ImplementationHelperItem;
   };
 }
