@@ -35,7 +35,6 @@
 #include <language/duchain/duchainutils.h>
 #include <language/duchain/classdeclaration.h>
 #include "../cppduchain/qtfunctiondeclaration.h"
-#include <language/duchain/use.h>
 #include <typeutils.h>
 #include <cppduchain.h>
 #include <templatedeclaration.h>
@@ -54,13 +53,6 @@ QString linePrefix( KTextEditor::Document* document, const KTextEditor::Range& w
     return document->text( linePrefixRange );
 }
 
-QString lineSuffix( KTextEditor::Document* document, const KTextEditor::Range& word )
-{
-    int endLineLength = document->lineLength( word.end().line() );
-    KTextEditor::Range lineSuffixRange( word.end().line(), word.end().column(), word.end().line(), endLineLength );
-    return document->text( lineSuffixRange );
-}
-
 QString lineSuffixAndWord( KTextEditor::Document* document, const KTextEditor::Range& word )
 {
     int endLineLength = document->lineLength( word.end().line() );
@@ -85,6 +77,60 @@ bool declarationNeedsTemplateParameters(const Declaration* decl) {
 
 namespace Cpp {
 
+AbstractType::Ptr applyPointerConversions(AbstractType::Ptr type, int pointerConversions, TopDUContext *top)
+{
+  if(pointerConversions == 0 || type.isNull())
+    return type;
+
+  if(pointerConversions > 0) {
+    for(int a = 0; a < pointerConversions && type; ++a) {
+      type = TypeUtils::increasePointerDepth(type);
+    }
+  }else{
+    for(int a = pointerConversions; a < 0 && type; ++a) {
+      type = TypeUtils::decreasePointerDepth(type, top, true);
+    }
+  }
+
+  return type;
+}
+
+int getMatchQuality(CodeCompletionContext *context, const Declaration* decl, TopDUContext* top)
+{
+  if (!context->parentContext() || decl->kind() == Declaration::Type
+      || decl->kind() == Declaration::Namespace || decl->kind() == Declaration::NamespaceAlias)
+  {
+    return 0;
+  }
+
+  const QList<IndexedType>& matchTypes = context->parentContext()->matchTypes();
+  if (matchTypes.isEmpty())
+    return 0;
+
+  int pointerConversions = context->pointerConversions();
+  if (pointerConversions > 1)
+    return 0; //Can't do "&&foo"
+
+  const IndexedType& effectiveDeclType = applyPointerConversions(effectiveType(decl), pointerConversions, top)->indexed();
+
+  bool fromLValue = (bool)decl->type<ReferenceType>() ||
+                    (!dynamic_cast<const AbstractFunctionDeclaration*>(decl) &&
+                      decl->kind() == Declaration::Instance);
+  if(pointerConversions > 0 && !fromLValue)
+    return 0;
+
+  Cpp::TypeConversion conv(top);
+
+  int bestQuality = 0;
+  foreach(const IndexedType& type, matchTypes) {
+    int quality = conv.implicitConversion( effectiveDeclType, type, fromLValue )  * 10 / Cpp::MaximumConversionResult;
+    if(quality > bestQuality)
+      bestQuality = quality;
+  }
+
+  return bestQuality;
+}
+
 void executeSignalSlotCompletionItem( KTextEditor::Document* document, const KTextEditor::Range& enteredWord,
                                       bool isSignal, const QString& name, const QString& signature )
 {
@@ -212,15 +258,14 @@ void NormalDeclarationCompletionItem::execute(KTextEditor::Document* document, c
   newText.prepend(prefixText);
   
   // Text that will be removed in a separate editing step (so the user can undo it)
-  std::auto_ptr<KTextEditor::MovingRange> removeInSecondStep;
+  QScopedPointer<KTextEditor::MovingRange> removeInSecondStep;
   
   KTextEditor::Cursor cursor = document->activeView()->cursorPosition();
   if(cursor != word.end())
   {
     KTextEditor::MovingInterface* moving = dynamic_cast<KTextEditor::MovingInterface*>(document);
     Q_ASSERT(moving);
-    removeInSecondStep =  std::auto_ptr<KTextEditor::MovingRange>(
-      moving->newMovingRange(KTextEditor::Range(cursor, word.end()), KTextEditor::MovingRange::DoNotExpand));
+    removeInSecondStep.reset(moving->newMovingRange(KTextEditor::Range(cursor, word.end()), KTextEditor::MovingRange::DoNotExpand));
     word.end() = cursor;
   }
   
@@ -268,26 +313,26 @@ void NormalDeclarationCompletionItem::execute(KTextEditor::Document* document, c
       //Do some intelligent stuff for functions with the parens:
       lock.unlock();
       KTextEditor::Cursor insertionPosition = end;
-      if (removeInSecondStep.get()) {
+      if (removeInSecondStep) {
         KTextEditor::Range removeRange = removeInSecondStep->toRange();
         insertionPosition += removeRange.end() - removeRange.start();
       }
       insertFunctionParenText(document, insertionPosition, m_declaration, jumpForbidden);
     }
     
-    if (removeInSecondStep.get()) {
+    if (removeInSecondStep) {
       //if we would remove text after the inserted text, skip that if it is a property
       //of the executed item and additionally insert a . or ->
       Identifier id(document->text(removeInSecondStep->toRange()));
       QString insertAccessor = keepRemainingWord(id);
       if (!insertAccessor.isEmpty()) {
         document->insertText(removeInSecondStep->toRange().start(), insertAccessor);
-        removeInSecondStep.release();
+        removeInSecondStep.reset(nullptr);
       }
     }
   }
   
-  if(removeInSecondStep.get())
+  if(removeInSecondStep)
   {
     KTextEditor::Range removeRange = removeInSecondStep->toRange();
     if(!removeRange.isEmpty() && removeRange.end() > end && removeRange.end().line() == end.line() && removeRange.end().column() <= document->lineLength(removeRange.end().line()))
@@ -303,7 +348,7 @@ void NormalDeclarationCompletionItem::execute(KTextEditor::Document* document, c
   }
 }
 
-QString NormalDeclarationCompletionItem::keepRemainingWord(Identifier id)
+QString NormalDeclarationCompletionItem::keepRemainingWord(const Identifier& id)
 {
   DUChainReadLocker lock;
 
@@ -459,19 +504,6 @@ KDevelop::QualifiedIdentifier NormalDeclarationCompletionItem::stripPrefix() con
   return QualifiedIdentifier();
 }
 
-QList<KDevelop::IndexedType> NormalDeclarationCompletionItem::typeForArgumentMatching() const {
-  QList<KDevelop::IndexedType> ret;
-  if (m_declaration && completionContext())
-    ret = completionContext()->matchTypes();
-  return ret;
-}
-
-QList<IndexedType> currentMatchContext;
-
-void setStaticMatchContext(QList< KDevelop::IndexedType > types) {
-  currentMatchContext = types;
-}
-
 QVariant NormalDeclarationCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const {
 
   DUChainReadLocker lock(DUChain::lock(), 500);
@@ -485,13 +517,6 @@ QVariant NormalDeclarationCompletionItem::data(const QModelIndex& index, int rol
     return QVariant();
   }
 
-  //Stuff that does not require a declaration:
-  switch (role) {
-    case CodeCompletionModel::SetMatchContext:
-      currentMatchContext = typeForArgumentMatching();
-      return QVariant(1);
-  };
-
   if(!m_declaration) {
     if(role == Qt::DisplayRole && index.column() == CodeCompletionModel::Name)
       return alternativeText;
@@ -509,30 +534,11 @@ QVariant NormalDeclarationCompletionItem::data(const QModelIndex& index, int rol
 
   switch (role) {
     case CodeCompletionModel::MatchQuality:
-    {
-      if(m_fixedMatchQuality != -1)
-        return QVariant(m_fixedMatchQuality);
-      
-      if( currentMatchContext.size()) {
-        
-        int bestQuality = 0;
-        foreach(const IndexedType& type, currentMatchContext) {
-          Cpp::TypeConversion conv(model->currentTopContext().data());
-  
-          AbstractType::Ptr ownType = effectiveType(dec);
-          
-          bool fromLValue = (bool)ownType.cast<ReferenceType>() || (!dynamic_cast<AbstractFunctionDeclaration*>(dec) && dec->kind() == Declaration::Instance);
-          
-          ///@todo fill the lvalue-ness correctly
-          int q = ( conv.implicitConversion( completionContext()->applyPointerConversionForMatching(ownType->indexed(), fromLValue), type, fromLValue )  * 10 ) / Cpp::MaximumConversionResult;
-          if(q > bestQuality)
-            bestQuality = q;
-        }
-        
-        return QVariant(bestQuality);
+      if(m_fixedMatchQuality == -1) {
+        m_fixedMatchQuality = getMatchQuality(completionContext().data(), dec, model->currentTopContext().data());
       }
-    }
-    return QVariant();
+      return m_fixedMatchQuality;
+
     case CodeCompletionModel::ItemSelected:
        return QVariant(Cpp::NavigationWidget::shortDescription(dec));
     case Qt::DisplayRole:
@@ -791,10 +797,6 @@ void TypeConversionCompletionItem::setPrefix(QString s) {
   m_prefix = s;
 }
 
-QList<KDevelop::IndexedType> TypeConversionCompletionItem::typeForArgumentMatching() const {
-  return QList<KDevelop::IndexedType>() << m_type;
-}
-
 QList<KDevelop::IndexedType> TypeConversionCompletionItem::type() const {
   return QList<KDevelop::IndexedType>() << m_type;
 }
@@ -804,12 +806,9 @@ void TypeConversionCompletionItem::execute(KTextEditor::Document* document, cons
     document->replaceText( word, m_text );
 }
 
-QVariant TypeConversionCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const {
-
+QVariant TypeConversionCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* /*model*/) const
+{
   switch (role) {
-    case CodeCompletionModel::SetMatchContext:
-      currentMatchContext = typeForArgumentMatching();
-      return QVariant(1);
     case Qt::DisplayRole:
       switch (index.column()) {
         case CodeCompletionModel::Prefix:
@@ -821,32 +820,6 @@ QVariant TypeConversionCompletionItem::data(const QModelIndex& index, int role,
       break;
     case CodeCompletionModel::ItemSelected:
       return QVariant();
-    case CodeCompletionModel::MatchQuality:
-    {
-      DUChainReadLocker lock(DUChain::lock(), 500);
-      if(!lock.locked()) {
-        kDebug(9007) << "Failed to lock the du-chain in time";
-        return QVariant();
-      }
-      
-      if( currentMatchContext.size() ) {
-        int bestQuality = 0;
-        
-        foreach(const IndexedType& type, currentMatchContext) {
-          Cpp::TypeConversion conv(model->currentTopContext().data());
-
-          ///@todo Think about lvalue-ness
-          foreach(const IndexedType& ownType, typeForArgumentMatching()) {
-            int quality = ( conv.implicitConversion( completionContext->applyPointerConversionForMatching(ownType, false), type, false )  * 10 ) / Cpp::MaximumConversionResult;
-            if(quality > bestQuality)
-              bestQuality = quality;
-          }
-        }
-        
-        return QVariant(bestQuality);
-      }
-    }
-    return QVariant();
   }
 
   return QVariant();
