@@ -22,50 +22,30 @@
 
 #include "model.h"
 
-#include <QIcon>
-#include <QMetaType>
-#include <QTextFormat>
-#include <QBrush>
-#include <QDir>
 #include <kdebug.h>
+
 #include <ktexteditor/view.h>
 #include <ktexteditor/document.h>
-#include <kiconloader.h>
+#include <ktexteditor/codecompletioninterface.h>
 
-
-#include "../cppduchain/cppduchain.h"
-#include "../cppduchain/typeutils.h"
-
-#include "../cppduchain/overloadresolutionhelper.h"
-
-#include <language/duchain/declaration.h>
-#include "../cppduchain/cpptypes.h"
-#include "../cppduchain/typeutils.h"
-#include <language/duchain/classfunctiondeclaration.h>
-#include <language/duchain/ducontext.h>
-#include <language/duchain/duchain.h>
-#include <language/duchain/namespacealiasdeclaration.h>
-#include <language/duchain/parsingenvironment.h>
-#include <language/duchain/duchainlock.h>
-#include <language/duchain/duchainbase.h>
-#include <language/duchain/topducontext.h>
-#include <language/duchain/dumpchain.h>
 #include <language/codecompletion/codecompletioncontext.h>
-#include "../cppduchain/navigation/navigationwidget.h"
-#include "../preprocessjob.h"
-#include <language/duchain/duchainutils.h>
+#include <language/backgroundparser/backgroundparser.h>
+
+#include <interfaces/icore.h>
+#include <interfaces/ilanguagecontroller.h>
+#include <interfaces/idocumentcontroller.h>
+
 #include "worker.h"
-#include "../cpplanguagesupport.h"
-#include <language/editor/modificationrevision.h>
-#include <language/duchain/specializationstore.h>
-#include "implementationhelperitem.h"
+#include "context.h"
+#include "../cppduchain/typeconversion.h"
 
 using namespace KTextEditor;
 using namespace KDevelop;
-using namespace TypeUtils;
 
 namespace Cpp {
 
+CodeCompletionModel* CodeCompletionModel::s_self = nullptr;
+
 bool useArgumentHintInAutomaticCompletion() {
   return false;
 }
@@ -74,6 +54,51 @@ CodeCompletionModel::CodeCompletionModel( QObject * parent )
   : KDevelop::CodeCompletionModel(parent)
 {
   setForceWaitForModel(true);
+  Q_ASSERT(!s_self);
+  s_self = this;
+
+  connect(ICore::self()->languageController()->backgroundParser(), SIGNAL(parseJobFinished(KDevelop::ParseJob*)),
+          this, SLOT(parseJobFinished(KDevelop::ParseJob*)));
+}
+
+CodeCompletionModel::~CodeCompletionModel()
+{
+  s_self = nullptr;
+}
+
+CodeCompletionModel* CodeCompletionModel::self()
+{
+  return s_self;
+}
+
+void CodeCompletionModel::startCompletionAfterParsing(const IndexedString& path)
+{
+  m_awaitDocument = path;
+}
+
+void CodeCompletionModel::parseJobFinished(ParseJob* job)
+{
+  if (job->document() != m_awaitDocument || ICore::self()->languageController()->backgroundParser()->isQueued(m_awaitDocument)) {
+    return;
+  }
+
+  IDocument* doc = ICore::self()->documentController()->documentForUrl(m_awaitDocument.toUrl());
+  m_awaitDocument = {};
+
+  if (!doc || !doc->textDocument()) {
+    return;
+  }
+
+  auto view = doc->textDocument()->activeView();
+  if (!view || !view->hasFocus()) {
+    return;
+  }
+
+  auto iface = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view);
+  if (iface) {
+    ///@todo 1. This is a non-public interface, and 2. Completion should be started in "automatic invocation" mode
+    QMetaObject::invokeMethod(view, "userInvokedCompletion");
+  }
 }
 
 bool CodeCompletionModel::shouldStartCompletion(KTextEditor::View* view, const QString& inserted, bool userInsertion, const KTextEditor::Cursor& position) {
@@ -138,10 +163,6 @@ KDevelop::CodeCompletionWorker* CodeCompletionModel::createCompletionWorker() {
   return new CodeCompletionWorker(this);
 }
 
-CodeCompletionModel::~CodeCompletionModel()
-{
-}
-
 Range CodeCompletionModel::updateCompletionRange(View* view, const KTextEditor::Range& range) {
   if(completionContext()) {
     Cpp::CodeCompletionContext* cppContext = dynamic_cast<Cpp::CodeCompletionContext*>(completionContext().data());
@@ -220,22 +241,9 @@ Range CodeCompletionModel::completionRange(View* view, const KTextEditor::Cursor
     return range;
 }
 
-void CodeCompletionModel::foundDeclarations(QList<KSharedPtr<KDevelop::CompletionTreeElement> > item, KSharedPtr<KDevelop::CodeCompletionContext> completionContext) {
-  //Set the static match-context, in case the argument-hints are not shown
-  
-  setStaticMatchContext(QList<IndexedType>());
-  
-  if(completionContext) {
-    Cpp::CodeCompletionContext* argumentFunctions = dynamic_cast<Cpp::CodeCompletionContext*>(completionContext->parentContext());
-    if(argumentFunctions) {
-      QList<IndexedType> types;
-      bool abort = false;
-      foreach(CompletionTreeItemPointer item, argumentFunctions->completionItems(abort, false))
-        types += item->typeForArgumentMatching();
-      
-      setStaticMatchContext(types);
-    }
-  }
+void CodeCompletionModel::foundDeclarations(QList<CompletionTreeElementPointer> item, KDevelop::CodeCompletionContext::Ptr completionContext)
+{
+  TypeConversion::startCache();
   KDevelop::CodeCompletionModel::foundDeclarations(item, completionContext);
 }
 
