@@ -23,28 +23,21 @@
 #ifndef KDEVCPPCODECOMPLETIONMODEL_H
 #define KDEVCPPCODECOMPLETIONMODEL_H
 
-#include <QPair>
-#include <QMap>
 #include <language/codecompletion/codecompletionmodel.h>
 #include <ksharedptr.h>
 #include <language/duchain/duchainpointer.h>
+#include <language/duchain/indexedstring.h>
 #include <language/codecompletion/codecompletioncontext.h>
-#include "item.h"
-#include <ktexteditor/codecompletionmodelcontrollerinterface.h>
-#include <kdeversion.h>
-
-class QIcon;
-class QString;
-class QMutex;
 
 namespace KDevelop {
   class DUContext;
   class Declaration;
+  class CodeCompletionContext;
   class CompletionTreeElement;
+  class ParseJob;
 }
 
 namespace Cpp {
-  class CodeCompletionContext;
   class NavigationWidget;
   class CodeCompletionWorker;
 
@@ -58,9 +51,11 @@ class CodeCompletionModel : public KDevelop::CodeCompletionModel
     CodeCompletionModel(QObject* parent);
     virtual ~CodeCompletionModel();
 
+    static CodeCompletionModel* self();
+
+    void startCompletionAfterParsing(const KDevelop::IndexedString& path);
+
   protected:
-    virtual void foundDeclarations(QList<KSharedPtr<KDevelop::CompletionTreeElement> > item, KSharedPtr<KDevelop::CodeCompletionContext> completionContext);
-    
     virtual KTextEditor::Range completionRange(KTextEditor::View* view, const KTextEditor::Cursor& position);
     virtual KTextEditor::Range updateCompletionRange(KTextEditor::View* view, const KTextEditor::Range& range);
     virtual void aborted(KTextEditor::View* view);
@@ -68,8 +63,16 @@ class CodeCompletionModel : public KDevelop::CodeCompletionModel
     virtual bool shouldStartCompletion (KTextEditor::View*, const QString&, bool userInsertion, const KTextEditor::Cursor&);
     virtual KDevelop::CodeCompletionWorker* createCompletionWorker();
 
+    virtual void foundDeclarations(QList< KSharedPtr< KDevelop::CompletionTreeElement > > item, KSharedPtr< KDevelop::CodeCompletionContext > completionContext);
+
+  private slots:
+    void parseJobFinished(KDevelop::ParseJob* job);
+
   private:
-    KSharedPtr<Cpp::CodeCompletionContext> m_completionContext;
+    static CodeCompletionModel* s_self;
+
+    // when a document with this path has finished its parse job, code-completion will be restarted
+    KDevelop::IndexedString m_awaitDocument;
 };
 
 }
