@@ -21,9 +21,14 @@
 
 #include <interfaces/iassistant.h>
 #include <language/duchain/indexedstring.h>
+#include <language/duchain/problem.h>
 #include <kurl.h>
 
 
+namespace KDevelop {
+  class IProject;
+}
+
 namespace Cpp {
 
 class AddCustomIncludePathAction : public KDevelop::IAssistantAction {
@@ -39,6 +44,17 @@ private:
   QString m_directive;
 };
 
+class OpenProjectConfigurationAction : public KDevelop::IAssistantAction
+{
+public:
+  OpenProjectConfigurationAction( KDevelop::IProject* project );
+  virtual QString description() const;
+  virtual void execute();
+
+private:
+  KDevelop::IProject* m_project;
+};
+
 class OpenProjectForFileAssistant : public KDevelop::IAssistantAction {
 public:
   OpenProjectForFileAssistant(KUrl);
@@ -53,12 +69,37 @@ class MissingIncludePathAssistant : public KDevelop::IAssistant {
 public:
   MissingIncludePathAssistant(KDevelop::IndexedString url, QString directive);
   virtual QString title() const;
-  virtual QList< KDevelop::IAssistantAction::Ptr > actions() const;
+  virtual void createActions();
 private:
   KDevelop::IndexedString m_url;
   QString m_directive;
 };
 
+class MissingIncludePathProblem : public KDevelop::Problem
+{
+public:
+    using Ptr = KSharedPtr<MissingIncludePathProblem>;
+
+    MissingIncludePathProblem() = default;
+    MissingIncludePathProblem(KDevelop::ProblemData& data)
+      : KDevelop::Problem(data)
+    {}
+
+    virtual KSharedPtr<KDevelop::IAssistant> solutionAssistant() const
+    {
+      return m_solution;
+    }
+
+    void setSolutionAssistant(const KSharedPtr<KDevelop::IAssistant>& assistant)
+    {
+      m_solution = assistant;
+    }
+
+private:
+    /// FIXME: persist the data for the MissingIncludePathAssistant!
+    KSharedPtr<KDevelop::IAssistant> m_solution;
+};
+
 }
 
 #endif // CPP_MISSINGINCLUDEASSISTANT_H
