@@ -33,13 +33,12 @@
 #include "name_compiler.h"
 #include <language/duchain/dumpchain.h>
 #include "environmentmanager.h"
+#include "expressionvisitor.h"
 
-#include <climits>
 #include "cppdebughelper.h"
 #include "debugbuilders.h"
 #include "rpp/chartools.h"
 #include "tokens.h"
-#include <klocalizedstring.h>
 
 using namespace KTextEditor;
 using namespace KDevelop;
@@ -103,6 +102,7 @@ ContextBuilder::ContextBuilder (ParseSession* session)
   , m_onlyComputeSimplified(false)
   , m_computeEmpty(false)
   , m_currentInitializer(0)
+  , m_currentCondition(0)
   , m_mapAst(false)
 {
 }
@@ -483,7 +483,7 @@ void ContextBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   //In case of an "enum class" this enum creates its own context.
   openContext(node, DUContext::Enum, node->isClass ? node->name : 0 );
 
-  {
+  if (!node->isClass) {
     DUChainWriteLocker lock(DUChain::lock());
     currentContext()->setPropagateDeclarations(true);
   }
@@ -546,7 +546,7 @@ void ContextBuilder::visitTypedef (TypedefAST *node)
 
 void ContextBuilder::visitFunctionDefinition (FunctionDefinitionAST *node)
 {
-  PushValue<bool> push(m_inFunctionDefinition, (bool)node->function_body);
+  PushValue<bool> push(m_inFunctionDefinition, (bool)node->function_body || node->defaultDeleted != FunctionDefinitionAST::NotDefaultOrDeleted);
 
   QualifiedIdentifier functionName;
   if (compilingContexts() && node->declarator && node->declarator->id) {
@@ -872,17 +872,17 @@ void ContextBuilder::handleRangeBasedFor(ExpressionAST* container, ForRangeDecla
   visit(iterator);
 }
 
-void ContextBuilder::createTypeForInitializer(InitializerAST* /*node*/) {
-}
+void ContextBuilder::createTypeForDeclarator(DeclaratorAST* /*node*/)
+{ }
 
-void ContextBuilder::closeTypeForInitializer(InitializerAST* /*node*/) {
-}
+void ContextBuilder::closeTypeForDeclarator(DeclaratorAST* /*node*/)
+{ }
 
-void ContextBuilder::createTypeForDeclarator(DeclaratorAST* /*node*/) {
-}
+void ContextBuilder::createTypeForInitializer(InitializerAST* /*node*/)
+{ }
 
-void ContextBuilder::closeTypeForDeclarator(DeclaratorAST* /*node*/) {
-}
+void ContextBuilder::createTypeForCondition(ConditionAST* /*node*/)
+{ }
 
 void ContextBuilder::visitParameterDeclarationClause(ParameterDeclarationClauseAST* node)
 {
@@ -929,9 +929,13 @@ void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
     return;
 
   createTypeForDeclarator(node);
-  
-  if(m_currentInitializer) //Needs to be visited now, so the type-builder can use the initializer to build a constant integral tyoe
-    createTypeForInitializer(m_currentInitializer); 
+
+  // These need to be visited now, so the type-builder can use them
+  // to build a constant integral types
+  if(m_currentInitializer)
+    createTypeForInitializer(m_currentInitializer);
+  else if(m_currentCondition)
+    createTypeForCondition(m_currentCondition);
 
   if (node->parameter_declaration_clause && (compilingContexts() || node->parameter_declaration_clause->ducontext)) {
     DUContext* ctx = openContext(node->parameter_declaration_clause, DUContext::Function, node->id);
@@ -947,9 +951,6 @@ void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
   visit(node->trailing_return_type);
   //END Finished with default visitor
 
-  if(m_currentInitializer)
-    closeTypeForInitializer(m_currentInitializer);
-  
   closeTypeForDeclarator(node);
 
   if (node->parameter_declaration_clause && (compilingContexts() || node->parameter_declaration_clause->ducontext))
@@ -1133,6 +1134,13 @@ void ContextBuilder::visitLambdaDeclarator(LambdaDeclaratorAST* node)
   }
 }
 
+void ContextBuilder::visitCondition(ConditionAST* node)
+{
+  m_currentCondition = node;
+  DefaultVisitor::visitCondition(node);
+  m_currentCondition = 0;
+}
+
 bool ContextBuilder::createContextIfNeeded(AST* node, DUContext* importedParentContext)
 {
   QVector<DUContext::Import> imports;
