@@ -50,7 +50,6 @@ While construction:
 
 #include <language/duchain/ducontext.h>
 
-#include <QSet>
 #include <QMutex>
 
 #include <language/duchain/abstractfunctiondeclaration.h>
@@ -61,10 +60,11 @@ While construction:
 #include <language/duchain/classfunctiondeclaration.h>
 #include <language/duchain/namespacealiasdeclaration.h>
 #include "typeutils.h"
-#include "cpptypes.h"
-#include "cppduchain.h"
 #include "templatedeclaration.h"
-#include "expressionparser.h"
+#include "expressionevaluationresult.h"
+#include "cppduchain.h"
+#include "cpptypes.h"
+#include "templatedeclaration.h"
 #include "cppdebughelper.h"
 
 using namespace KDevelop;
@@ -79,10 +79,8 @@ extern QMutex cppDuContextInstantiationsMutex;
           Q_ASSERT(m_source);
         }
         void openQualifiedIdentifier( bool isExplicitlyGlobal ) {
-          QualifiedIdentifier i;
-          i.setExplicitlyGlobal( isExplicitlyGlobal );
           StatePtr s(new State);
-          s->identifier = i;
+          s->identifier.setExplicitlyGlobal( isExplicitlyGlobal );
           m_states << s;
         }
 
@@ -606,8 +604,9 @@ class CppDUContext : public BaseContext {
     }
 
     ///@see TemplateDeclaration::instantiate
-    DUContext* instantiate(const InstantiationInformation& info, const TopDUContext* source) {
-      if(!info.isValid() || m_instantiatedWith == info.indexed() || !this->parentContext())
+    DUContext* instantiate(const InstantiationInformation& info, const TopDUContext* source, const int depth=0) {
+      const int maxDepth = 50;
+      if(!info.isValid() || m_instantiatedWith == info.indexed() || !this->parentContext() ||depth > maxDepth)
         return this;
 
       if(m_instantiatedFrom)
@@ -635,7 +634,7 @@ class CppDUContext : public BaseContext {
         //This context does not have an attached declaration, but it needs to be instantiated.
         CppDUContext<DUContext>* parent = dynamic_cast<CppDUContext<DUContext>* >(this->parentContext());
         if(parent)
-          surroundingContext = parent->instantiate(info.previousInstantiationInformation.information(), source);
+          surroundingContext = parent->instantiate(info.previousInstantiationInformation.information(), source, depth+1);
       }
       
       if(!surroundingContext)
@@ -660,19 +659,17 @@ class CppDUContext : public BaseContext {
         first = *m_instatiations.begin();
         
         Q_ASSERT(first != oldFirst);
+        Q_UNUSED(oldFirst);
+        oldFirst = first;
         
         l.unlock();
         
-        if(first->isAnonymous()) {
-          Q_ASSERT(first->m_instantiatedFrom == this);
-          delete first;
-        } else {
-          Q_ASSERT(first->m_instantiatedFrom == this);
-          first->setInstantiatedFrom(0, InstantiationInformation());
-          Q_ASSERT(first->m_instantiatedFrom == 0);
-        }
-        
-        oldFirst = first;
+        ///TODO: anonymous contexts should get deleted but that is crashy
+        ///      see also declarationbuilder which also encountered this
+        ///      issue before and also removed the context deletion...
+        Q_ASSERT(first->m_instantiatedFrom == this);
+        first->setInstantiatedFrom(0, InstantiationInformation());
+        Q_ASSERT(first->m_instantiatedFrom == 0);
         
         l.relock();
       }      
