@@ -16,7 +16,7 @@
 #include <kdebug.h>
 #include "rpp/pp-macro.h"
 #include "rpp/pp-environment.h"
-#include <language/interfaces/iproblem.h>
+#include <language/duchain/problem.h>
 #include "cpppreprocessenvironment.h"
 #include <language/duchain/repositories/itemrepository.h>
 #include <language/editor/modificationrevision.h>
@@ -226,12 +226,15 @@ bool EnvironmentFile::matchEnvironment(const ParsingEnvironment* _environment) c
 #endif
       return true;
     }
-  
-  ReferenceCountedStringSet environmentMacroNames = cppEnvironment->macroNameSet();
-  
-  ReferenceCountedStringSet conflicts = (environmentMacroNames & strings()) - d_func()->m_usedMacroNames;
+
+  const auto& environmentMacroNames = cppEnvironment->macroNameSet();
+
+  const ReferenceCountedStringSet& conflicts = strings() - d_func()->m_usedMacroNames;
 
   for( ReferenceCountedStringSet::Iterator it(conflicts.iterator()); it; ++it ) {
+    if (!environmentMacroNames.contains( *it )) {
+      continue;
+    }
     rpp::pp_macro* m = cppEnvironment->retrieveStoredMacro( *it );
     if(m && !m->isUndef()) {
       
@@ -487,7 +490,7 @@ void EnvironmentFile::addStrings( const std::set<Utils::BasicSetRepository::Inde
 }
 
 //The parameter should be a EnvironmentFile that was lexed AFTER the content of this file
-void EnvironmentFile::merge( const EnvironmentFile& file, CppPreprocessEnvironment* env ) {
+void EnvironmentFile::merge( const EnvironmentFile& file ) {
   ENSURE_WRITE_LOCKED
   //We have to read the other file
   ENSURE_FILE_READ_LOCKED(file)
