@@ -14,27 +14,22 @@
 #ifndef ENVIRONMENTMANAGER_H
 #define ENVIRONMENTMANAGER_H
 
-#include <map>
-#include <QtCore/qglobal.h>
-
-#include <QtCore/QDateTime>
 #include <QtCore/QList>
 #include <QtCore/QMap>
 #include <QMutex>
 
-#include <kurl.h>
 #include <ksharedptr.h>
 
 #include <language/duchain/parsingenvironment.h>
 #include <language/editor/modificationrevision.h>
-#include <language/editor/editorintegrator.h>
-#include <language/interfaces/iproblem.h>
-#include <language/duchain/topducontext.h>
 
 #include "cppduchainexport.h"
-#include <language/util/setrepository.h>
 #include "parser/rpp/macrorepository.h"
 
+namespace KDevelop {
+  class TopDUContext;
+}
+
 /** 
  * The environment-manager helps achieving right representation of the way c++ works:
  * When a file is processed by the preprocessor, the same file may create totally
@@ -213,7 +208,7 @@ public:
 class KDEVCPPDUCHAIN_EXPORT EnvironmentFile : public KDevelop::ParsingEnvironmentFile {
   public:
     ///@todo Respect changing include-paths: Check if the included files are still the same(maybe new files are found that were not found before)
-    EnvironmentFile( KDevelop::IndexedString url, KDevelop::TopDUContext* topContext );
+    EnvironmentFile( const KDevelop::IndexedString& url, KDevelop::TopDUContext* topContext );
 
     EnvironmentFile( EnvironmentFileData& data );
     
@@ -234,12 +229,10 @@ class KDEVCPPDUCHAIN_EXPORT EnvironmentFile : public KDevelop::ParsingEnvironmen
     
     ///The parameter should be a EnvironmentFile that was processed AFTER the content of this file
     ///@param file The file to merge
-    ///@param env Optionally, the current environment that is active _before_ the
-    ///           included file is "virtually" processed.
     ///@warning The file must _not_ be merged yet into the environment when this is called. Also,
     ///         after merging the environment-files a file cannot be merged into the environment any more,
     ///         so this should only be used indirectly through CppPreprocessEnvironment::merge
-    void merge( const EnvironmentFile& file, CppPreprocessEnvironment* env = 0 );
+    void merge( const EnvironmentFile& file );
 
     bool operator <  ( const EnvironmentFile& rhs ) const {
       return url() < rhs.url();
