@@ -35,13 +35,13 @@
 #include "type_visitor.h"
 #include "lexer.h"
 #include "overloadresolution.h"
-#include "cppduchain.h"
 #include "overloadresolutionhelper.h"
 #include "builtinoperators.h"
 #include "qtfunctiondeclaration.h"
 #include "missingdeclarationtype.h"
 #include "missingdeclarationproblem.h"
 #include "dumpchain.h"
+#include "ptrtomembertype.h"
 
 //If this is enabled and a type is not found, it is searched again with verbose debug output.
 //#define DEBUG_RESOLUTION_PROBLEMS
@@ -94,45 +94,9 @@ namespace Cpp {
 using namespace KDevelop;
 using namespace TypeUtils;
 
-QHash<quint16, QString> initOperatorNames() {
-  QHash<quint16, QString> ret;
-  ret['+'] = "+";
-  ret['-'] = "-";
-  ret['*'] = "*";
-  ret['/'] = "/";
-  ret['%'] = "%";
-  ret['^'] = "^";
-  ret['&'] = "&";
-  ret['|'] = "|";
-  ret['~'] = "~";
-  ret['!'] = "!";
-  ret['='] = "=";
-  ret['<'] = "<";
-  ret['>'] = ">";
-  ret[','] = ",";
-  ret[Token_assign] = "+=";
-  ret[Token_leftshift] = "<<";
-  ret[Token_rightshift] = ">>";
-  ret[Token_eq] = "==";
-  ret[Token_not_eq] = "!=";
-  ret[Token_leq] = "<=";
-  ret[Token_geq] = ">=";
-  ret[Token_not_eq] = "!=";
-  ret[Token_and] = "&&";
-  ret[Token_or] = "||";
-
-  return ret;
-}
-
-QHash<quint16, QString> operatorNames = initOperatorNames();
-//BUG use the much more complete list from tokens.cpp
 QString operatorNameFromTokenKind( quint16 tokenKind )
 {
-  QHash<quint16, QString>::const_iterator it = operatorNames.constFind(tokenKind);
-  if( it == operatorNames.constEnd() )
-    return QString();
-  else
-    return *it;
+  return token_text(tokenKind);
 }
 
 QList<DeclarationPointer> convert( const QList<Declaration*>& list ) {
@@ -243,13 +207,13 @@ void ExpressionVisitor::reportRealProblems(bool report) {
   m_reportRealProblems = report;
 }
 
-void ExpressionVisitor::realProblem( ProblemPointer problem ) {
+void ExpressionVisitor::realProblem( const ProblemPointer& problem ) {
   if(m_reportRealProblems && m_problems.size() < maxExpressionVisitorProblems) {
     m_problems << problem;
   }
 }
 
-QList< KSharedPtr< KDevelop::Problem > > ExpressionVisitor::realProblems() const {
+QList<ProblemPointer> ExpressionVisitor::realProblems() const {
   return m_problems;
 }
 
@@ -290,16 +254,15 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
     isConst |= isConstant(base);
 
-    IdentifiedType* idType = dynamic_cast<IdentifiedType*>( base.unsafeData() );
     //Make sure that it is a structure-type, because other types do not have members
-    StructureType* structureType = dynamic_cast<StructureType*>( base.unsafeData() );
+    const StructureType::Ptr& structureType = base.cast<StructureType>();
 
-    if( !structureType || !idType ) {
+    if( !structureType ) {
       problem( node, QString("findMember called on non-identified or non-structure type \"%1\"").arg(base ? base->toString() : "<type disappeared>") );
       return;
     }
 
-    Declaration* declaration = idType->declaration(topContext());
+    Declaration* declaration = structureType->declaration(topContext());
     MUST_HAVE(declaration);
     MUST_HAVE(declaration->context());
 
@@ -408,7 +371,8 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
   }
 
 
-  AbstractType::Ptr ExpressionVisitor::realLastType(bool* constant) const {
+  AbstractType::Ptr ExpressionVisitor::realLastType() const
+  {
     LOCKDUCHAIN;
     return AbstractType::Ptr(realType( m_lastType, topContext() ));
   }
@@ -420,7 +384,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
     clearLast();
 
-    PointerType* pnt = dynamic_cast<PointerType*>( base.unsafeData() );
+    const PointerType::Ptr& pnt = base.cast<PointerType>();
     if( pnt ) {
       if( constant )
         (*constant) |= (pnt->modifiers() & AbstractType::ConstModifier);
@@ -479,17 +443,16 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
       isConst |= isConstant(m_lastType);
 
-      IdentifiedType* idType = dynamic_cast<IdentifiedType*>( m_lastType.unsafeData() );
       //Make sure that it is a structure-type, because other types do not have members
-      StructureType* structureType = dynamic_cast<StructureType*>( m_lastType.unsafeData() );
+      const StructureType::Ptr& structureType = m_lastType.cast<StructureType>();
 
-      if( !structureType || !idType ) {
+      if( !structureType ) {
         problem( node, QString("member searched in non-identified or non-structure type \"%1\"").arg(m_lastType ? m_lastType->toString() : "<type disappeared>") );
         clearLast();
         return;
       }
 
-      Declaration* declaration = idType->declaration(topContext());
+      Declaration* declaration = structureType->declaration(topContext());
       MUST_HAVE(declaration);
       MUST_HAVE(declaration->context());
 
@@ -508,19 +471,19 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       return;
     }
 
-    QualifiedIdentifier identifier = nameV.identifier();
+    const QualifiedIdentifier& identifier = nameV.identifier();
 
     ///@todo It would be better if the parser would treat true and false exactly
     ///like constant-integer expressions, storing them in a primary expression.
-    static QualifiedIdentifier trueIdentifier("true");
-    static QualifiedIdentifier falseIdentifier("false");
+    static const QualifiedIdentifier trueIdentifier("true");
+    static const QualifiedIdentifier falseIdentifier("false");
 
     if( identifier == trueIdentifier || identifier == falseIdentifier ) {
       ///We have a boolean constant, we need to catch that here
-      LOCKDUCHAIN;
-      m_lastType = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeBoolean));
+      ConstantIntegralType::Ptr type(new ConstantIntegralType(IntegralType::TypeBoolean));
+      type->setValue<qint64>( identifier == trueIdentifier );
+      m_lastType = type.cast<AbstractType>();
       m_lastInstance = Instance( true );
-      static_cast<ConstantIntegralType*>(m_lastType.unsafeData())->setValue<qint64>( identifier == trueIdentifier );
     } else {
       LOCKDUCHAIN;
 
@@ -545,7 +508,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
         missing->searchStartContext = const_cast<DUContext*>(m_currentContext);
 
         if(m_reportRealProblems && m_problems.size() < maxExpressionVisitorProblems) {
-          KSharedPtr<KDevelop::Problem> problem(new Cpp::MissingDeclarationProblem(missing));
+          ProblemPointer problem(new Cpp::MissingDeclarationProblem(missing));
           problem->setSource(KDevelop::ProblemData::SemanticAnalysis);
           CppEditorIntegrator editor(session());
           
@@ -580,7 +543,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
           m_lastInstance = Instance(false);
 
         //A CppTemplateParameterType represents an unresolved template-parameter, so create a DelayedType instead.
-        if( dynamic_cast<CppTemplateParameterType*>(m_lastType.unsafeData()) )
+        if( m_lastType.cast<CppTemplateParameterType>() )
           createDelayedType(node, false);
       }
     }
@@ -628,7 +591,6 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     if (token.kind == Token_number_literal) {
       QString num = m_session->token_stream->symbolString(token);
 
-      LOCKDUCHAIN;
       if( num.indexOf('.') != -1 || num.endsWith('f') || num.endsWith('d') ) {
         double val = 0;
         bool ok = false;
@@ -639,11 +601,13 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
 
         if( num.endsWith('f') ) {
-          m_lastType = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeFloat));
-          static_cast<ConstantIntegralType*>(m_lastType.unsafeData())->setValue<float>((float)val);
+          ConstantIntegralType::Ptr type(new ConstantIntegralType(IntegralType::TypeFloat));
+          type->setValue<float>((float)val);
+          m_lastType = type.cast<AbstractType>();
         } else {
-          m_lastType = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeDouble));
-          static_cast<ConstantIntegralType*>(m_lastType.unsafeData())->setValue<double>(val);
+          ConstantIntegralType::Ptr type(new ConstantIntegralType(IntegralType::TypeDouble));
+          type->setValue<double>(val);
+          m_lastType = type.cast<AbstractType>();
         }
       } else {
         qint64 val = 0;
@@ -671,8 +635,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       return;
     } else if(token.kind == Token_char_literal) {
       // char literal e.g. 'x'
-      LOCKDUCHAIN;
-      ConstantIntegralType* charType = new ConstantIntegralType(IntegralType::TypeChar);
+      ConstantIntegralType::Ptr charType(new ConstantIntegralType(IntegralType::TypeChar));
       if ( token.size == 3 ) {
         charType->setValue<char>( m_session->token_stream->symbolByteArray(token).at(1) );
       } else {
@@ -698,14 +661,14 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
         }
       }
 
-      m_lastType = AbstractType::Ptr(charType);
+      m_lastType = charType.cast<AbstractType>();
       m_lastInstance = Instance( true );
     } else if (token.kind == Token_true || token.kind == Token_false) {
       ///We have a boolean constant, we need to catch that here
-      LOCKDUCHAIN;
-      m_lastType = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeBoolean));
+      ConstantIntegralType::Ptr type(new ConstantIntegralType(IntegralType::TypeBoolean));
+      type->setValue<qint64>( token.kind == Token_true );
+      m_lastType = type.cast<AbstractType>();
       m_lastInstance = Instance( true );
-      static_cast<ConstantIntegralType*>(m_lastType.unsafeData())->setValue<qint64>( token.kind == Token_true );
     } else if( token.kind == Token_this ) {
       LOCKDUCHAIN;
 
@@ -852,7 +815,7 @@ struct ConstantUnaryExpressionEvaluator {
   /**
    * Writes the results into endValue, type, and modifier.
    * */
-  ConstantUnaryExpressionEvaluator( quint16 tokenKind, ConstantIntegralType* left ) {
+  ConstantUnaryExpressionEvaluator( quint16 tokenKind, const ConstantIntegralType::Ptr& left ) {
     endValue = 0;
     type = left->dataType();
     modifier = left->modifiers();
@@ -866,13 +829,15 @@ struct ConstantUnaryExpressionEvaluator {
       break;
       case Token_incr:
         endValue = left->value<Type>()+1;
+      break;
       case Token_decr:
         endValue = left->value<Type>()-1;
+      break;
     }
   }
 
   //This function is used to disable some operators on bool and double values
-  void evaluateSpecialTokens( quint16 tokenKind, ConstantIntegralType* left ) {
+  void evaluateSpecialTokens( quint16 tokenKind, const ConstantIntegralType::Ptr& left ) {
     switch( tokenKind ) {
       case '~':
         endValue = ~left->value<Type>();
@@ -883,22 +848,23 @@ struct ConstantUnaryExpressionEvaluator {
     }
   }
 
-  AbstractType::Ptr createType() {
-    AbstractType::Ptr ret = AbstractType::Ptr(new ConstantIntegralType(type));
+  AbstractType::Ptr createType() const
+  {
+    ConstantIntegralType::Ptr ret(new ConstantIntegralType(type));
     ret->setModifiers(modifier);
-    static_cast<ConstantIntegralType*>(ret.unsafeData())->setValue<Type>( endValue );
-    return ret;
+    ret->setValue<Type>( endValue );
+    return ret.cast<AbstractType>();
   }
 };
 
 template<>
-void ConstantUnaryExpressionEvaluator<double>::evaluateSpecialTokens( quint16 tokenKind, ConstantIntegralType* left ) {
+void ConstantUnaryExpressionEvaluator<double>::evaluateSpecialTokens( quint16 tokenKind, const ConstantIntegralType::Ptr& left ) {
   Q_UNUSED(tokenKind);
   Q_UNUSED(left);
 }
 
 template<>
-void ConstantUnaryExpressionEvaluator<float>::evaluateSpecialTokens( quint16 tokenKind, ConstantIntegralType* left ) {
+void ConstantUnaryExpressionEvaluator<float>::evaluateSpecialTokens( quint16 tokenKind, const ConstantIntegralType::Ptr& left ) {
   Q_UNUSED(tokenKind);
   Q_UNUSED(left);
 }
@@ -1036,7 +1002,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       return;
     }
 
-    if( dynamic_cast<DelayedType*>(rightType.unsafeData()) || dynamic_cast<DelayedType*>(leftType.unsafeData()) ) {
+    if( rightType.cast<DelayedType>() || leftType.cast<DelayedType>() ) {
       m_lastInstance = Instance(true);
       createDelayedType(node);
       return;
@@ -1131,7 +1097,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
         ///Allow non-types, because we sometimes don't know whether something is a type or not, and it may get parsed as a type.
         m_lastInstance = Instance(decls.first());
 
-      if( dynamic_cast<CppTemplateParameterType*>(m_lastType.unsafeData()) )
+      if( m_lastType.cast<CppTemplateParameterType>() )
         createDelayedType(ast, false);
     } else {
       problem(ast, "Could not resolve type");
@@ -1161,7 +1127,16 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   {
     if(node->declarator)
     {
-      CppClassType::Ptr constructedType = computeConstructedType();
+      // apply pointer ops to lvalue type
+      visitNodes(this, node->declarator->ptr_ops);
+      CppClassType::Ptr constructedType;
+      if (!m_lastType || !isPointerType(m_lastType)) {
+        // Do not blindly dereference, esp. for 'foo* f = new foo;' expressions.
+        // Note how computeConstructedType only takes the declaration in
+        // m_lastDeclarations into account.
+        // Thus, if the lvalue is a pointer we definitely should not get a ClassType.
+        constructedType = computeConstructedType();
+      }
 
       //Build constructor uses (similar to visitFunctionCall)
 
@@ -1173,30 +1148,37 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
       bool fail = true;
 
-      size_t token = node->start_token;
+      size_t start_token = node->start_token;
       //NOTE: we might have an initializer in a class for pure virtual methods,
       //      but we just ignore that. See also TestDUChain::testForwardDeclaration4
-      if(node->initializer && m_currentContext->type() != DUContext::Class)
+      if(m_currentContext->type() != DUContext::Class)
       {
-        if(node->initializer->expression && !node->initializer->initializer_clause)
+        if (!node->initializer) {
+          // No InitializerAST => Default-initialization (e.g. '{ A a; }')
+          start_token = node->end_token;
+          fail = false;
+        } else if(node->initializer->expression && !node->initializer->initializer_clause)
         {
-          token = node->initializer->start_token;
+          start_token = node->initializer->start_token;
           fail = !buildParametersFromExpression(node->initializer->expression);
         } else if(!node->initializer->expression && node->initializer->initializer_clause && constructedType)
         { // report operator= use in i.e.: foo = bar;
-          token = node->initializer->start_token;
+          start_token = node->initializer->start_token;
           fail = !buildParametersFromExpression(node->initializer->initializer_clause);
-          LOCKDUCHAIN;
           declarations.clear();
+          LOCKDUCHAIN;
           if ( ClassDeclaration* cdec = dynamic_cast<ClassDeclaration*>(constructedType->declaration(m_source)) ) {
+            // constructors are handled automatically in the overload resultion
+            declarations << DeclarationPointer(cdec);
             ///TODO: global operator= functions, for now only class members are handled
-            foreach(Declaration* dec, cdec->internalContext()->findDeclarations(Identifier("operator="))) {
+            static const Identifier opEq("operator=");
+            foreach(Declaration* dec, cdec->internalContext()->findDeclarations(opEq)) {
               declarations << DeclarationPointer(dec);
             }
           }
         } else if (!node->initializer->expression && !node->initializer->initializer_clause) {
           // ctor without parameters, i.e.: foo();
-          token = node->initializer->start_token;
+          start_token = node->initializer->start_token;
           fail = false;
         }
       }
@@ -1204,8 +1186,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       if(fail || !constructedType) {
         DefaultVisitor::visitInitDeclarator(node);
         return;
-      } else {
-        visitNodes(this,node->declarator->ptr_ops);
       }
 
       DeclarationPointer chosenFunction;
@@ -1226,7 +1206,10 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       }
 
       if(chosenFunction) {
-        newUse( node , token, token+1, chosenFunction );
+        // if there's an InitializerAST, we let 'Show Uses' point to the opening bracket of the ctor call
+        // unfortunately for cases '{ A a; }', we don't have anything we can point to
+        const size_t end_token = start_token + (node->initializer ? 1 : 0);
+        newUse( node , start_token, end_token, chosenFunction );
         if(m_mapAst) session()->mapCallAstToType(node, chosenFunction->type<FunctionType>());
       }
     }else{
@@ -1268,7 +1251,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     visit(node->parameter_declaration_clause);
     visit(node->exception_spec);
     {
-    LOCKDUCHAIN;
     if( node->array_dimensions && oldLastType ) {
       ArrayType::Ptr p( new ArrayType() );
       p->setElementType( oldLastType );
@@ -1443,7 +1425,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     if( m_lastType )
     {
-      LOCKDUCHAIN;
       ///@todo cv-qualifiers
       PointerType::Ptr p( new PointerType() );
       p->setBaseType( m_lastType );
@@ -1514,7 +1495,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     visit(node->condition);
 
 
-    if( dynamic_cast<DelayedType*>(m_lastType.unsafeData()) ) {
+    if( m_lastType.cast<DelayedType>() ) {
       //Store the expression so it's evaluated later
       m_lastInstance = Instance(true);
       createDelayedType(node);
@@ -1535,8 +1516,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     visit(node->right_expression);
 
     {
-      LOCKDUCHAIN;
-      if( ConstantIntegralType* condition = dynamic_cast<ConstantIntegralType*>( conditionType.unsafeData() ) ) {
+      if( const ConstantIntegralType::Ptr& condition = conditionType.cast<ConstantIntegralType>() ) {
         ///For constant integral types, the condition could be evaluated, so we choose the correct result.
         if( condition->value<quint64>() == 0 ) {
           ///The right expression is the correct one, so do nothing
@@ -1581,7 +1561,8 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       expressionType( node, m_lastType, m_lastInstance );
   }
 
-  bool ExpressionVisitor::dereferenceLastPointer(AST* node) {
+  bool ExpressionVisitor::dereferenceLastPointer()
+  {
     if( PointerType::Ptr pt = realLastType().cast<PointerType>() )
     {
       //Dereference
@@ -1622,7 +1603,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     case '*':
     {
       LOCKDUCHAIN;
-      if( dereferenceLastPointer(node) ) {
+      if( dereferenceLastPointer() ) {
       } else {
         //Get return-value of operator*
         findMember(node, m_lastType, Identifier("operator*") );
@@ -1649,11 +1630,11 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     break;
     default:
     {
-      KDevelop::IntegralType* integral = dynamic_cast<KDevelop::IntegralType*>(m_lastType.unsafeData());
+      const IntegralType::Ptr& integral = m_lastType.cast<IntegralType>();
       if( integral ) {
         //The type of integral types does not change on unary operators
         //Eventually evaluate the value of constant integral types
-        ConstantIntegralType* constantIntegral = dynamic_cast<ConstantIntegralType*>(integral);
+        const ConstantIntegralType::Ptr& constantIntegral = integral.cast<ConstantIntegralType>();
 
         if( constantIntegral ) {
 
@@ -1733,7 +1714,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     if( !m_lastType )
       return;
 
-    KDevelop::FunctionType* f = dynamic_cast<KDevelop::FunctionType*>( m_lastType.unsafeData() );
+    const FunctionType::Ptr& f = m_lastType.cast<FunctionType>();
     if( !f ) {
       LOCKDUCHAIN;
       problem(node, QString("cannot get return-type of type %1, it is not a function-type").arg(m_lastType->toString()));
@@ -1751,8 +1732,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   {
     CppClassType::Ptr constructedType;
 
-    AbstractType::Ptr oldLastType = m_lastType;
-
     if(!m_lastInstance) {
       LOCKDUCHAIN;
       if(m_lastDeclarations.isEmpty() && m_lastType && !m_lastInstance) {
@@ -1923,7 +1902,8 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     MissingDeclarationType::Ptr missing;
     
-    if( declarations.isEmpty()) {
+    Declaration* dec = declarations.isEmpty() ? 0 : declarations.first().data();
+    if( !dec ) {
       missing = oldLastType.cast<Cpp::MissingDeclarationType>();
       if (missing) {
         // Eventually use ADL lookup to find the missing declaration
@@ -1936,7 +1916,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }else{
       // Eventually use ADL
       if(!m_hadMemberAccess)
-        helper.setFunctionNameForADL(QualifiedIdentifier(declarations.first()->identifier()));
+        helper.setFunctionNameForADL(QualifiedIdentifier(dec->identifier()));
     }
     
     ViableFunction viable;
@@ -1954,7 +1934,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     if( !chosenFunction && constructedType )
     {
       //Default-constructor is used
-      m_lastType = AbstractType::Ptr(constructedType.unsafeData());
+      m_lastType = constructedType.cast<AbstractType>();
       DeclarationPointer decl(constructedType->declaration(topContext()));
       m_lastInstance = Instance(decl.data());
       m_lastDeclarations.clear();
@@ -2001,8 +1981,8 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       // Remove the MissingDeclarationProblem which has been created alongside MissingDeclarationType
       for(int idx = m_problems.size()-1; idx >= 0; --idx)
       {
-        KSharedPtr<KDevelop::Problem>& prob(m_problems[idx]);
-        MissingDeclarationProblem * pMissing = dynamic_cast<MissingDeclarationProblem*>(prob.data());
+        const ProblemPointer& prob = m_problems[idx];
+        const MissingDeclarationProblem* pMissing = dynamic_cast<const MissingDeclarationProblem*>(prob.constData());
         if (pMissing && pMissing->type == missing )
         {
           m_problems.removeAt(idx);
@@ -2015,7 +1995,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     if( constructedType ) {
       //Constructor was called
-      m_lastType = AbstractType::Ptr(constructedType.unsafeData());
+      m_lastType = constructedType.cast<AbstractType>();
       m_lastInstance = Instance(constructedType->declaration(topContext()));
     } else if (chosenFunction) {
       KDevelop::FunctionType::Ptr functionType = chosenFunction->abstractType().cast<KDevelop::FunctionType>();
@@ -2034,7 +2014,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     if(chosenFunction)
       if(m_mapAst) session()->mapCallAstToType(node, chosenFunction->type<FunctionType>());
 
-    static IndexedString functionCallOperatorIdentifier("operator()");
+    static const IndexedString functionCallOperatorIdentifier("operator()");
 
     bool createUseOnParen = (bool)chosenFunction && (constructedType || chosenFunction->identifier().identifier() == functionCallOperatorIdentifier);
     //Re-create the use we have discarded earlier, this time with the correct overloaded function chosen.
@@ -2075,8 +2055,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitSignalSlotExpression(SignalSlotExpressionAST* node) {
 
     //So uses for the argument-types are built
-    LOCKDUCHAIN;
-
     putStringType();
 
     if(m_parameters.isEmpty())
@@ -2084,6 +2062,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     DUContext* container = 0;///@todo check whether signal/slot match, warn if not.
 
+    LOCKDUCHAIN;
     StructureType::Ptr slotStructure = TypeUtils::targetType(TypeUtils::matchingClassPointer(qObjectPtrType(), TypeUtils::realType(m_parameters.back().type, m_topContext), m_topContext), m_topContext).cast<StructureType>();
     if(slotStructure)
       container = slotStructure->internalContext(m_topContext);
@@ -2165,7 +2144,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       LOCKDUCHAIN;
 
       //If the type the subscript-operator is applied on is a pointer, dereference it
-      if( dereferenceLastPointer(node) ) {
+      if( dereferenceLastPointer() ) {
         //Make visit the sub-expression, so uses are built
         lock.unlock();
 
@@ -2225,13 +2204,13 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitSizeofExpression(SizeofExpressionAST* node)  {
     visit(node->type_id);
     visit(node->expression);
-    LOCKDUCHAIN;
     m_lastType = AbstractType::Ptr( new KDevelop::IntegralType(IntegralType::TypeInt) );
     m_lastInstance = Instance(true);
   }
 
-  void ExpressionVisitor::visitCondition(ConditionAST* node)  {
-    LOCKDUCHAIN;
+  void ExpressionVisitor::visitCondition(ConditionAST* node)
+  {
+    DefaultVisitor::visitCondition(node);
     m_lastType = AbstractType::Ptr( new KDevelop::IntegralType(IntegralType::TypeBoolean) );
     m_lastInstance = Instance(true);
   }
@@ -2255,15 +2234,14 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     PointerType::Ptr p( new PointerType() );
 
-    p->setBaseType( AbstractType::Ptr(i.unsafeData()) );
+    p->setBaseType( i.cast<AbstractType>() );
 
     m_lastType = p.cast<AbstractType>();
     m_lastInstance = Instance(true);
   }
 
 
-  void ExpressionVisitor::visitStringLiteral(StringLiteralAST* node)  {
-    LOCKDUCHAIN;
+  void ExpressionVisitor::visitStringLiteral(StringLiteralAST* /*node*/) {
     ///TODO: proper support for wchar_t, char16_t and char32_t strings
     putStringType();
   }
@@ -2272,7 +2250,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     ///post-fix increment/decrement like "i++" or "i--"
     ///This does neither change the evaluated value, nor the type(except for overloaded operators)
 
-    if( dynamic_cast<KDevelop::IntegralType*>(m_lastType.unsafeData()) ) {
+    if( m_lastType.cast<IntegralType>() ) {
       ///Leave the type and its value alone
     } else {
       ///It is not an integral type, try finding an overloaded operator and use the return-value
@@ -2288,7 +2266,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
         helper.setOperator( OverloadResolver::Parameter(m_lastType, isLValue( m_lastType, m_lastInstance ), m_lastInstance.declaration.data() ) );
 
         //Overloaded postfix operators have one additional int parameter
-        static AbstractType::Ptr integer = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeInt));
+        static const AbstractType::Ptr integer = AbstractType::Ptr(new ConstantIntegralType(IntegralType::TypeInt));
         helper.setKnownParameters( OverloadResolver::Parameter( integer, false ) );
 
         ViableFunction viable = helper.resolve();
