@@ -311,9 +311,15 @@ MissingDeclarationAssistant::MissingDeclarationAssistant(const MissingDeclaratio
 : problem(p)
 , type(p->type)
 {
+  bool actionAdded = false;
   DUChainReadLocker lock;
-  if(p->type->identifier().identifier().identifier().isEmpty())
+
+  // support for namespaced types/variable names is broken atm, so just rather disable it
+  // i.e. writing 'foo::bar()' will just add 'bar' to the container
+  auto qualifiedIdentifier = p->type->identifier().identifier().identifier();
+  if (qualifiedIdentifier.isEmpty() || qualifiedIdentifier.count() > 1)
     return;
+
   kDebug() << "creating assistant for" << type->toString() << "assigned:" << type->assigned.toString();
 
   DUContext* searchFrom = type->searchStartContext.data();
@@ -340,7 +346,7 @@ MissingDeclarationAssistant::MissingDeclarationAssistant(const MissingDeclaratio
     // public is always possible
     CreateMemberDeclarationAction* publicAction = new CreateMemberDeclarationAction(problem, Declaration::Public);
 
-    if (actions().size()) {
+    if (actionAdded) {
       // place label between first action and the following actions
       const QString label = i18nc("assistant, declare member in class identified by %1",
                                   "member of <code>%1</code>:",
