@@ -19,24 +19,21 @@
 #ifndef OVERLOADRESOLUTION_H
 #define OVERLOADRESOLUTION_H
 
+#include <language/duchain/declaration.h>
 #include <language/duchain/duchainpointer.h>
-#include "typeconversion.h"
 #include <QList>
 #include "cppduchainexport.h"
 #include <language/duchain/identifier.h>
 
-class CppTemplateParameterType;
-
 namespace KDevelop {
   class DUContext;
   class Declaration;
-  class FunctionType;
 }
 
 namespace Cpp {
 using namespace KDevelop;
   class ViableFunction;
-  class ADLHelper;
+
 /**
  * Models overloaded function resolution
  * The du-chain must be locked for the whole lifetime of this object.
@@ -90,13 +87,24 @@ class KDEVCPPDUCHAIN_EXPORT OverloadResolver {
       }
     };
 
+    enum Constness {
+      // only const methods are viable
+      Const,
+      // both, const and non-const methods are viable,
+      // but non-const methods are preferred
+      NonConst,
+      // both const and non-const methods are viable
+      Unknown
+    };
+
     /**
      * @param container The container in which to search for the functions. If it is a class, base-classes will be respected too.
      * @param topContext The top-context of the file where where code-completion/parsing is started from. Needed to resolve forward-declarations.
+     * @param constness If set, constness of class methods is taken into account
      * @param forceIsInstance If this is true, every class given to this overload-resolver as an object the function-call is applied on is considered
-     *                                        an instance of that object, rather then the type itself. This means that "operator()" will be used instead of a constructor.
+     *                        an instance of that object, rather then the type itself. This means that "operator()" will be used instead of a constructor.
      * */
-    OverloadResolver( DUContextPointer context, TopDUContextPointer topContext, bool forceIsInstance = false );
+    OverloadResolver( DUContextPointer context, TopDUContextPointer topContext, Constness constness = Unknown, bool forceIsInstance = false );
 
     /**
      * Resolve one function with the given name that matches the given parameters.
@@ -202,8 +210,19 @@ class KDEVCPPDUCHAIN_EXPORT OverloadResolver {
     TopDUContextPointer m_topContext;
     uint m_worstConversionRank;
     bool m_forceIsInstance;
+    Constness m_constness;
 };
 
 }
 
+/**
+ * {q,k}Debug() stream operator: Writes the parameter to the debug output.
+ */
+KDEVCPPDUCHAIN_EXPORT QDebug operator<<(QDebug s, const Cpp::OverloadResolver::Parameter& param);
+
+/**
+ * {q,k}Debug() stream operator: Writes the parameters to the debug output.
+ */
+KDEVCPPDUCHAIN_EXPORT QDebug operator<<(QDebug s, const Cpp::OverloadResolver::ParameterList& params);
+
 #endif
