@@ -29,9 +29,7 @@
 #include <language/duchain/aliasdeclaration.h>
 #include <language/duchain/functiondeclaration.h>
 #include <language/duchain/functiondefinition.h>
-#include <language/duchain/repositories/itemrepository.h>
 #include <language/duchain/classfunctiondeclaration.h>
-#include <language/duchain/appendedlist.h>
 
 #include "templateparameterdeclaration.h"
 #include "qtfunctiondeclaration.h"
@@ -41,13 +39,7 @@
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/duchainregister.h>
 #include <util/pushvalue.h>
-#include <name_compiler.h>
 #include <parsesession.h>
-#include <rpp/chartools.h>
-#include <rpp/pp-location.h>
-#include <control.h>
-#include <parser.h>
-#include <typeinfo>
 
 using namespace KDevelop;
 using namespace Cpp;
@@ -78,8 +70,11 @@ namespace Cpp {
 AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::IndexedTypeIdentifier& id ) {
   AbstractType::Ptr ret = ptr;
 
-  if(ret && static_cast<bool>(ret->modifiers() & AbstractType::ConstModifier) != id.isConstant())
-    ret->setModifiers(id.isConstant() ? AbstractType::ConstModifier : AbstractType::NoModifiers);
+  if(ret && ((static_cast<bool>(ret->modifiers() & AbstractType::ConstModifier) != id.isConstant())
+         || (static_cast<bool>(ret->modifiers() & AbstractType::VolatileModifier) != id.isVolatile()))) {
+    ret->setModifiers((id.isConstant() ? AbstractType::ConstModifier : AbstractType::NoModifiers)
+                    | (id.isVolatile() ? AbstractType::VolatileModifier : AbstractType::NoModifiers));
+  }
 
   for( int a = 0; a < id.pointerDepth(); ++a ) {
     uint modifiers = AbstractType::NoModifiers;
@@ -95,7 +90,9 @@ AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::
   if(id.isReference() ) {
     uint modifiers = AbstractType::NoModifiers;
     if( id.isConstant() )
-      modifiers = AbstractType::ConstModifier;
+      modifiers |= AbstractType::ConstModifier;
+    if( id.isVolatile() )
+      modifiers |= AbstractType::VolatileModifier;
 
     ReferenceType::Ptr newRet( new ReferenceType() );
     newRet->setModifiers(modifiers);
@@ -114,14 +111,17 @@ struct DelayedTypeSearcher : public KDevelop::SimpleTypeVisitor {
   DelayedTypeSearcher() : found(false) {
   }
 
-  virtual bool visit ( const AbstractType* type ) {
+  // We want to use SimpleTypeVisitor implementations
+  using SimpleTypeVisitor::visit;
+
+  virtual bool visit ( const AbstractType* type ) override {
     if( found ) return false;
     if( dynamic_cast<const DelayedType*>(type) )
       found = true;
     return !found;
   }
 
-  virtual bool visit (const StructureType *) {
+  virtual bool visit (const StructureType *) override {
     ///We do not want to visit member-types, so return false here
     return false;
   }
@@ -219,7 +219,7 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger
   virtual AbstractType::Ptr exchange( const AbstractType::Ptr& type )
   {
     ThreadLocalData& data = threadDataLocal();
-    PushValue<uint> inc(data.delayedDepth, +1);
+    PushValue<uint> inc(data.delayedDepth, data.delayedDepth + 1);
     if( data.delayedDepth > 30 ) {
       kDebug(9007) << "Too much depth in DelayedTypeResolver::exchange, while exchanging" << (type ? type->toString() : QString("(null)"));
       return type;
@@ -257,7 +257,18 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger
       else
         res = p.evaluateType( delayedType->identifier().toString().toUtf8(), DUContextPointer(const_cast<DUContext*>(searchContext)), source );
 
-      return res.type.abstractType();
+      // NOTE: This looks hacky, but see e.g. TestDUChain::testDecltypeTypedef - we really _never_
+      //       can replace a delayed type with a CppTemplateParameterType. Instead we need to create
+      //       a new delayed type for that with its identifier.
+      //TODO: try to add support for replacing CppTemplateParameterTypes directly eventually...
+      //      dunno if its actually doable though
+      AbstractType::Ptr ret = res.type.abstractType();
+      if (CppTemplateParameterType::Ptr tplParam = ret.cast<CppTemplateParameterType>()) {
+        DelayedType::Ptr delayed(new DelayedType);
+        delayed->setIdentifier(IndexedTypeIdentifier(tplParam->declarationId().qualifiedIdentifier().last().toString()));
+        return delayed.cast<AbstractType>();
+      }
+      return ret;
     }else{
       if( containsDelayedType(type) )
       {
@@ -273,7 +284,6 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger
         if( aType )
           aType->exchangeTypes(this);
 
-        
         return typeCopy;
       }
     }
@@ -593,6 +603,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
             ///Resolve the delayed type, and import the context
             DelayedTypeResolver res(contextCopy, source);
             AbstractType::Ptr newType( res.exchange(delayed.cast<AbstractType>()) );
+            newType = TypeUtils::unAliasedType(newType);
 
             if( CppClassType::Ptr baseClass = newType.cast<CppClassType>() )
             {
@@ -604,7 +615,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
               newInstance.baseClass = newType->indexed();
               klass->replaceBaseClass( num, newInstance );
             } else {
-//               kDebug(9007) << "Resolved bad base-class";
+              kWarning(9007) << "Resolved bad base-class" << delayed->toString() << (newType ? newType->toString() : QString());
             }
           }
           ++num;
@@ -633,7 +644,7 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
         ///an AliasDeclaration represents a C++ "using bla::bla;" declaration.
         if(AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(instantiatedDeclaration)) {
           ThreadLocalData& data = threadDataLocal();
-          PushValue<uint> safety(data.aliasDepth, +1);
+          PushValue<uint> safety(data.aliasDepth, data.delayedDepth + 1);
           if(data.aliasDepth > 30) {
             kWarning() << "depth-limit reached while resolving alias-declaration" << alias->identifier().toString() << "within" << parentContext->scopeIdentifier(true).toString();
           }else {
@@ -744,7 +755,6 @@ void TemplateDeclaration::deleteAllInstantiations()
   foreach( TemplateDeclaration* decl, instantiations ) {
     Q_ASSERT(decl);
     decl->m_instantiatedFrom = 0;
-    Declaration* realDecl = dynamic_cast<Declaration*>(decl);
     //Only delete real insantiations, not specializations
     //FIXME: before this checked for decl->isAnonymous
     //This was a problem because some instantiations are not anonymous, so they end up orphaned from their m_instantiatedFrom
