@@ -28,13 +28,9 @@
 #include "declarationbuilder.h"
 #include "usebuilder.h"
 #include <language/duchain/declaration.h>
-#include <language/editor/documentrange.h>
 #include <language/duchain/dumpchain.h>
-#include "cppeditorintegrator.h"
-#include "dumptypes.h"
 #include "environmentmanager.h"
 #include "templatedeclaration.h"
-#include "cppducontext.h"
 #include <rpp/chartools.h>
 
 
@@ -44,7 +40,7 @@
 #include "rpp/preprocessor.h"
 #include "expressionvisitor.h"
 #include "expressionparser.h"
-#include <language/duchain/classfunctiondeclaration.h>
+#include "typeconversion.h"
 
 #include <tests/autotestshell.h>
 #include <tests/testcore.h>
@@ -88,7 +84,7 @@ TestExpressionParser::TestExpressionParser()
 
 void TestExpressionParser::initTestCase()
 {
-  AutoTestShell::init();
+  AutoTestShell::init(QStringList() << "kdevcppsupport");
   TestCore* core = new TestCore();
   core->initialize(KDevelop::Core::NoUi);
   EnvironmentManager::init();
@@ -198,6 +194,15 @@ void TestExpressionParser::testIntegralType() {
     QCOMPARE(ciType->dataType(), (uint)ConstantIntegralType::TypeChar);
     QCOMPARE(ciType->value<char>(), 'x');
   }
+  {
+    Cpp::ExpressionEvaluationResult result = parser.evaluateType("char(1)", KDevelop::DUContextPointer(top));
+    QVERIFY(result.isValid());
+
+    AbstractType::Ptr aType(result.type.abstractType());
+    IntegralType::Ptr iType = aType.cast<IntegralType>();
+    QVERIFY(iType);
+    QCOMPARE(iType->dataType(), (uint)IntegralType::TypeChar);
+  }
   {
       Cpp::ExpressionEvaluationResult result = parser.evaluateType("5", KDevelop::DUContextPointer(top));
       QVERIFY(result.isValid());
@@ -451,11 +456,6 @@ void TestExpressionParser::testSmartPointer() {
   QCOMPARE(baseTemplateContext->type(), DUContext::Template);
   kDebug(9007) << typeid(baseTemplateContext).name();
   kDebug(9007) << typeid(specialTemplateContext).name();
-/*  CppDUContext<DUContext>* baseTemplateCtx = dynamic_cast<CppDUContext<DUContext>*>(baseTemplateContext);
-  CppDUContext<DUContext>* specialTemplateCtx = dynamic_cast<CppDUContext<DUContext>*>(specialTemplateContext);
-  QVERIFY(baseTemplateCtx);
-  QVERIFY(specialTemplateCtx);
-  QCOMPARE(specialTemplateCtx->instantiatedFrom(), baseTemplateContext);*/
 
   QCOMPARE(specialTemplateContext->localDeclarations().count(), 1);
   QCOMPARE(baseTemplateContext->localDeclarations().count(), 1);
@@ -675,7 +675,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int(5)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
@@ -684,7 +684,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "5.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("5.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double(5.5)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
@@ -693,7 +693,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "5.5 + 1.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("7"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double(7)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
@@ -702,7 +702,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "3 + 0.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("3.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double(3.5)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
@@ -711,7 +711,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "3u + 0.5", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("3.5"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("double(3.5)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
@@ -720,7 +720,7 @@ void TestExpressionParser::testSimpleExpression() {
   result = parser.evaluateExpression( "0", KDevelop::DUContextPointer(testContext));
   lock.lock();
   QVERIFY(result.isValid());
-  QCOMPARE(result.type.abstractType()->toString(), QString("0"));
+  QCOMPARE(result.type.abstractType()->toString(), QString("int(0)"));
   QVERIFY(result.isInstance);
   QVERIFY(result.type.type<ConstantIntegralType>());
   lock.unlock();
