@@ -22,10 +22,8 @@
 #include <QHash>
 #include <language/duchain/types/typeutils.h>
 #include "cppduchainexport.h"
-#include <language/duchain/declaration.h>
 #include "cpptypes.h"
 
-struct C;
 namespace KDevelop {
   class DUContext;
   class Declaration;
@@ -107,7 +105,7 @@ namespace TypeUtils {
    * @param functions A hash that will map functions to their types
    * @param functionName Name of the functions
    * */
-  KDEVCPPDUCHAIN_EXPORT void getMemberFunctions(const CppClassType::Ptr& klass, const TopDUContext* topContext, QHash<KDevelop::FunctionType::Ptr, ClassFunctionDeclaration*>& functions, const QString& functionName, bool mustBeConstant=false);
+  KDEVCPPDUCHAIN_EXPORT void getMemberFunctions(const CppClassType::Ptr& klass, const TopDUContext* topContext, QHash<KDevelop::FunctionType::Ptr, ClassFunctionDeclaration*>& functions, const QString& functionName, bool mustBeConstant=false, int recursionCount=0);
 
   /**
    * Same as above, except that it adds the functions to a list.
@@ -116,7 +114,7 @@ namespace TypeUtils {
   /**
    * Returns all constructors
    * */
-  KDEVCPPDUCHAIN_EXPORT void getConstructors(const CppClassType::Ptr& klass, const TopDUContext* topContext, QList<Declaration*>& functions);
+  KDEVCPPDUCHAIN_EXPORT QList<Declaration*> getConstructors(const CppClassType::Ptr& klass, const TopDUContext* topContext);
   /**
    * Tries to return the internal context of a declaration, for example the internal context of a class can be found by calling this with the class'es declaration.
    **/
@@ -141,6 +139,9 @@ namespace TypeUtils {
   
   ///Transforms from constants like '5' to their underlying types like 'int' (Or from enumerators to their enumeration-types)
   KDEVCPPDUCHAIN_EXPORT AbstractType::Ptr removeConstants(KDevelop::AbstractType::Ptr type, const KDevelop::TopDUContext* source);
+
+  ///Check whether the given type resembles a varargs ellipsis.
+  KDEVCPPDUCHAIN_EXPORT bool isVarArgs(const KDevelop::AbstractType::Ptr &type);
 }
 
 #endif
