@@ -33,8 +33,6 @@
 #include <QReadWriteLock>
 #include <kactioncollection.h>
 #include <kaction.h>
-#include <QExtensionFactory>
-#include <QtDesigner/QExtensionFactory>
 
 #include <kdebug.h>
 #include <kcomponentdata.h>
@@ -49,17 +47,22 @@
 #include <language/codecompletion/codecompletion.h>
 
 #include <interfaces/icore.h>
-#include <language/interfaces/iproblem.h>
+#include <language/duchain/problem.h>
 #include <interfaces/iproject.h>
 #include <interfaces/idocument.h>
 #include <interfaces/idocumentcontroller.h>
 #include <interfaces/ilanguage.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/iprojectcontroller.h>
+#include <interfaces/isourceformattercontroller.h>
+#include <interfaces/isourceformatter.h>
 #include <project/interfaces/ibuildsystemmanager.h>
 #include <language/interfaces/iquickopen.h>
 #include <interfaces/iplugincontroller.h>
+#include <language/interfaces/editorcontext.h>
 #include <project/projectmodel.h>
+#include <language/assistant/renameassistant.h>
+#include <language/assistant/staticassistantsmanager.h>
 #include <language/backgroundparser/backgroundparser.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/duchainutils.h>
@@ -68,10 +71,12 @@
 #include <language/duchain/topducontext.h>
 #include <language/duchain/functiondefinition.h>
 #include <language/codegen/coderepresentation.h>
+
 #include <interfaces/contextmenuextension.h>
 
 #include "preprocessjob.h"
 #include "rpp/preprocessor.h"
+#include "rpp/chartools.h"
 #include "ast.h"
 #include "parsesession.h"
 #include "cpphighlighting.h"
@@ -82,9 +87,10 @@
 #include "environmentmanager.h"
 #include "cppduchain/navigation/navigationwidget.h"
 #include "cppduchain/cppduchain.h"
-#include "codegen/codeassistant.h"
 #include <interfaces/foregroundlock.h>
 //#include "codegen/makeimplementationprivate.h"
+#include "codegen/adaptsignatureassistant.h"
+#include "codegen/unresolvedincludeassistant.h"
 
 #include "includepathresolver.h"
 #include "setuphelpers.h"
@@ -93,7 +99,6 @@
 #include "codegen/simplerefactoring.h"
 #include "codegen/cppclasshelper.h"
 #include "includepathcomputer.h"
-#include "codecompletion/missingincludemodel.h"
 
 //#include <valgrind/callgrind.h>
 
@@ -121,11 +126,36 @@ using namespace KDevelop;
 
 CppLanguageSupport* CppLanguageSupport::m_self = 0;
 
+namespace
+{
+void fillEditIncludeDirectoriesContextMenu(ContextMenuExtension& extension, KDevelop::Context* context)
+{
+    auto ec = dynamic_cast<KDevelop::EditorContext*>(context);
+    if (ec && ec->currentLine().contains(QRegExp("^\\s*#include"))) {
+        KDevelop::IAssistantAction::Ptr assistantAction;
+        if (auto project = ICore::self()->projectController()->findProjectForUrl(ec->url())) {
+            assistantAction.attach(new Cpp::OpenProjectConfigurationAction(project));
+        } else {
+            assistantAction.attach(new Cpp::AddCustomIncludePathAction(IndexedString(ec->url()), QString()));
+        }
+        auto action = assistantAction->toKAction();
+        action->setText(i18n("Edit include directories"));
+        extension.addAction(extension.ExtensionGroup, action);
+    }
+}
+}
+
 KDevelop::ContextMenuExtension CppLanguageSupport::contextMenuExtension(KDevelop::Context* context)
 {
-    ContextMenuExtension cm;
-    SimpleRefactoring::self().doContextMenu(cm, context);
-    return cm;
+  ContextMenuExtension cm;
+  EditorContext *ec = dynamic_cast<KDevelop::EditorContext *>(context);
+
+  if (ec && ICore::self()->languageController()->languagesForUrl(ec->url()).contains(language())) {
+    // It's a C++ file, let's add our context menu.
+    m_refactoring->fillContextMenu(cm, context);
+    fillEditIncludeDirectoriesContextMenu(cm, context);
+  }
+  return cm;
 }
 
 ///Tries to find a definition for the declaration at given cursor-position and document-url. DUChain must be locked.
@@ -164,7 +194,6 @@ static QStringList mimeTypesList()
 CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*args*/ )
     : KDevelop::IPlugin( KDevCppSupportFactory::componentData(), parent ),
       KDevelop::ILanguageSupport(),
-      m_standardMacros(0),
       m_mimeTypes(mimeTypesList())
 {
     m_self = this;
@@ -173,8 +202,8 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     setXMLFile( "kdevcppsupport.rc" );
 
     m_highlights = new CppHighlighting( this );
+    m_refactoring = new SimpleRefactoring(this);
     m_cc = new KDevelop::CodeCompletion( this, new Cpp::CodeCompletionModel(0), name() );
-    m_missingIncludeCompletion = new KDevelop::CodeCompletion( this, new Cpp::MissingIncludeCompletionModel(0), name() );
 
     Cpp::EnvironmentManager::init();
     Cpp::EnvironmentManager::self()->setSimplifiedMatching(true);
@@ -183,9 +212,6 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
 //     Cpp::EnvironmentManager::self()->setMatchingLevel(Cpp::EnvironmentManager::Full);
 
     CppUtils::standardMacros();
-    CppUtils::standardIncludePaths();
-
-    m_includeResolver = new CppTools::IncludePathResolver;
 
     m_quickOpenDataProvider = new IncludeFileDataProvider();
 
@@ -196,10 +222,11 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     // else we are in NoUi mode (duchainify, unit tests, ...) and hence cannot find the Quickopen plugin
 
 #ifdef DEBUG_UI_LOCKUP
-    m_blockTester = new UIBlockTester(LOCKUP_INTERVAL);
+    new UIBlockTester(LOCKUP_INTERVAL, this);
 #endif
 
-    m_assistant = new Cpp::StaticCodeAssistant;
+    core()->languageController()->staticAssistantsManager()->registerAssistant(StaticAssistant::Ptr(new RenameAssistant(this)));
+    core()->languageController()->staticAssistantsManager()->registerAssistant(StaticAssistant::Ptr(new Cpp::AdaptSignatureAssistant(this)));
 
     foreach(QString mimeType, m_mimeTypes){
         KDevelop::IBuddyDocumentFinder::addFinder(mimeType,this);
@@ -224,12 +251,12 @@ void CppLanguageSupport::createActionsForMainWindow (Sublime::MainWindow* /*wind
     renameDeclarationAction->setText( i18n("Rename Declaration") );
     renameDeclarationAction->setIcon(KIcon("edit-rename"));
     renameDeclarationAction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_R);
-    connect(renameDeclarationAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executeRenameAction()));
+    connect(renameDeclarationAction, SIGNAL(triggered(bool)), m_refactoring, SLOT(executeRenameAction()));
 
     KAction* moveIntoSourceAction = actions.addAction("code_move_definition");
     moveIntoSourceAction->setText( i18n("Move into Source") );
     moveIntoSourceAction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_S);
-    connect(moveIntoSourceAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executeMoveIntoSourceAction()));
+    connect(moveIntoSourceAction, SIGNAL(triggered(bool)), m_refactoring, SLOT(executeMoveIntoSourceAction()));
 }
 
 void CppLanguageSupport::switchDefinitionDeclaration()
@@ -254,9 +281,9 @@ void CppLanguageSupport::switchDefinitionDeclaration()
     cursor = SimpleCursor(doc->textDocument()->activeView()->cursorPosition()); 
   }
   
-  KUrl switchCandidate = CppUtils::sourceOrHeaderCandidate(docUrl);
+  const QString switchCandidate = CppUtils::sourceOrHeaderCandidate(docUrl.toLocalFile());
   
-  if(switchCandidate.isValid())
+  if(!switchCandidate.isEmpty())
   {
     
     DUChainReadLocker lock;
@@ -268,7 +295,11 @@ void CppLanguageSupport::switchDefinitionDeclaration()
     {
       lock.unlock();
       kDebug(9007) << "Parsing switch-candidate before switching" << switchCandidate;
-      DUChain::self()->waitForUpdate(IndexedString(switchCandidate), TopDUContext::VisibleDeclarationsAndContexts);
+      ReferencedTopDUContext updatedContext = DUChain::self()->waitForUpdate(IndexedString(switchCandidate), TopDUContext::VisibleDeclarationsAndContexts);
+      if (!updatedContext) {
+        kDebug(9007) << "Failed to update document:" << switchCandidate;
+        return;
+      }
     }
   }
   
@@ -375,8 +406,8 @@ void CppLanguageSupport::switchDefinitionDeclaration()
   lock.unlock();
   ///- If no definition/declaration could be found to switch to, just switch the document using normal header/source heuristic by file-extension
 
-  if(switchCandidate.isValid()) {
-    core()->documentController()->openDocument(switchCandidate);
+  if(!switchCandidate.isEmpty()) {
+    core()->documentController()->openDocument(KUrl(switchCandidate));
   }else{
     kDebug(9007) << "Found no source/header candidate to switch";
   }
@@ -397,12 +428,9 @@ CppLanguageSupport::~CppLanguageSupport()
     // Remove any documents waiting to be parsed from the background parser.
     core()->languageController()->backgroundParser()->clear(this);
 
-
-    delete m_includeResolver;
 #ifdef DEBUG_UI_LOCKUP
     delete m_blockTester;
 #endif
-    delete m_assistant;
 
     foreach(QString mimeType, m_mimeTypes){
         KDevelop::IBuddyDocumentFinder::removeFinder(mimeType);
@@ -423,6 +451,11 @@ KDevelop::ICodeHighlighting *CppLanguageSupport::codeHighlighting() const
     return m_highlights;
 }
 
+BasicRefactoring* CppLanguageSupport::refactoring() const
+{
+    return m_refactoring;
+}
+
 ICreateClassHelper* CppLanguageSupport::createClassHelper() const
 {
     return new CppClassHelper;
@@ -431,7 +464,7 @@ ICreateClassHelper* CppLanguageSupport::createClassHelper() const
 
 void CppLanguageSupport::findIncludePathsForJob(CPPParseJob* job)
 {
-  IncludePathComputer* comp = new IncludePathComputer(KUrl(job->document().str()), job->preprocessorProblemsPointer());
+  IncludePathComputer* comp = new IncludePathComputer(job->document().str());
   comp->computeForeground();
   job->gotIncludePaths(comp);
 }
@@ -792,11 +825,22 @@ QWidget* CppLanguageSupport::specialLanguageObjectNavigationWidget(const KUrl& u
     QString preprocessedBody;
     //Check whether tail contains arguments
     QString tail = found.second.trimmed(); ///@todo make this better.
-    if(tail.startsWith("(")) {
-      int i = findClose( tail, 0 );
-      if(i != -1) {
-        text += tail.left(i+1);
+    if(isFunctionLike(m.second, url)) {
+     if(tail.endsWith('\\'))
+       tail.truncate(tail.length() - 1);
+      //properly support macro expansions when arguments contain newlines
+      int foundClosingBrace = findClose( tail, 0 );
+      KDevelop::IDocument* doc = core()->documentController()->documentForUrl(url);
+      if(doc && doc->textDocument() && doc->textDocument()->activeView() && foundClosingBrace < 0) {
+        const int lines = doc->textDocument()->lines();
+        for (int lineNumber = position.line + 1; foundClosingBrace < 0 && lineNumber < lines; lineNumber++) {
+          tail += doc->textDocument()->line(lineNumber).trimmed();
+          if(tail.endsWith('\\'))
+            tail.truncate(tail.length() - 1);
+          foundClosingBrace = findClose( tail, 0 );
+        }
       }
+      text += tail.left(foundClosingBrace + 1);
     }
 
     {
@@ -811,9 +855,71 @@ QWidget* CppLanguageSupport::specialLanguageObjectNavigationWidget(const KUrl& u
       }
     }
 
+    KMimeType::Ptr mime = KMimeType::mimeType("text/x-c++hdr");
+    ISourceFormatter* i = core()->sourceFormatterController()->formatterForMimeType(mime);
+    if(i){
+      SourceFormatterStyle style = core()->sourceFormatterController()->styleForMimeType(mime);
+      preprocessedBody = i->formatSourceWithStyle(style, preprocessedBody, KUrl(), mime);
+    }
+
     return new Cpp::NavigationWidget(*m.second, preprocessedBody);
 }
 
+bool CppLanguageSupport::isFunctionLike(const rpp::pp_macro* pm, const KUrl& url)
+{
+  if(pm->function_like)
+    return true;
+
+  DUChainReadLocker lock(DUChain::lock(), 100);
+  if(!lock.locked()) {
+    kDebug(9007) << "Failed to lock the du-chain in time";
+    return false;
+  }
+
+  TopDUContext* ctx = standardContext(url, true);
+  if(!ctx || !ctx->parsingEnvironmentFile())
+    return false;
+
+  Cpp::EnvironmentFilePointer p(dynamic_cast<Cpp::EnvironmentFile*>(ctx->parsingEnvironmentFile().data()));
+
+  Q_ASSERT(p);
+  QByteArray strdef = stringFromContents((uint*)pm->definition(), pm->definitionSize()).trimmed();
+
+  Cpp::ReferenceCountedMacroSet::Iterator it = p->usedMacros().iterator();
+
+  while(!strdef.isEmpty()) {
+    IndexedString preName(strdef);
+
+    if(pm->name == preName)
+      return false;
+
+    if(!p->usedMacroNames().contains(preName) && !p->definedMacroNames().contains(preName))
+      return false;
+
+    while(it) {
+      if(it.ref().name.byteArray() == strdef) {
+        if(it.ref().function_like)
+          return true;
+        strdef = stringFromContents((uint*)it.ref().definition(), it.ref().definitionSize());
+      }
+      ++it;
+    }
+
+    it = p->definedMacros().iterator();
+
+    while(it) {
+      if(it.ref().name.byteArray() == strdef) {
+        if(it.ref().function_like)
+          return true;
+        strdef = stringFromContents((uint*)it.ref().definition(), it.ref().definitionSize());
+      }
+      ++it;
+    }
+  };
+
+  return false;
+}
+
 UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent ) : QThread(), m_parent( parent ), m_stop(false) {
 }
 
@@ -835,7 +941,11 @@ UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent )
          m_stop = true;
  }
 
- UIBlockTester::UIBlockTester( uint milliseconds ) : m_thread( *this ), m_msecs( milliseconds ) {
+UIBlockTester::UIBlockTester( uint milliseconds, QObject* parent )
+  : QObject(parent)
+  , m_thread( *this )
+  , m_msecs( milliseconds )
+{
          m_timer = new QTimer( this );
          m_timer->start( milliseconds/10 );
          connect( m_timer, SIGNAL(timeout()), this, SLOT(timer()) );
