@@ -31,20 +31,32 @@ class QString;
 class QByteArray;
 typedef QVector<unsigned int> PreprocessedContents;
 
+/**
+ * Attempt to fix bug 328285
+ *
+ * This is more or less a work-around to make rpp a bit more robust against
+ * non-7bit ASCII chars.
+ *
+ * See explanation at: http://stackoverflow.com/a/6693878
+ */
 inline bool isSpace(char c) {
-  return std::isspace(c);
+#ifdef _MSC_VER
+  return isspace(static_cast<uchar>(c));
+#else
+  return std::isspace(static_cast<uchar>(c));
+#endif
 }
 
 inline bool isLetter(char c) {
-  return std::isalpha(c);
+  return std::isalpha(static_cast<uchar>(c));
 }
 
 inline bool isLetterOrNumber(char c) {
-  return std::isalnum(c);
+  return std::isalnum(static_cast<uchar>(c));
 }
 
 inline bool isNumber(char c) {
-  return std::isdigit(c);
+  return std::isdigit(static_cast<uchar>(c));
 }
 
 //Takes an index as delt with during preprocessing, and determines whether it is a fake-index that represents
@@ -79,6 +91,18 @@ inline bool isNumber(unsigned int c) {
   return isCharacter(c) && isNumber(characterFromIndex(c));
 }
 
+/**
+ * Whether input @p c is a valid macro identifier character
+ *
+ * In theory the standard only allows 'a-z', 'A-Z', '0-9', and '_',
+ * however, GCC + MSVC accept '$' as well
+ *
+ * See http://stackoverflow.com/a/369524
+ */
+inline bool isValidMacroIdentifierToken(char c) {
+  return isLetterOrNumber(c) || c == '_' || c == '$';
+}
+
 ///Opposite of convertFromByteArray
 KDEVCPPRPP_EXPORT QByteArray stringFromContents(const PreprocessedContents& contents, int offset = 0, int count = 0);
 
