@@ -42,7 +42,7 @@ using namespace rpp;
 #define RETURN_ON_FAIL(x) if(!(x)) { ++input; kDebug(9007) << "Preprocessor: Condition not satisfied"; return; }
 
 pp::pp(Preprocessor* preprocessor)
-  : m_environment(new Environment(this))
+  : m_environment(new Environment)
   , expand(this, 0, true)
   , m_preprocessor(preprocessor)
   , nextToken(0)
@@ -315,7 +315,7 @@ void pp::handle_define (Stream& input)
   macro->sourceLine = input.originalInputPosition().line;
 
   skip_blanks (input, devnull());
-  macro->name = KDevelop::IndexedString::fromIndex(skip_identifier(input)); //@todo make macros utf8 too
+  macro->name = KDevelop::IndexedString::fromIndex(skip_identifier(input, true)); //@todo make macros utf8 too
 
   if (!input.atEnd() && input == '(')
   {
@@ -585,7 +585,7 @@ Value pp::eval_multiplicative(Stream& input)
       result *= value;
 
     } else if (token == '/') {
-      if (value.is_zero()) {
+      if (value.is_zero() || (!value.is_ulong() && !result.is_ulong() && value.l == -1 && result.l == LLONG_MIN)) {
         KDevelop::ProblemPointer problem(new KDevelop::Problem);
         problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(input.originalInputPosition().castToSimpleCursor(), 0)));
         problem->setDescription(i18n("Division by zero"));
@@ -598,7 +598,7 @@ Value pp::eval_multiplicative(Stream& input)
       }
 
     } else {
-      if (value.is_zero()) {
+      if (value.is_zero() || (!value.is_ulong() && !result.is_ulong() && value.l == -1 && result.l == LLONG_MIN)) {
         KDevelop::ProblemPointer problem(new KDevelop::Problem);
         problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(input.originalInputPosition().castToSimpleCursor(), 0)));
         problem->setDescription(i18n("Division by zero"));
@@ -860,7 +860,7 @@ void pp::handle_if (Stream& input)
 {
   if (test_if_level())
   {
-    pp_macro_expander expand_condition(this);
+    pp_macro_expander expand_condition(this, 0, false, true);
     skip_blanks(input, devnull());
 
     Anchor inputPosition = input.inputPosition();
@@ -880,7 +880,7 @@ void pp::handle_if (Stream& input)
 
   } else {
     // Capture info for precompiled macros
-    pp_macro_expander expand_condition(this);
+    pp_macro_expander expand_condition(this, 0, false, true);
     skip_blanks(input, devnull());
     PreprocessedContents condition;
     {
@@ -933,7 +933,7 @@ void pp::handle_elif(Stream& input)
   }
   else
   {
-    pp_macro_expander expand_condition(this);
+    pp_macro_expander expand_condition(this, 0, false, true);
     skip_blanks(input, devnull());
 
     Anchor inputPosition = input.inputPosition();
@@ -1011,7 +1011,7 @@ void pp::handle_ifdef (bool check_undefined, Stream& input)
   {
     pp_macro* macro = m_environment->retrieveMacro(macro_name, true);
     bool value = false;
-    if( macro && macro->defined )
+    if( macro && macro->defined && !(macro->file == currentFileName() && macro->sourceLine > input.originalInputPosition().line))
       value = true;
 
     if (check_undefined)
