@@ -28,7 +28,7 @@
 #include <kdebug.h>
 #include <klocale.h> 
 
-#include <language/interfaces/iproblem.h>
+#include <language/duchain/problem.h>
 #include <language/duchain/indexedstring.h>
 
 #include "pp-internal.h"
@@ -86,7 +86,7 @@ pp_frame::pp_frame(pp_macro* __expandingMacro, const QList<pp_actual>& __actuals
 {
 }
 
-pp_actual pp_macro_expander::resolve_formal(IndexedString name, Stream& input)
+pp_actual pp_macro_expander::resolve_formal(const IndexedString& name, Stream& input)
 {
   if (!m_frame)
     return pp_actual();
@@ -125,12 +125,14 @@ pp_actual pp_macro_expander::resolve_formal(IndexedString name, Stream& input)
 #define RETURN_IF_INPUT_BROKEN    if(input.atEnd()) { kDebug() << "too early end while expanding" << macro->name.str(); return; }
 
 
-pp_macro_expander::pp_macro_expander(pp* engine, pp_frame* frame, bool inHeaderSection)
+pp_macro_expander::pp_macro_expander(pp* engine, pp_frame* frame, bool inHeaderSection, bool has_if)
   : m_engine(engine)
   , m_frame(frame)
   , m_in_header_section(inHeaderSection)
   , m_search_significant_content(false)
   , m_found_significant_content(false)
+  , m_has_if(has_if)
+  , m_has_defined(false)
 {
   if(m_in_header_section)
     m_search_significant_content = true; //Find the end of the header section
@@ -193,7 +195,15 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
     {
       skip_comment_or_divop(input, output, true);
     }else{
-      if (input == '\n')
+      if (input == '\\' && input.peekNextCharacter() == '\n')
+      {
+        ++input;
+        ++input;
+        skip_blanks(input, output);
+        if (!input.atEnd() && input == '#')
+          break;
+      }
+      else if (input == '\n')
       {
         output << input;
 
@@ -219,7 +229,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
           IndexedString previous;
           if (output.offset() > 0) {
             previous = IndexedString::fromIndex(output.popLastOutput()); //Previous already has been expanded
-            while(output.offset() > 0 && isCharacter(previous.index()) && isSpace(characterFromIndex(previous.index())))
+            while(output.offset() > 0 && isSpace(previous.index()))
               previous = IndexedString::fromIndex(output.popLastOutput());   
           }
           output.appendString(output.currentOutputAnchor(), previous);
@@ -351,21 +361,33 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
         }
 
         // TODO handle inbuilt "defined" etc functions
+        static const IndexedString definedIndex = IndexedString("defined");
+
+        if (name == definedIndex && m_has_if && !m_has_defined){
+          m_has_defined = true;
+          continue;
+        }
+        if(m_has_if && m_has_defined){
+          pp_macro* macro = m_engine->environment()->retrieveMacro(name, true);
+          unsigned int value = 0;
+          if(macro && macro->defined)
+            value = 1;
+          output.appendString(inputPosition, convertFromByteArray(QByteArray::number(value)));
+          m_has_defined = false;
+          continue;
+        }
 
         pp_macro* macro = m_engine->environment()->retrieveMacro(name, false);
-        
+
         if (!macro || !macro->defined || macro->hidden || macro->function_like || m_engine->hideNextMacro())
         {
-          static const IndexedString definedIndex = IndexedString("defined");
-          m_engine->setHideNextMacro(name == definedIndex);
+          static const IndexedString lineIndex = IndexedString("__LINE__");
+          static const IndexedString fileIndex = IndexedString("__FILE__");
+          static const IndexedString dateIndex = IndexedString("__DATE__");
+          static const IndexedString timeIndex = IndexedString("__TIME__");
 
-
-        static const IndexedString lineIndex = IndexedString("__LINE__");
-        static const IndexedString fileIndex = IndexedString("__FILE__");
-        static const IndexedString dateIndex = IndexedString("__DATE__");
-        static const IndexedString timeIndex = IndexedString("__TIME__");
           if (name == lineIndex)
-            output.appendString(inputPosition, convertFromByteArray(QString::number(input.inputPosition().line).toUtf8()));
+            output.appendString(inputPosition, convertFromByteArray(QByteArray::number(input.inputPosition().line)));
           else if (name == fileIndex)
             output.appendString(inputPosition, convertFromByteArray(QString("\"%1\"").arg(m_engine->currentFileNameString()).toUtf8()));
           else if (name == dateIndex)
@@ -424,8 +446,8 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
         //Eventually execute a function-macro
           
         IndexedString previous = IndexedString::fromIndex(indexFromCharacter(' ')); //Previous already has been expanded
-        uint stepsBack = 0;
-        while(isCharacter(previous.index()) && isSpace(characterFromIndex(previous.index())) && output.peekLastOutput(stepsBack)) {
+        int stepsBack = 0;
+        while(isSpace(previous.index()) && output.peekLastOutput(stepsBack)) {
           previous = IndexedString::fromIndex(output.peekLastOutput(stepsBack));
           ++stepsBack;
         }
@@ -443,7 +465,20 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
         QList<pp_actual> actuals;
         ++input; // skip '('
         
-        RETURN_IF_INPUT_BROKEN
+        if(input.atEnd())
+        {
+          // If the input has ended too early, seek back, and flush the input into the output
+          input.seek(openingPosition);
+          input.setInputPosition(openingPositionCursor);
+          while(!input.atEnd())
+          {
+            output << input;
+            ++input;
+          }
+          
+          kDebug() << "too early end while expanding" << macro->name.str();
+          return;
+        }
 
         pp_macro_expander expand_actual(m_engine, m_frame);
         skip_actual_parameter(input, *macro, actuals, expand_actual);
@@ -452,7 +487,21 @@ void pp_macro_expander::operator()(Stream& input, Stream& output, bool substitut
         {
           ++input; // skip ','
           
-          RETURN_IF_INPUT_BROKEN
+          if(input.atEnd())
+          {
+            // If the input has ended too early, seek back, and flush the input into the output
+            input.seek(openingPosition);
+            input.setInputPosition(openingPositionCursor);
+            while(!input.atEnd())
+            {
+              output << input;
+              ++input;
+            }
+            
+            kDebug() << "too early end while expanding" << macro->name.str();
+            return;
+          }
+          
           skip_actual_parameter(input, *macro, actuals, expand_actual);
         }
 
