@@ -56,6 +56,7 @@ Stream::Stream()
   , m_locationTable(0L)
   , m_originalInputPosition(KDevelop::CursorInRevision::invalid())
 {
+  c = nullptr;
   end = 0;
 }
 
@@ -220,8 +221,8 @@ unsigned int rpp::Stream::popLastOutput() {
   return ret;
 }
 
-unsigned int rpp::Stream::peekLastOutput(uint backOffset) const {
-  if(m_pos - backOffset > 0)
+unsigned int rpp::Stream::peekLastOutput(int backOffset) const {
+  if(m_pos > backOffset)
     return m_string->at(m_pos - backOffset - 1);
   return 0;
 }
@@ -308,7 +309,7 @@ Stream& Stream::appendString( const Anchor& inputPosition, const PreprocessedCon
   return *this;
 }
 
-Stream& Stream::appendString( const Anchor& inputPosition, KDevelop::IndexedString string )
+Stream& Stream::appendString( const Anchor& inputPosition, const KDevelop::IndexedString& string )
 {
 
   if (!isNull()) {
