@@ -22,9 +22,6 @@
 
 #include "preprocessjob.h"
 
-//#include <valgrind/memcheck.h>
-
-
 #include <QFile>
 #include <QFileInfo>
 #include <QByteArray>
@@ -40,11 +37,13 @@
 #include <language/duchain/duchainutils.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/topducontext.h>
-#include <language/interfaces/iproblem.h>
+#include <language/duchain/problem.h>
 
 #include <threadweaver/Thread.h>
 
 #include <interfaces/ilanguage.h>
+#include <interfaces/icore.h>
+#include <interfaces/ilanguagecontroller.h>
 
 #include "cpplanguagesupport.h"
 #include "cppparsejob.h"
@@ -65,12 +64,13 @@
 
 const uint maxIncludeDepth = 50;
 
-QString urlsToString(const QList<KUrl>& urlList) {
-  QString paths;
-  foreach( const KUrl& u, urlList )
-      paths += u.pathOrUrl() + "\n";
-
-  return paths;
+static QString pathsToString(const Path::List& paths)
+{
+  QString str;
+  foreach( const Path& p, paths ) {
+    str += p.pathOrUrl() + "\n";
+  }
+  return str;
 }
 
 PreprocessJob::PreprocessJob(CPPParseJob * parent)
@@ -88,7 +88,7 @@ PreprocessJob::~PreprocessJob() {
 }
 
 KDevelop::ParsingEnvironment* PreprocessJob::createStandardEnvironment() {
-    CppPreprocessEnvironment* ret = new CppPreprocessEnvironment(0, Cpp::EnvironmentFilePointer());
+    CppPreprocessEnvironment* ret = new CppPreprocessEnvironment(Cpp::EnvironmentFilePointer());
     ret->merge( CppUtils::standardMacros() );
     
     return ret;
@@ -148,21 +148,17 @@ void PreprocessJob::run()
     m_pp = &preprocessor;
 
     //Eventually initialize the environment with the parent-environment to get its macros
-    m_currentEnvironment = new CppPreprocessEnvironment( &preprocessor, m_firstEnvironmentFile );
-
+    m_currentEnvironment = new CppPreprocessEnvironment( m_firstEnvironmentFile );
     //If we are included from another preprocessor, copy its macros
     if( parentJob()->parentPreprocessor() ) {
         m_currentEnvironment->swapMacros( parentJob()->parentPreprocessor()->m_currentEnvironment );
     } else {
         //Insert standard-macros
-        KDevelop::ParsingEnvironment* standardEnv = createStandardEnvironment();
-        parentJob()->mergeDefines(static_cast<CppPreprocessEnvironment&>(*standardEnv));
-        
-        m_currentEnvironment->swapMacros( dynamic_cast<CppPreprocessEnvironment*>(standardEnv) );
-        delete standardEnv;
+        m_currentEnvironment->merge( CppUtils::standardMacros() );
+        parentJob()->mergeDefines(static_cast<CppPreprocessEnvironment&>(*m_currentEnvironment));
     }
     
-    Cpp::ReferenceCountedStringSet macroNamesAtBeginning = m_currentEnvironment->macroNameSet();
+    const auto& macroNamesAtBeginning = m_currentEnvironment->macroNameSet();
     
     KDevelop::ParsingEnvironmentFilePointer updatingEnvironmentFile;
     
@@ -180,8 +176,7 @@ void PreprocessJob::run()
           bool fileFeaturesSatisfied = updatingEnvironmentFile->featuresSatisfied(parentJob()->minimumFeatures());
           bool slaveFeaturesSatisfied = updatingEnvironmentFile->featuresSatisfied(parentJob()->slaveMinimumFeatures());
           if(fileFeaturesSatisfied && slaveFeaturesSatisfied) {
-            KUrl localPath(parentJob()->document().toUrl());
-            localPath.setFileName(QString());
+            Path localPath = Path(parentJob()->document().str()).parent();
             Cpp::EnvironmentFile* cppEnv = dynamic_cast<Cpp::EnvironmentFile*>(updatingEnvironmentFile.data());
             Q_ASSERT(cppEnv);
             //When possible, we determine whether an update is needed without getting the include-paths, because that's very expensive
@@ -191,7 +186,7 @@ void PreprocessJob::run()
                   kDebug(9007) << updatingEnvironmentFile->url().str() << "has missing include:" << (*it).str();
                 
                 readLock.unlock();
-                KUrl::List includePaths = parentJob()->includePathUrls();
+                Path::List includePaths = parentJob()->includePathUrls();
                 readLock.lock();
                 
                 needsUpdate = CppUtils::needsUpdate(Cpp::EnvironmentFilePointer(cppEnv), localPath, includePaths);
@@ -209,7 +204,7 @@ void PreprocessJob::run()
     }
     
     //We do this down here, so we eventually can prevent determining the include-paths if nothing needs to be updated
-    m_firstEnvironmentFile->setIncludePaths( parentJob()->masterJob()->includePaths() );
+    m_firstEnvironmentFile->setIncludePaths( parentJob()->masterJob()->indexedIncludePaths() );
     
     if(m_secondEnvironmentFile)
       m_secondEnvironmentFile->setIncludePaths(m_firstEnvironmentFile->includePaths());
@@ -242,6 +237,15 @@ void PreprocessJob::run()
     
     preprocessor.setEnvironment( m_currentEnvironment );
 
+    // Process files in include paths as #include "file"
+    // this is analogous to -include command line option of clang/cc
+    for (auto i: parentJob()->masterJob()->indexedIncludePaths()) {
+        auto include = i.toUrl().toLocalFile();
+        QFileInfo info(include);
+        if (info.isFile() && CppUtils::isHeader(include)) {
+            sourceNeeded(include, IncludeLocal, -1, false);
+        }
+    }
     PreprocessedContents result = preprocessor.processFile(parentJob()->document().str(), m_contents);
 
     if(Cpp::EnvironmentManager::self()->matchingLevel() <= Cpp::EnvironmentManager::Naive && !m_headerSectionEnded && !m_firstEnvironmentFile->headerGuard().isEmpty()) {
@@ -263,7 +267,6 @@ void PreprocessJob::run()
     m_currentEnvironment->finishEnvironment(m_currentEnvironment->environmentFile() == m_updatingEnvironmentFile);
     
     foreach (KDevelop::ProblemPointer p, preprocessor.problems()) {
-      p->setLocationStack(parentJob()->includeStack());
       p->setSource(KDevelop::ProblemData::Preprocessor);
       parentJob()->addPreprocessorProblem(p);
     }
@@ -367,8 +370,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
             Q_ASSERT(m_updatingEnvironmentFile || contentEnvironment->identityOffset() == m_secondEnvironmentFile->identityOffset());            
 
             ///@todo think whether localPath is needed
-            KUrl localPath(parentJob()->document().str());
-            localPath.setFileName(QString());
+            Path localPath = Path(parentJob()->document().str()).parent();
             
             if(contentEnvironment->matchEnvironment(m_currentEnvironment) && !CppUtils::needsUpdate(contentEnvironment, localPath, parentJob()->includePathUrls()) && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(content)) && (content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->minimumFeatures()) && content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->slaveMinimumFeatures())) 
               && Cpp::EnvironmentManager::self()->matchingLevel() != Cpp::EnvironmentManager::Disabled) {
@@ -434,19 +436,23 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
     ifDebug( kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": searching for include" << fileName; )
 
-    KUrl localPath(parentJob()->document().str());
-    localPath.setFileName(QString());
-    QStack<DocumentCursor> includeStack = parentJob()->includeStack();
-
-    KUrl from;
+    Path from;
     if (skipCurrentPath)
       from = parentJob()->includedFromPath();
 
-    QPair<KUrl, KUrl> included = CppUtils::findInclude( parentJob()->includePathUrls(), localPath, fileName, type, from );
-    KUrl includedFile = included.first;
+    QPair<Path, Path> included;
+    QFileInfo info(fileName);
+    if (info.isFile() && info.isAbsolute()) {
+        //this is an auto included file i.e. there is no #include directive in the source file for it
+        included.first = Path(fileNameUrl.toLocalFile());
+        included.second = Path(fileNameUrl.directory());
+    } else {
+        included = CppUtils::findInclude(parentJob()->includePathUrls(), parentJob()->localPath(), fileName, type, from);
+    }
+
+    Path includedFile = included.first;
     if (includedFile.isValid()) {
-      
-        IndexedString indexedFile(includedFile);
+        const IndexedString indexedFile(includedFile.pathOrUrl());
         
         {
           //Prevent recursion that may cause a crash
@@ -457,7 +463,6 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
               p->setSource(KDevelop::ProblemData::Preprocessor);
               p->setDescription(i18n("File was included recursively from within itself: %1", fileName ));
               p->setFinalLocation(DocumentRange(parentJob()->document(), SimpleRange(sourceLine,0, sourceLine+1,0)));
-              p->setLocationStack(parentJob()->includeStack());
               parentJob()->addPreprocessorProblem(p);
               return 0;
             }
@@ -473,7 +478,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
         {
             KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
-            includedContext = KDevelop::DUChain::self()->chainForDocument(includedFile, m_currentEnvironment, (bool)m_secondEnvironmentFile);
+            includedContext = KDevelop::DUChain::self()->chainForDocument(indexedFile, m_currentEnvironment, (bool)m_secondEnvironmentFile);
             
             //Check if the same file _is_ one of the parents, and if it is, import it later on
             if(Cpp::EnvironmentManager::self()->matchingLevel() <= Cpp::EnvironmentManager::Naive) {
@@ -507,7 +512,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
             if(includedContext) {
               Cpp::EnvironmentFilePointer includedEnvironment(dynamic_cast<Cpp::EnvironmentFile*>(includedContext->parsingEnvironmentFile().data()));
               if( includedEnvironment ) {
-                updateNeeded = CppUtils::needsUpdate(includedEnvironment, localPath, parentJob()->includePathUrls());
+                updateNeeded = CppUtils::needsUpdate(includedEnvironment, parentJob()->localPath(), parentJob()->includePathUrls());
                 //The ForceUpdateRecursive flag is removed before checking for satisfied features, so we can prevent double-updating through "wasUpdated()" below (see *1)
                 updateNeeded |= !includedEnvironment->featuresSatisfied((TopDUContext::Features)(slaveMinimumFeatures & (~TopDUContext::ForceUpdateRecursive)));
                 //(*1) Do not update again if ForceUpdate is given and the context was already updated during this run
@@ -567,15 +572,12 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
             ///The second parameter is zero because we are in a background-thread and we here
             ///cannot create a slave of the foreground cpp-support-part.
-            CPPParseJob* slaveJob = new CPPParseJob(includedFile, this);
+            CPPParseJob* slaveJob = new CPPParseJob(indexedFile, parentJob()->cpp(), this);
             
             slaveJob->setMinimumFeatures(slaveMinimumFeatures);
 
             slaveJob->setIncludedFromPath(included.second);
 
-            includeStack.append(DocumentCursor(parentJob()->document(), SimpleCursor(sourceLine, 0)));
-            slaveJob->setIncludeStack(includeStack);
-
             slaveJob->parseForeground();
 
             // Add the included file.
@@ -599,14 +601,13 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
     
     } else {
         kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": include not found:" << fileName;
-        KDevelop::ProblemPointer p(new Problem()); ///@todo create special include-problem
+        Cpp::MissingIncludePathProblem::Ptr p(new Cpp::MissingIncludePathProblem); ///@todo create special include-problem
         p->setSource(KDevelop::ProblemData::Preprocessor);
         p->setDescription(i18n("Included file was not found: %1", fileName ));
-        p->setExplanation(i18n("Searched include path:\n%1", urlsToString(parentJob()->includePathUrls())));
+        p->setExplanation(i18n("Searched include path:\n%1", pathsToString(parentJob()->includePathUrls())));
         p->setFinalLocation(DocumentRange(parentJob()->document(), SimpleRange(sourceLine,0, sourceLine+1,0)));
-        p->setLocationStack(parentJob()->includeStack());
         p->setSolutionAssistant(KSharedPtr<KDevelop::IAssistant>(new Cpp::MissingIncludePathAssistant(parentJob()->masterJob()->document(), _fileName)));
-        parentJob()->addPreprocessorProblem(p);
+        parentJob()->addPreprocessorProblem(KSharedPtr<Problem>::staticCast(p));
 
         ///@todo respect all the specialties like starting search at a specific path
         ///Before doing that, model findInclude(..) exactly after the standard
@@ -649,14 +650,11 @@ bool PreprocessJob::checkAbort()
 bool PreprocessJob::readContents()
 {
   KDevelop::ProblemPointer p = parentJob()->readContents();
-  
   if(p)
   {
-    p->setLocationStack(parentJob()->includeStack());
     parentJob()->addPreprocessorProblem(p);
     return false;
   }
-  
   m_firstEnvironmentFile->setModificationRevision( parentJob()->contents().modification );
 
   m_contents = parentJob()->contents().contents;
