@@ -32,25 +32,6 @@ using namespace KDevelop;
 
 namespace CppTools {
 
-#ifdef _MSC_VER
-QStringList msvcSetupStandardIncludePaths();
-const QVector<rpp::pp_macro*>& msvcStandardMacros();
-#else
-QStringList gccSetupStandardIncludePaths();
-const QVector<rpp::pp_macro*>& gccStandardMacros();
-#endif
-
-QStringList setupStandardIncludePaths()
-{
-//TODO: this should happen depending on the actual compiler used for the target and not on the compiler used for kdevelop itself.
-#ifdef _MSC_VER
-    QStringList includePaths = msvcSetupStandardIncludePaths();
-#else
-    QStringList includePaths = gccSetupStandardIncludePaths();
-#endif
-    return includePaths;
-}
-
 void insertMacro(Cpp::ReferenceCountedMacroSet& macros, const rpp::pp_macro& macro)
 {
   macros.insert(macro);
@@ -62,6 +43,7 @@ Cpp::ReferenceCountedMacroSet setupStandardMacros()
     //Add some macros to be compatible with the gnu c++ compiler
     //Used in several headers like sys/time.h
     insertMacro( macros, rpp::pp_macro("__restrict") );
+    insertMacro( macros, rpp::pp_macro("__restrict__") );
     insertMacro( macros, rpp::pp_macro("__extension__") );
     //Add macros that allow special treatment when within kdevelop
     insertMacro( macros, rpp::pp_macro("IN_KDEVELOP_PARSER") );
@@ -173,13 +155,16 @@ Cpp::ReferenceCountedMacroSet setupStandardMacros()
       insertMacro( macros, m );
     }
 
-#ifdef _MSC_VER    
-    foreach(const rpp::pp_macro* macro, msvcStandardMacros())
-#else
-    foreach(const rpp::pp_macro* macro, gccStandardMacros())
-#endif
-      insertMacro(macros, *macro);
-    
+    {
+      // see: https://bugs.kde.org/show_bug.cgi?id=325882
+      rpp::pp_macro m("__final");
+      m.setDefinitionText( "final" );
+      insertMacro( macros, m );
+    }
+
+    // see: https://bugs.kde.org/show_bug.cgi?id=308556
+    insertMacro( macros, rpp::pp_macro("__VA_ARGS__") );
+
     return macros;
 }
 
