@@ -23,14 +23,6 @@
 #include "parsesession.h"
 #include "ast.h"
 #include "astutilities.h"
-//#include "codegen/cppnewclass.h"
-//#include "codegen/makeimplementationprivate.h"
-
-#include <language/backgroundparser/backgroundparser.h>
-#include <language/codegen/coderepresentation.h>
-#include <language/codegen/documentchangeset.h>
-#include <language/duchain/duchain.h>  
-#include <language/duchain/duchainlock.h>
 
 #include <interfaces/ilanguagecontroller.h>
 
@@ -39,25 +31,27 @@
 #include <tests/autotestshell.h>
 #include <tests/testcore.h>
 #include <tests/testfile.h>
-#include <tests/testproject.h>
 #include <qtest_kde.h>
 #include <language/codecompletion/codecompletiontesthelper.h>
 #include <dumpchain.h>
+#include <templatedeclaration.h>
+#include <language/backgroundparser/backgroundparser.h>
+#include <language/codegen/coderepresentation.h>
+#include <language/duchain/duchain.h>
+#include <language/duchain/duchainlock.h>
+#include <language/duchain/duchainutils.h>
 #include <language/duchain/dumpchain.h>
 #include <language/duchain/parsingenvironment.h>
 #include <language/duchain/codemodel.h>
-#include <typeinfo>
-#include <KTempDir>
 #include <language/duchain/classdeclaration.h>
-#include <cppducontext.h>
+#include <language/duchain/types/structuretype.h>
 #include <interfaces/iassistant.h>
 #include <interfaces/foregroundlock.h>
-#include <language/duchain/use.h>
-#include <language/duchain/duchainutils.h>
+#include <interfaces/isourceformattercontroller.h>
 
 #include "codegen/simplerefactoring.h"
 
-QTEST_KDEMAIN(TestCppCodegen, GUI )
+QTEST_KDEMAIN(TestCppCodegen, NoGUI )
 
 using namespace KDevelop;
 
@@ -66,11 +60,12 @@ ForegroundLock* globalTestLock = 0;
 void TestCppCodegen::initTestCase()
 {
   //Initialize KDevelop components
-  AutoTestShell::init();
-  Core::initialize(0, KDevelop::Core::NoUi);
-  
+  AutoTestShell::init(QStringList() << "kdevcppsupport");
+  TestCore::initialize(Core::NoUi);
+
   Core::self()->languageController()->backgroundParser()->setDelay(1);
-  
+  DUChain::self()->disablePersistentStorage();
+  Core::self()->sourceFormatterController()->disableSourceFormatting(true);
   CodeRepresentation::setDiskChangesForbidden(true);
   
   globalTestLock = new ForegroundLock;
@@ -78,7 +73,7 @@ void TestCppCodegen::initTestCase()
 
 void TestCppCodegen::cleanupTestCase()
 {
-  Core::self()->cleanup();
+  TestCore::shutdown();
   delete globalTestLock;
   globalTestLock = 0;
 }
@@ -114,16 +109,12 @@ void TestCppCodegen::testAssistants()
 
   DUChainReadLocker lock;
 
-  // There is one problem from the include-path resolver as it couldn't
-  // resolve include paths for the artificial code
-  // The second problem is the missing-declaration assistant problem
-  QCOMPARE(code->problems().size(), 2);
-  QVERIFY(code->problems()[1]->solutionAssistant());
-  QCOMPARE(code->problems()[1]->solutionAssistant()->actions().size(), numAssistants);
-  code->problems()[1]->solutionAssistant()->actions()[executeAssistant]->execute();
+  QCOMPARE(code->problems().size(), 1);
+  QVERIFY(code->problems().first()->solutionAssistant());
+  QCOMPARE(code->problems().first()->solutionAssistant()->actions().size(), numAssistants);
+  code->problems().first()->solutionAssistant()->actions()[executeAssistant]->execute();
 
   //Make sure the assistant has inserted the correct solution
-  kDebug() << code.m_insertedCode.text();
   QVERIFY(code.m_insertedCode.text().contains(insertionText));
 }
 
@@ -134,6 +125,8 @@ void TestCppCodegen::testAssistants_data()
   QTest::addColumn<int>("executeAssistant");
   QTest::addColumn<QString>("insertionText");
 
+  /// NOTE: THIS CODE MUST _NOT_ DEPEND ON A SPECIFIC FORMATTER CONFIGURATION!
+
   QTest::newRow("local") <<
     "enum Honk { Hank };\n"
     "void test() {\n"
@@ -148,11 +141,10 @@ void TestCppCodegen::testAssistants_data()
     "   val = Hank;\n"
     "  }\n"
     "};\n";
-  QTest::newRow("local_class") << inClass << 5 << 0 << "Honk val = Hank;";
-  // 1 is the label
-  QTest::newRow("private_class") << inClass << 5 << 2 << "private:\n    Honk val;";
-  QTest::newRow("protected_class") << inClass << 5 << 3 << "protected:\n    Honk val;";
-  QTest::newRow("public_class") << inClass << 5 << 4 << "}\n    Honk val;";
+  QTest::newRow("local_class") << inClass << 4 << 0 << "Honk val = Hank;";
+  QTest::newRow("private_class") << inClass << 4 << 1 << "private:\n  Honk val;";
+  QTest::newRow("protected_class") << inClass << 4 << 2 << "protected:\n  Honk val;";
+  QTest::newRow("public_class") << inClass << 4 << 3 << "}\n  Honk val;";
 
   QString inOtherClass =
     "class other {\n"
@@ -165,9 +157,9 @@ void TestCppCodegen::testAssistants_data()
     "  }\n"
     "};\n";
   QTest::newRow("public_other") << inOtherClass << 2 << 1
-    << "class other {\npublic:\n    void foo(int arg1, double arg2);";
+    << "class other {\npublic:\nvoid foo(int arg1, double arg2);";
   QTest::newRow("protected_other") << inOtherClass << 2 << 0
-    << "class other {\nprotected:\n    void foo(int arg1, double arg2);";
+    << "class other {\nprotected:\nvoid foo(int arg1, double arg2);";
 }
 
 void TestCppCodegen::testUpdateIndices()
@@ -646,83 +638,6 @@ void TestCppCodegen::testAstDuChainMapping()
   }
 }
 
-void TestCppCodegen::testClassGeneration()
-{
-  #if 0
-  TopDUContext * top = 0;
-  GET_CONTEXT("AbstractClass.h", top);
-  
-  
-  CppNewClass newClass;
-  newClass.identifier("GeneratedClass");
-  newClass.addBaseClass("AbstractClass");
-  newClass.setHeaderUrl(CodeRepresentation::artificialUrl("GeneratedClass.h"));
-  newClass.setImplementationUrl(CodeRepresentation::artificialUrl("GeneratedClass.cpp"));
-  
-  DocumentChangeSet changes;/* = newClass.generate();
-  changes.applyAllToTemp();
-  QCOMPARE(changes.tempNamesForAll().size(), 2);
-  parseFile(changes.tempNamesForAll()[0].second);
-  parseFile(changes.tempNamesForAll()[1].second);*/
-  #endif
-}
-
-void TestCppCodegen::testPrivateImplementation()
-{
-#if 0
-  TopDUContext * top = 0;
-  GET_CONTEXT("ClassA.h", top);
-  QVERIFY(top);
-  
-  MakeImplementationPrivate generator;
-  DocumentRange range;
-  
-  {
-    DUChainReadLocker lock(DUChain::lock());
-    generator.autoGenerate(top->localDeclarations()[0]->internalContext(), &range);
-  }
-  generator.setStructureName("ClassAPrivate");
-  generator.setPointerName("d");
-  
-  QVERIFY2(generator.execute(), generator.errorText().toAscii());
-  generator.documentChangeSet().setReplacementPolicy(DocumentChangeSet::StopOnFailedChange);
-  
-  //Formatting plugin does not like source code in a single line
-  generator.documentChangeSet().setFormatPolicy(DocumentChangeSet::NoAutoFormat);
-  DocumentChangeSet::ChangeResult result = generator.documentChangeSet().applyAllToTemp();
-  QVERIFY2(result, result.m_failureReason.toAscii());
-  kDebug() << "tempName: " << generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.h"))).str();
-  kDebug() << "Generated Text:" << createCodeRepresentation(generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.h"))))->text();
-  kDebug() << "GeneratedText:" << createCodeRepresentation(generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.cpp"))))->text();
-  
-  QVERIFY(generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.h"))) != IndexedString(CodeRepresentation::artificialUrl("ClassA.h")));
-  QVERIFY(generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.cpp"))) != IndexedString(CodeRepresentation::artificialUrl("ClassA.cpp")));
-  
-  IndexedString tempName1 = generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.h")));
-  IndexedString tempName2 = generator.documentChangeSet().tempNameForFile(IndexedString(CodeRepresentation::artificialUrl("ClassA.cpp")));
-  parseFile(tempName1);
-  parseFile(tempName2);
-  
-  TopDUContext * newHeader = m_contexts[tempName1].data();
-  TopDUContext * newImplementation = m_contexts[tempName2].data();
-  
-  QVERIFY(newHeader);
-  QVERIFY(newImplementation);
-  
-  DUChainReadLocker lock(DUChain::lock());
-  kDebug() << "HeaderProblems: ";
-  foreach(ProblemPointer p, newHeader->problems())
-    kDebug() << p->description();
-  kDebug() << "Implementation Problems: ";
-  foreach(ProblemPointer p, newImplementation->problems())
-    kDebug() << p->description();
-  
-  //There is a problem from include path resolver, we care about semantic problems
-  QCOMPARE(newHeader->problems().size(), 1);
-  QCOMPARE(newImplementation->problems().size(), 1);
-#endif
-}
-
 void TestCppCodegen::testMacroDeclarationOrder()
 {
   InsertIntoDUChain source("thefile.h",
@@ -751,11 +666,17 @@ void TestCppCodegen::testMoveIntoSource()
   QFETCH(QString, newImpl);
   QFETCH(QualifiedIdentifier, id);
 
+  // make sure there aren't any stale queued documents to be parsed
+  QCOMPARE(ICore::self()->languageController()->backgroundParser()->queuedCount(), 0);
+
   TestFile header(origHeader, "h");
   TestFile impl(origImpl, "cpp", &header);
 
   header.parse(KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
   QVERIFY(header.waitForParsed());
+  impl.parse(KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
+  QVERIFY(impl.waitForParsed());
+
   ReferencedTopDUContext refTop = header.topContext();
   QVERIFY(refTop);
 
@@ -769,11 +690,18 @@ void TestCppCodegen::testMoveIntoSource()
     QVERIFY(declaration.isValid());
   }
   CodeRepresentation::setDiskChangesForbidden(false);
-  QCOMPARE(SimpleRefactoring::self().moveIntoSource(declaration), QString());
+  SimpleRefactoring *refactoring = new SimpleRefactoring();
+  QCOMPARE(refactoring->moveIntoSource(declaration), QString());
+  delete refactoring;
   CodeRepresentation::setDiskChangesForbidden(true);
 
   QCOMPARE(header.fileContents(), newHeader);
   QCOMPARE(impl.fileContents(), newImpl);
+
+  // SimpleRefactoring::moveIntoSource re-adds the documents to the BackgroundParser
+  // make sure we don't leave any stale requests for the next tests in the BackgroundParser
+  ICore::self()->languageController()->backgroundParser()->revertAllRequests(0);
+  QCOMPARE(ICore::self()->languageController()->backgroundParser()->queuedCount(), 0);
 }
 
 void TestCppCodegen::testMoveIntoSource_data()
@@ -786,55 +714,57 @@ void TestCppCodegen::testMoveIntoSource_data()
 
   const QualifiedIdentifier fooId("::foo");
 
+  /// NOTE: THIS CODE MUST _NOT_ DEPEND ON A SPECIFIC FORMATTER CONFIGURATION!
+
   QTest::newRow("globalfunction") << QString("int foo()\n{\n    int i = 0;\n    return 0;\n}\n")
                                   << QString()
                                   << QString("int foo();\n")
-                                  << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                  << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}")
                                   << fooId;
 
   QTest::newRow("staticfunction") << QString("static int foo()\n{\n    int i = 0;\n    return 0;\n}\n")
                                   << QString()
                                   << QString("static int foo();\n")
-                                  << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                  << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}")
                                   << fooId;
 
   QTest::newRow("funcsameline") << QString("int foo() {\n    int i = 0;\n    return 0;\n}\n")
                                 << QString()
                                 << QString("int foo();\n")
-                                << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}\n")
+                                << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}")
                                 << fooId;
 
   QTest::newRow("func-comment") << QString("int foo()\n/* foobar */ {\n    int i = 0;\n    return 0;\n}\n")
                                 << QString()
                                 << QString("int foo()\n/* foobar */;\n")
                                 ///TODO: should the comment be moved as well?
-                                << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}\n")
+                                << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}")
                                 << fooId;
 
-  QTest::newRow("func-odd-spaces") << QString("int foo()\n/*asdf*/\n {\n    int i = 0;\n    return 0;\n}\n")
+  QTest::newRow("func-comment2") << QString("int foo()\n/*asdf*/\n{\n    int i = 0;\n    return 0;\n}\n")
                                 << QString()
                                 << QString("int foo()\n/*asdf*/;\n")
                                 ///TODO: should the comment be moved as well?
-                                << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}")
                                 << fooId;
 
   const QualifiedIdentifier aFooId("a::foo");
   QTest::newRow("class-method") << QString("class a {\n    int foo(){\n        return 0;\n    }\n};\n")
                                 << QString()
                                 << QString("class a {\n    int foo();\n};\n")
-                                << QString("\nint a::foo() {\n    return 0;\n}\n")
+                                << QString("\nint a::foo() {\n        return 0;\n    }")
                                 << aFooId;
 
   QTest::newRow("class-method-const") << QString("class a {\n    int foo() const\n    {\n        return 0;\n    }\n};\n")
                                 << QString()
                                 << QString("class a {\n    int foo() const;\n};\n")
-                                << QString("\nint a::foo() const\n{\n    return 0;\n}\n")
+                                << QString("\nint a::foo() const\n    {\n        return 0;\n    }")
                                 << aFooId;
 
   QTest::newRow("class-method-const-sameline") << QString("class a {\n    int foo() const{\n        return 0;\n    }\n};\n")
                                 << QString()
                                 << QString("class a {\n    int foo() const;\n};\n")
-                                << QString("\nint a::foo() const {\n    return 0;\n}\n")
+                                << QString("\nint a::foo() const {\n        return 0;\n    }")
                                 << aFooId;
 }
 
