@@ -33,15 +33,17 @@ class TestCppCodegen : public QObject
     
     // Tests
     void testAssistants();
+    void testAssistants_data();
     
     void testSimplifiedUpdating();
     void testUpdateIndices();
     
     void testAstDuChainMapping();
-    void testClassGeneration();
-    void testPrivateImplementation();
 
     void testMacroDeclarationOrder();
+
+    void testMoveIntoSource();
+    void testMoveIntoSource_data();
 };
 
 #endif // TEST_CPPCODEGEN_H
