@@ -44,14 +44,9 @@ MakeBuilderPreferences::MakeBuilderPreferences(QWidget* parent, const QVariantLi
     connect( m_prefsUi->makeBinary, SIGNAL(urlSelected(KUrl)), SLOT(changed()) );
     l->addWidget( w );
 
-    KDevelop::EnvironmentGroupList env( KGlobal::config() );
-    m_prefsUi->kcfg_environmentProfile->addItems( env.groups() );
     m_prefsUi->configureEnvironment->setSelectionWidget( m_prefsUi->kcfg_environmentProfile );
 
     addConfig( MakeBuilderSettings::self(), w );
-
-    load();
-
 }
 
 void MakeBuilderPreferences::load()
@@ -95,6 +90,7 @@ void MakeBuilderPreferences::defaults()
 
 MakeBuilderPreferences::~MakeBuilderPreferences()
 {
+    delete m_prefsUi;
 }
 
 QString MakeBuilderPreferences::standardMakeComannd()
