@@ -42,15 +42,16 @@
 
 using namespace KDevelop;
 
-MakeJob::MakeJob(MakeBuilder* builder, KDevelop::ProjectBaseItem* item,
+MakeJob::MakeJob(QObject* parent, KDevelop::ProjectBaseItem* item,
                  CommandType c,  const QStringList& overrideTargets,
                  const MakeVariables& variables )
-    : OutputExecuteJob(builder)
+    : OutputExecuteJob(parent)
     , m_idx(item->index())
     , m_command(c)
     , m_overrideTargets(overrideTargets)
     , m_variables(variables)
 {
+    Q_ASSERT(item && item->model() && m_idx.isValid() && this->item() == item);
     setCapabilities( Killable );
     setFilteringStrategy( OutputModel::CompilerFilter );
     setProperties( NeedWorkingDirectory | PortableMessages | DisplayStderr | IsBuilderHint );
@@ -114,7 +115,7 @@ KUrl MakeJob::workingDirectory() const
 
     KDevelop::IBuildSystemManager *bldMan = it->project()->buildSystemManager();
     if( bldMan )
-        return bldMan->buildDirectory( it ); // the correct build dir
+        return bldMan->buildDirectory( it ).toUrl(); // the correct build dir
     else
     {
         // Just build in-source, where the build directory equals the one with particular target/source.
@@ -122,7 +123,7 @@ KUrl MakeJob::workingDirectory() const
             switch( item->type() ) {
             case KDevelop::ProjectBaseItem::Folder:
             case KDevelop::ProjectBaseItem::BuildFolder:
-                return static_cast<KDevelop::ProjectFolderItem*>(item)->url();
+                return static_cast<KDevelop::ProjectFolderItem*>(item)->path().toUrl();
                 break;
             case KDevelop::ProjectBaseItem::Target:
             case KDevelop::ProjectBaseItem::File:
@@ -183,7 +184,7 @@ QStringList MakeJob::commandLine() const
         cmdline << "-k";
     }
 
-    int jobnumber = builderGroup.readEntry("Number Of Jobs", 1);
+    int jobnumber = builderGroup.readEntry("Number Of Jobs", 2);
     if(jobnumber>1) {
         QString jobNumberArg = QString("-j%1").arg(jobnumber);
         cmdline << jobNumberArg;
@@ -238,5 +239,5 @@ QString MakeJob::environmentProfile() const
         return QString();
     KSharedConfig::Ptr configPtr = it->project()->projectConfiguration();
     KConfigGroup builderGroup( configPtr, "MakeBuilder" );
-    return builderGroup.readEntry( "Default Make Environment Profile", "default" );
+    return builderGroup.readEntry( "Default Make Environment Profile", QString() );
 }
