@@ -37,6 +37,8 @@
 #include "icmakemanager.h"
 #include "cmakeprojectvisitor.h"
 
+class WaitAllJobs;
+class CMakeCommitChangesJob;
 class QFileSystemWatcher;
 struct CMakeProjectData;
 class QStandardItem;
@@ -55,6 +57,8 @@ namespace KDevelop
     class ProjectBaseItem;
     class ProjectFileItem;
     class ProjectTargetItem;
+    class ProjectFilterManager;
+    class IProjectFilter;
     class ParseJob;
     class ContextMenuExtension;
     class Context;
@@ -84,12 +88,12 @@ public:
     virtual Features features() const { return Features(Folders | Targets | Files ); }
 //     virtual KDevelop::IProject* project() const;
     virtual KDevelop::IProjectBuilder* builder() const;
-    virtual KUrl buildDirectory(KDevelop::ProjectBaseItem*) const;
-    virtual KUrl::List includeDirectories(KDevelop::ProjectBaseItem *) const;
+    virtual KDevelop::Path buildDirectory(KDevelop::ProjectBaseItem*) const;
+    virtual KDevelop::Path::List includeDirectories(KDevelop::ProjectBaseItem *) const;
     virtual QHash<QString, QString> defines(KDevelop::ProjectBaseItem *) const;
 
-    virtual KDevelop::ProjectFolderItem* addFolder( const KUrl& folder, KDevelop::ProjectFolderItem* parent );
-    virtual KDevelop::ProjectFileItem* addFile( const KUrl&, KDevelop::ProjectFolderItem* );
+    virtual KDevelop::ProjectFolderItem* addFolder( const KDevelop::Path& folder, KDevelop::ProjectFolderItem* parent );
+    virtual KDevelop::ProjectFileItem* addFile( const KDevelop::Path&, KDevelop::ProjectFolderItem* );
     virtual KDevelop::ProjectTargetItem* createTarget( const QString&, KDevelop::ProjectFolderItem* ) { return 0; }
     virtual bool addFilesToTarget( const QList<KDevelop::ProjectFileItem*> &files, KDevelop::ProjectTargetItem* target);
 
@@ -97,16 +101,17 @@ public:
     virtual bool removeFilesFromTargets( const QList<KDevelop::ProjectFileItem*> &files );
     virtual bool removeFilesAndFolders( const QList<KDevelop::ProjectBaseItem*> &items);
 
-    virtual bool renameFile(KDevelop::ProjectFileItem*, const KUrl&);
-    virtual bool renameFolder(KDevelop::ProjectFolderItem*, const KUrl&);
-    virtual bool moveFilesAndFolders( const QList< KDevelop::ProjectBaseItem* > &items, KDevelop::ProjectFolderItem *newParent );
-    virtual bool copyFilesAndFolders(const KUrl::List &items, KDevelop::ProjectFolderItem* newParent);
+    virtual bool renameFile(KDevelop::ProjectFileItem*, const KDevelop::Path&);
+    virtual bool renameFolder(KDevelop::ProjectFolderItem*, const KDevelop::Path&);
+    virtual bool moveFilesAndFolders(const QList< KDevelop::ProjectBaseItem* > &items, KDevelop::ProjectFolderItem *newParent);
+    virtual bool copyFilesAndFolders(const KDevelop::Path::List &items, KDevelop::ProjectFolderItem* newParent);
 
     QList<KDevelop::ProjectTargetItem*> targets() const;
     QList<KDevelop::ProjectTargetItem*> targets(KDevelop::ProjectFolderItem* folder) const;
 
     virtual QList<KDevelop::ProjectFolderItem*> parse( KDevelop::ProjectFolderItem* dom );
     virtual KDevelop::ProjectFolderItem* import( KDevelop::IProject *project );
+    virtual KJob* createImportJob(KDevelop::ProjectFolderItem* item);
     
     virtual bool reload(KDevelop::ProjectFolderItem*);
 
@@ -121,63 +126,47 @@ public:
     virtual KDevelop::ICodeHighlighting* codeHighlighting() const;
     virtual QWidget* specialLanguageObjectNavigationWidget(const KUrl& url, const KDevelop::SimpleCursor& position);
     
-    void deleteItemLater(KDevelop::ProjectBaseItem* item);
-    void deleteAllLater(const QList< KDevelop::ProjectBaseItem* >& items);
-public slots:
-    void cleanupItems();
+    void addPending(const KDevelop::Path& path, CMakeFolderItem* folder);
+    CMakeFolderItem* takePending(const KDevelop::Path& path);
+    void addWatcher(KDevelop::IProject* p, const QString& path);
     
+    CMakeProjectData projectData(KDevelop::IProject* project);
+
+    KDevelop::ProjectFilterManager* filterManager() const;
+
 signals:
-    void folderRenamed(const KUrl& oldFolder, KDevelop::ProjectFolderItem* newFolder);
-    void fileRenamed(const KUrl& oldFile, KDevelop::ProjectFileItem* newFile);
-    
+    void folderRenamed(const KDevelop::Path& oldFolder, KDevelop::ProjectFolderItem* newFolder);
+    void fileRenamed(const KDevelop::Path& oldFile, KDevelop::ProjectFileItem* newFile);
+
 private slots:
     void dirtyFile(const QString& file);
 
     void jumpToDeclaration();
     void projectClosing(KDevelop::IProject*);
-    void reimportDone(KJob* job);
     
-    void deletedWatched(const QString& directory);
     void directoryChanged(const QString& dir);
     void filesystemBuffererTimeout();
+    void importFinished(KJob* job);
 
-    void createTestSuites(const QList<Test>& testSuites, KDevelop::ProjectFolderItem* folder);
 private:
-    void addDeleteItem(KDevelop::ProjectBaseItem* item);
-    void reimport(CMakeFolderItem* fi);
-    CacheValues readCache(const KUrl &path) const;
-    bool isReloading(KDevelop::IProject* p);
-    bool isCorrectFolder(const KUrl& url, KDevelop::IProject* p) const;
-    void cleanupToDelete(KDevelop::IProject* p);
-    bool renameFileOrFolder(KDevelop::ProjectBaseItem *item, const KUrl &newUrl);
-    
-    QMutex m_reparsingMutex;
-    QMutex m_busyProjectsMutex;
-    QMutex m_dirWatchersMutex;
-    KDevelop::ReferencedTopDUContext initializeProject(CMakeFolderItem*);
-    
-    KDevelop::ReferencedTopDUContext includeScript(const QString& file, KDevelop::IProject * project, const QString& currentDir,
-                                                    KDevelop::ReferencedTopDUContext parent);
-    
-    void setTargetFiles(KDevelop::ProjectTargetItem* target, const KUrl::List& files);
-    void reloadFiles(KDevelop::ProjectFolderItem* item);
+    QStringList processGeneratorExpression(const QStringList& expr, KDevelop::IProject* project, KDevelop::ProjectTargetItem* target) const;
 
-    QMap<KDevelop::IProject*, CMakeProjectData> m_projectsData;
-    QMap<KDevelop::IProject*, QFileSystemWatcher*> m_watchers;
-    QMap<KUrl, CMakeFolderItem*> m_pending;
+    bool renameFileOrFolder(KDevelop::ProjectBaseItem *item, const KDevelop::Path &newUrl);
+    void realDirectoryChanged(const QString& dir);
+    void deletedWatchedDirectory(KDevelop::IProject* p, const KUrl& dir);
     
-    QSet<KDevelop::IProject*> m_busyProjects;
+    QHash<KDevelop::IProject*, CMakeProjectData*> m_projectsData;
+    QHash<KDevelop::IProject*, QFileSystemWatcher*> m_watchers;
+    QHash<KDevelop::Path, CMakeFolderItem*> m_pending;
     
     KDevelop::ICodeHighlighting *m_highlight;
     
     QList<KDevelop::ProjectBaseItem*> m_clickedItems;
-    QSet<QString> m_toDelete;
-    QHash<KUrl, KUrl> m_renamed;
-    QList<KDevelop::ProjectBaseItem*> m_cleanupItems;
+    QSet<KDevelop::ProjectBaseItem*> m_cleanupItems;
 
     QTimer* m_fileSystemChangeTimer;
     QSet<QString> m_fileSystemChangedBuffer;
-    void realDirectoryChanged(const QString& dir);
+    KDevelop::ProjectFilterManager* const m_filter;
 };
 
 #endif
