@@ -29,6 +29,7 @@
 #include <language/duchain/topducontext.h>
 #include <parser/cmakelistsparser.h>
 #include <parser/cmaketypes.h>
+#include <util/path.h>
 
 namespace KDevelop {
     class IProject;
@@ -53,34 +54,28 @@ class KDEVCMAKECOMMON_EXPORT DUChainAttatched
     public:
         // Required, and must be non-inline, for dynamic_cast to work
         virtual ~DUChainAttatched();
-        DUChainAttatched(KDevelop::IndexedDeclaration _decl) : decl(_decl) {}
+        DUChainAttatched() {}
+        DUChainAttatched(const KDevelop::IndexedDeclaration& _decl) : decl(_decl) {}
         KDevelop::IndexedDeclaration declaration() const { return decl; }
+        void setDeclaration(KDevelop::IndexedDeclaration declaration) { decl = declaration; }
     private:
         KDevelop::IndexedDeclaration decl;
 };
 
-class KDEVCMAKECOMMON_EXPORT DefinesAttached
+class KDEVCMAKECOMMON_EXPORT CompilationDataAttached
 {
     public:
         // Required, and must be non-inline, for dynamic_cast to work
-        virtual ~DefinesAttached();
+        virtual ~CompilationDataAttached();
+        void setIncludeDirectories(const QStringList &l);
+        QStringList includeDirectories(KDevelop::ProjectBaseItem* placeInHierarchy) const;
+
         CMakeDefinitions definitions(CMakeFolderItem* parent) const;
         void setDefinitions(const CMakeDefinitions& defs) { m_defines=defs; }
-        void addDefinitions(const CMakeDefinitions& defs) { m_defines.unite(defs); }
-        void defineVariables(const QStringList& vars);
+        void addDefinitions(const QStringList& vars);
+
     private:
         CMakeDefinitions m_defines;
-};
-
-class KDEVCMAKECOMMON_EXPORT IncludesAttached
-{
-    public:
-        // Required, and must be non-inline, for dynamic_cast to work
-        virtual ~IncludesAttached();
-        void setIncludeDirectories(const QStringList &l) { m_includeList=l; }
-        void addIncludeDirectories(const QStringList& l) { m_includeList+=l; }
-        QStringList includeDirectories(KDevelop::ProjectBaseItem* placeInHierarchy) const;
-    private:
         QStringList m_includeList;
 };
 
@@ -93,10 +88,10 @@ class KDEVCMAKECOMMON_EXPORT IncludesAttached
 
 class KDEVCMAKECOMMON_EXPORT CMakeFolderItem
     : public KDevelop::ProjectBuildFolderItem
-    , public DescriptorAttatched, public DefinesAttached, public IncludesAttached
+    , public DescriptorAttatched, public CompilationDataAttached
 {
     public:
-        CMakeFolderItem( KDevelop::IProject* project, const KUrl& folder, const QString& build, CMakeFolderItem* item);
+        CMakeFolderItem( KDevelop::IProject* project, const KDevelop::Path& path, const QString& build, CMakeFolderItem* item);
         // Required, and must be non-inline, for dynamic_cast to work
         virtual ~CMakeFolderItem();
         
@@ -114,8 +109,6 @@ class KDEVCMAKECOMMON_EXPORT CMakeFolderItem
         
         KDevelop::ProjectTargetItem* targetNamed(Target::Type type, const QString& targetName) const;
         KDevelop::ProjectFolderItem* folderNamed(const QString& name) const;
-        QList<ProjectBaseItem*> cleanupBuildFolders(const QList<Subdirectory>& subs);
-        QList<ProjectBaseItem*> cleanupTargets(const QList<CMakeTarget>& targets);
     private:
         KDevelop::ReferencedTopDUContext m_topcontext;
         CMakeFolderItem* m_formerParent;
@@ -124,29 +117,28 @@ class KDEVCMAKECOMMON_EXPORT CMakeFolderItem
 
 class KDEVCMAKECOMMON_EXPORT CMakeExecutableTargetItem 
     : public KDevelop::ProjectExecutableTargetItem
-    , public DUChainAttatched, public DescriptorAttatched, public DefinesAttached, public IncludesAttached
+    , public DUChainAttatched, public DescriptorAttatched, public CompilationDataAttached
 {
     public:
         CMakeExecutableTargetItem(KDevelop::IProject* project, const QString &name,
-                                  CMakeFolderItem *parent, KDevelop::IndexedDeclaration c,
-                                  const QString& _outputName, const KUrl& basepath);
+                                  CMakeFolderItem *parent, const QString& _outputName, const KDevelop::Path& basepath);
         
         virtual KUrl builtUrl() const;
         virtual KUrl installedUrl() const;
         
     private:
         QString outputName;
-        KUrl path;
+        KDevelop::Path path;
 };
 
 class KDEVCMAKECOMMON_EXPORT CMakeLibraryTargetItem
     : public KDevelop::ProjectLibraryTargetItem
-    , public DUChainAttatched, public DescriptorAttatched, public DefinesAttached, public IncludesAttached
+    , public DUChainAttatched, public DescriptorAttatched, public CompilationDataAttached
 {
     public:
         CMakeLibraryTargetItem(KDevelop::IProject* project, const QString &name,
-                               CMakeFolderItem *parent, KDevelop::IndexedDeclaration c,
-                               const QString& _outputName, const KUrl& /*basepath*/);
+                               CMakeFolderItem *parent,
+                               const QString& _outputName, const KDevelop::Path& /*basepath*/);
             
     private:
         QString outputName;
@@ -157,8 +149,8 @@ class KDEVCMAKECOMMON_EXPORT CMakeCustomTargetItem
 {
     public:
         CMakeCustomTargetItem(KDevelop::IProject* project, const QString &name,
-                               CMakeFolderItem *parent, KDevelop::IndexedDeclaration c, const QString& _outputName)
-            : KDevelop::ProjectTargetItem( project, name, parent), DUChainAttatched(c), outputName(_outputName) {}
+                               CMakeFolderItem *parent, const QString& _outputName)
+            : KDevelop::ProjectTargetItem( project, name, parent), outputName(_outputName) {}
             
     private:
         QString outputName;
