@@ -46,33 +46,32 @@ namespace Config
 {
 namespace Old
 {
-static QString currentBuildDirKey = "CurrentBuildDir";
-static QString currentCMakeBinaryKey = "Current CMake Binary";
-static QString currentBuildTypeKey = "CurrentBuildType";
-static QString currentInstallDirKey = "CurrentInstallDir";
-static QString currentEnvironmentKey = "CurrentEnvironment";
-static QString currentExtraArgumentsKey = "Extra Arguments";
-static QString projectRootRelativeKey = "ProjectRootRelative";
-static QString projectBuildDirs = "BuildDirs";
-static QString cmakeDirectory = "CMakeDir";
+static const QString currentBuildDirKey = "CurrentBuildDir";
+static const QString currentCMakeBinaryKey = "Current CMake Binary";
+static const QString currentBuildTypeKey = "CurrentBuildType";
+static const QString currentInstallDirKey = "CurrentInstallDir";
+static const QString currentEnvironmentKey = "CurrentEnvironment";
+static const QString currentExtraArgumentsKey = "Extra Arguments";
+static const QString projectRootRelativeKey = "ProjectRootRelative";
+static const QString projectBuildDirs = "BuildDirs";
 }
 
-static QString buildDirIndexKey = "Current Build Directory Index";
-static QString buildDirOverrideIndexKey = "Temporary Build Directory Index";
-static QString buildDirCountKey = "Build Directory Count";
+static const QString buildDirIndexKey = "Current Build Directory Index";
+static const QString buildDirOverrideIndexKey = "Temporary Build Directory Index";
+static const QString buildDirCountKey = "Build Directory Count";
 
 namespace Specific
 {
-static QString buildDirPathKey = "Build Directory Path";
-static QString cmakeBinKey = "CMake Binary";
-static QString cmakeBuildTypeKey = "Build Type";
-static QString cmakeInstallDirKey = "Install Directory";
-static QString cmakeEnvironmentKey = "Environment Profile";
-static QString cmakeArgumentsKey = "Extra Arguments";
+static const QString buildDirPathKey = "Build Directory Path";
+static const QString cmakeBinKey = "CMake Binary";
+static const QString cmakeBuildTypeKey = "Build Type";
+static const QString cmakeInstallDirKey = "Install Directory";
+static const QString cmakeEnvironmentKey = "Environment Profile";
+static const QString cmakeArgumentsKey = "Extra Arguments";
 }
 
-static QString groupNameBuildDir = "CMake Build Directory %1";
-static QString groupName = "CMake";
+static const QString groupNameBuildDir = "CMake Build Directory %1";
+static const QString groupName = "CMake";
 
 } // namespace Config
 
@@ -81,6 +80,9 @@ namespace
 
 KConfigGroup baseGroup( KDevelop::IProject* project )
 {
+    if (!project)
+        return KConfigGroup();
+
     return project->projectConfiguration()->group( Config::groupName );
 }
 
@@ -122,7 +124,6 @@ void writeProjectParameter( KDevelop::IProject* project, const QString& key, con
     {
         KConfigGroup buildDirGrp = buildDirGroup( project, buildDirIndex );
         buildDirGrp.writeEntry( key, value );
-        buildDirGrp.sync();
     }
 
     else
@@ -135,7 +136,16 @@ void writeProjectBaseParameter( KDevelop::IProject* project, const QString& key,
 {
     KConfigGroup baseGrp = baseGroup(project);
     baseGrp.writeEntry( key, value );
-    baseGrp.sync();
+}
+
+inline QString replaceBuildDir(QString in, QString buildDir)
+{
+    return in.replace("#[bin_dir]", buildDir);
+}
+
+inline  QString replaceInstallDir(QString in, QString installDir)
+{
+    return in.replace("#[install_dir]", installDir);
 }
 
 } // namespace
@@ -143,17 +153,49 @@ void writeProjectBaseParameter( KDevelop::IProject* project, const QString& key,
 namespace CMake
 {
 
+KDevelop::Path::List resolveSystemDirs(KDevelop::IProject* project, const QStringList& dirs)
+{
+    const KDevelop::Path buildDir(CMake::currentBuildDir(project));
+    const KDevelop::Path installDir(CMake::currentInstallDir(project));
+
+    KDevelop::Path::List newList;
+    newList.reserve(dirs.size());
+    foreach(const QString& s, dirs)
+    {
+        KDevelop::Path dir;
+        if(s.startsWith(QString::fromUtf8("#[bin_dir]")))
+        {
+            dir = KDevelop::Path(buildDir, s);
+        }
+        else if(s.startsWith(QString::fromUtf8("#[install_dir]")))
+        {
+            dir = KDevelop::Path(installDir, s);
+        }
+        else
+        {
+            dir = KDevelop::Path(s);
+        }
+
+//         kDebug(9042) << "resolved" << s << "to" << d;
+
+        if (!newList.contains(dir))
+        {
+            newList.append(dir);
+        }
+    }
+    return newList;
+}
+
 ///NOTE: when you change this, update @c defaultConfigure in cmakemanagertest.cpp
-bool checkForNeedingConfigure( KDevelop::ProjectBaseItem* item )
+bool checkForNeedingConfigure( KDevelop::IProject* project )
 {
-    KDevelop::IProject* project = item->project();
     KUrl builddir = currentBuildDir(project);
     if( !builddir.isValid() )
     {
         CMakeBuildDirChooser bd;
         
-        KUrl folderUrl=item->project()->folder();
-        QString relative=CMake::projectRootRelative(item->project());
+        KUrl folderUrl=project->folder();
+        QString relative=CMake::projectRootRelative(project);
         folderUrl.cd(relative);
         
         bd.setSourceFolder( folderUrl );
@@ -199,9 +241,18 @@ bool checkForNeedingConfigure( KDevelop::ProjectBaseItem* item )
 
 KUrl projectRoot(KDevelop::IProject* project)
 {
+    if (!project) {
+        return KUrl();
+    }
+
     KUrl projectPath = project->folder();
-    bool correct=projectPath.cd(CMake::projectRootRelative(project));
+
+    // We need this as its own variable as .cd mutates the object and would be
+    // turned into a no-op by assert
+    const bool correct = projectPath.cd(CMake::projectRootRelative(project));
     Q_ASSERT(correct);
+    Q_UNUSED(correct);
+
     return projectPath;
 }
 
@@ -230,6 +281,11 @@ QString projectRootRelative( KDevelop::IProject* project )
     return baseGroup(project).readEntry( Config::Old::projectRootRelativeKey, "." );
 }
 
+bool hasProjectRootRelative(KDevelop::IProject* project)
+{
+    return baseGroup(project).hasKey( Config::Old::projectRootRelativeKey );
+}
+
 QString currentExtraArguments( KDevelop::IProject* project )
 {
     return readProjectParameter( project, Config::Specific::cmakeArgumentsKey, QString() );
@@ -287,16 +343,6 @@ void setCurrentBuildDirIndex( KDevelop::IProject* project, int buildDirIndex )
     writeProjectBaseParameter( project, Config::buildDirIndexKey, QString::number (buildDirIndex) );
 }
 
-KUrl cmakeDirectory( KDevelop::IProject* project )
-{
-    return baseGroup(project).readEntry( Config::Old::cmakeDirectory, QString() );
-}
-
-void setCmakeDirectory( KDevelop::IProject* project, const KUrl& url )
-{
-    writeProjectBaseParameter( project, Config::Old::cmakeDirectory, url.url() );
-}
-
 void setCurrentEnvironment( KDevelop::IProject* project, const QString& environment )
 {
     writeProjectParameter( project, Config::Specific::cmakeEnvironmentKey, environment );
@@ -346,8 +392,6 @@ void removeBuildDirConfig( KDevelop::IProject* project )
         src.copyTo(&dest);
         src.deleteGroup();
     }
-
-    project->projectConfiguration()->sync();
 }
 
 void updateConfig( KDevelop::IProject* project, int buildDirIndex, CMakeCacheModel* model )
@@ -374,7 +418,6 @@ void updateConfig( KDevelop::IProject* project, int buildDirIndex, CMakeCacheMod
     buildDirGrp.writeEntry( Config::Specific::cmakeBinKey, KUrl( model->value("CMAKE_COMMAND") ).url() );
     buildDirGrp.writeEntry( Config::Specific::cmakeInstallDirKey, KUrl( model->value("CMAKE_INSTALL_PREFIX") ).url() );
     buildDirGrp.writeEntry( Config::Specific::cmakeBuildTypeKey, model->value("CMAKE_BUILD_TYPE") );
-    buildDirGrp.sync();
     if (deleteModel)
         delete model;
 }
@@ -433,7 +476,6 @@ void attemptMigrate( KDevelop::IProject* project )
     baseGrp.deleteEntry( Config::Old::currentEnvironmentKey );
     baseGrp.deleteEntry( Config::Old::currentExtraArgumentsKey );
     baseGrp.deleteEntry( Config::Old::projectBuildDirs );
-    baseGrp.sync();
 }
 
 void setOverrideBuildDirIndex( KDevelop::IProject* project, int overrideBuildDirIndex )
@@ -451,7 +493,6 @@ void removeOverrideBuildDirIndex( KDevelop::IProject* project, bool writeToMainI
         baseGrp.writeEntry( Config::buildDirIndexKey, baseGrp.readEntry(Config::buildDirOverrideIndexKey) );
 
     baseGrp.deleteEntry(Config::buildDirOverrideIndexKey);
-    baseGrp.sync();
 }
 
 ICMakeDocumentation* cmakeDocumentation()
