@@ -22,12 +22,15 @@
 #define CMAKEUTILS_H
 
 #include "cmakeexport.h"
+#include <KUrl>
 #include <QStringList>
+#include <util/path.h>
 
 namespace KDevelop
 {
     class ProjectBaseItem;
     class IProject;
+    class Path;
 }
 
 class KUrl;
@@ -43,7 +46,7 @@ namespace CMake
      *
      * @returns true if configure should be run, false otherwise
      */
-    KDEVCMAKECOMMON_EXPORT bool checkForNeedingConfigure( KDevelop::ProjectBaseItem* item );
+    KDEVCMAKECOMMON_EXPORT bool checkForNeedingConfigure( KDevelop::IProject* project );
 
     /**
      * @returns the current builddir for the given project or an empty url if none
@@ -77,6 +80,11 @@ namespace CMake
      */
     KDEVCMAKECOMMON_EXPORT QString projectRootRelative( KDevelop::IProject* project );
     
+    /**
+     * @returns whether there's projectRootRelative defined
+     */
+    KDEVCMAKECOMMON_EXPORT bool hasProjectRootRelative( KDevelop::IProject* project );
+    
     /**
      * Convenience function to get the project root.
      */
@@ -127,16 +135,6 @@ namespace CMake
      */
     KDEVCMAKECOMMON_EXPORT QStringList allBuildDirs(KDevelop::IProject* project);
 
-    /**
-     * Returns current CMake modules directory for @p project.
-     */
-    KDEVCMAKECOMMON_EXPORT KUrl cmakeDirectory( KDevelop::IProject* project );
-
-    /**
-     * Sets current CMake modules directory for @p project.
-     */
-    KDEVCMAKECOMMON_EXPORT void setCmakeDirectory( KDevelop::IProject* project, const KUrl& url );
-
     /**
      * Attempts to migrate the CMake configuration to per-builddir format.
      * Silently returns if the migration has already been performed.
@@ -195,6 +193,8 @@ namespace CMake
      * Override is then cleared and index set to -1.
      */
     KDEVCMAKECOMMON_EXPORT void removeBuildDirConfig( KDevelop::IProject* project );
+
+    KDEVCMAKECOMMON_EXPORT KDevelop::Path::List resolveSystemDirs(KDevelop::IProject* project, const QStringList& dirs);
 }
 
 #endif
