@@ -29,14 +29,19 @@ class KDEVCMAKECOMMON_EXPORT CMakeCondition
         CMakeCondition(const CMakeProjectVisitor* v);
         bool condition(const QStringList &expression);
         QList<int> variableArguments() const { return m_argUses; }
+        QStringList matches() const { return m_matches; }
         
         enum conditionToken { None=0, variable, NOT, AND, OR, COMMAND, EXISTS, IS_NEWER_THAN, IS_DIRECTORY, IS_ABSOLUTE, MATCHES,
             LESS, GREATER, EQUAL, STRLESS, STRGREATER, STREQUAL, DEFINED, LPR, RPR, VERSION_LESS, VERSION_EQUAL, VERSION_GREATER, Last };
+
+        static bool textIsTrue(const QString& text);
     private:
         static conditionToken typeName(const QString& name);
         QStringList::const_iterator prevOperator(QStringList::const_iterator it, QStringList::const_iterator itStop) const;
         bool evaluateCondition(QStringList::const_iterator it, QStringList::const_iterator itEnd);
         bool isTrue(const QStringList::const_iterator& var);
+        int compareVersion(QStringList::const_iterator left, QStringList::const_iterator right, bool* ok);
+        QString value(QStringList::const_iterator it);
         const VariableMap* m_vars;
         const CacheValues* m_cache;
         const CMakeProjectVisitor *m_visitor;
@@ -52,6 +57,7 @@ class KDEVCMAKECOMMON_EXPORT CMakeCondition
 
         QStringList::const_iterator conditionBegin;
         QRegExp m_numberRx;
+        QStringList m_matches;
 };
 
 #endif
