@@ -72,7 +72,7 @@ QString CMakeFunctionArgument::unescapeValue(const QString& value)
 void CMakeFunctionDesc::addArguments( const QStringList& args, bool addEvenIfEmpty )
 {
     if(addEvenIfEmpty && args.isEmpty())
-        arguments += QString();
+        arguments += CMakeFunctionArgument();
     else foreach( const QString& arg, args )
     {
         CMakeFunctionArgument cmakeArg( arg );
@@ -94,18 +94,22 @@ QString CMakeFunctionDesc::writeBack() const
     return output;
 }
 
-CMakeFileContent CMakeListsParser::readCMakeFile(const QString & fileName)
+CMakeFileContent CMakeListsParser::readCMakeFile(const QString & _fileName)
 {
     cmListFileLexer* lexer = cmListFileLexer_New();
     if ( !lexer )
         return CMakeFileContent();
-    if ( !cmListFileLexer_SetFileName( lexer, qPrintable( fileName ) ) ) {
-        kDebug(9042) << "cmake read error. could not read " << fileName;
+    if ( !cmListFileLexer_SetFileName( lexer, qPrintable( _fileName ) ) ) {
+        kDebug(9042) << "cmake read error. could not read " << _fileName;
         cmListFileLexer_Delete(lexer);
         return CMakeFileContent();
     }
 
     CMakeFileContent ret;
+    KUrl u = KUrl::fromPath(_fileName);
+    u.cleanPath();
+    QString fileName = u.toLocalFile();
+
     bool readError = false, haveNewline = true;
     cmListFileLexer_Token* token;
 
@@ -123,12 +127,12 @@ CMakeFileContent CMakeListsParser::readCMakeFile(const QString & fileName)
             {
                 haveNewline = false;
                 CMakeFunctionDesc function;
-                function.name = QString(token->text).toLower();
+                function.name = QString::fromLocal8Bit(token->text).toLower();
                 function.filePath = fileName;
                 function.line = token->line;
                 function.column = token->column;
 
-                readError = !readCMakeFunction( lexer, function, fileName );
+                readError = !readCMakeFunction( lexer, function);
                 ret.append(function);
 
                 if(readError)
@@ -143,7 +147,7 @@ CMakeFileContent CMakeListsParser::readCMakeFile(const QString & fileName)
     return ret;
 }
 
-bool CMakeListsParser::readCMakeFunction(cmListFileLexer *lexer, CMakeFunctionDesc &func, const QString & fileName)
+bool CMakeListsParser::readCMakeFunction(cmListFileLexer *lexer, CMakeFunctionDesc &func)
 {
         // Command name has already been parsed.  Read the left paren.
     cmListFileLexer_Token* token;
@@ -157,7 +161,6 @@ bool CMakeListsParser::readCMakeFunction(cmListFileLexer *lexer, CMakeFunctionDe
     }
 
     // Arguments.
-    unsigned long lastLine = cmListFileLexer_GetCurrentLine(lexer);
     int parenthesis=1;
     while((token = cmListFileLexer_Scan(lexer)))
     {
@@ -172,25 +175,24 @@ bool CMakeListsParser::readCMakeFunction(cmListFileLexer *lexer, CMakeFunctionDe
                 } else if(parenthesis<0)
                     return false;
                 else
-                    func.arguments << CMakeFunctionArgument( token->text, false, fileName, token->line, token->column );
+                    func.arguments << CMakeFunctionArgument( QString::fromLocal8Bit(token->text), false, token->line, token->column );
                 break;
             case cmListFileLexer_Token_ParenLeft:
                 parenthesis++;
-                func.arguments << CMakeFunctionArgument( token->text, false, fileName, token->line, token->column );
+                func.arguments << CMakeFunctionArgument( QString::fromLocal8Bit(token->text), false, token->line, token->column );
                 break;
             case cmListFileLexer_Token_Identifier:
             case cmListFileLexer_Token_ArgumentUnquoted:
-                func.arguments << CMakeFunctionArgument( token->text, false, fileName, token->line, token->column );
+                func.arguments << CMakeFunctionArgument( QString::fromLocal8Bit(token->text), false, token->line, token->column );
                 break;
             case cmListFileLexer_Token_ArgumentQuoted:
-                func.arguments << CMakeFunctionArgument( token->text, true, fileName, token->line, token->column+1 );
+                func.arguments << CMakeFunctionArgument( QString::fromLocal8Bit(token->text), true, token->line, token->column+1 );
                 break;
             case cmListFileLexer_Token_Newline:
                 break;
             default:
                 return false;
         }
-        lastLine = cmListFileLexer_GetCurrentLine(lexer);
     }
 
     return false;
@@ -199,11 +201,19 @@ bool CMakeListsParser::readCMakeFunction(cmListFileLexer *lexer, CMakeFunctionDe
 
 CMakeFunctionDesc::CMakeFunctionDesc(const QString& name, const QStringList& args)
     : name(name)
+    , line(0)
+    , column(0)
+    , endLine(0)
+    , endColumn(0)
 {
     addArguments(args);
 }
 
 CMakeFunctionDesc::CMakeFunctionDesc()
+    : line(0)
+    , column(0)
+    , endLine(0)
+    , endColumn(0)
 {}
 
 bool CMakeFunctionDesc::operator==(const CMakeFunctionDesc & other) const
@@ -228,10 +238,13 @@ bool CMakeFunctionDesc::operator==(const CMakeFunctionDesc & other) const
     value=unescapeValue(value);
 }*/
 
-CMakeFunctionArgument::CMakeFunctionArgument(const QString & v, bool q, const QString &, quint32 l, quint32 c)
-    : value(v), quoted(q)/*, filePath(file)*/, line(l), column(c)
+CMakeFunctionArgument::CMakeFunctionArgument(const QString& v, bool q, quint32 l, quint32 c)
+    : value(unescapeValue(v)), quoted(q), line(l), column(c)
 {
-    value=unescapeValue(value);
 }
 
+CMakeFunctionArgument::CMakeFunctionArgument(const QString& v)
+    : value(v), quoted(false), line(0), column(0)
+{
+}
 
