@@ -23,12 +23,10 @@
 
 #include <QString>
 #include <QStringList>
-#include <QMap>
 #include <QHash>
 #include <QStack>
 
 #include "cmakeastvisitor.h"
-#include "cmakelistsparser.h"
 #include "cmaketypes.h"
 #include <language/duchain/topducontext.h>
 
@@ -87,8 +85,10 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
         virtual int visit( const SetDirectoryPropsAst * );
         virtual int visit( const StringAst * );
         virtual int visit( const SubdirsAst * );
-        virtual int visit( const TryCompileAst * );
+        virtual int visit( const SetTestsPropsAst* );
         virtual int visit( const TargetLinkLibrariesAst * );
+        virtual int visit( const TargetIncludeDirectoriesAst* );
+        virtual int visit( const TryCompileAst * );
         virtual int visit( const GetCMakePropertyAst * );
         virtual int visit( const GetPropertyAst* );
         virtual int visit( const RemoveDefinitionsAst * );
@@ -109,14 +109,13 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
 
         const VariableMap* variables() const { return m_vars; }
         const CacheValues* cache() const { return m_cache; }
-        const CMakeDefinitions& definitions() const { return m_defs; }
+        CMakeDefinitions definitions() const { return m_defs; }
         
         QString projectName() const { return m_projectName; }
-        QList<Subdirectory> subdirectories() const { return m_subdirectories; }
-        QList<Target> targets() const { return m_targetForId.values(); }
+        QVector<Subdirectory> subdirectories() const { return m_subdirectories; }
+        QVector<Target> targets() const { return m_targetForId.values().toVector(); }
         QStringList resolveDependencies(const QStringList& target) const;
-        QStringList includeDirectories() const { return m_includeDirectories; }
-        QList<Test> testSuites() const { return m_testSuites; }
+        QVector<Test> testSuites() const { return m_testSuites; }
             
         int walk(const CMakeFileContent& fc, int line, bool isClean=false);
         
@@ -143,6 +142,8 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
         static void setMessageCallback(message_callback f) { s_msgcallback=f; }
         
         QStringList variableValue(const QString& var) const;
+        void setProperties(const CMakeProperties& properties) { m_props = properties; }
+        QHash<QString, QString> targetAlias() { return m_targetAlias; }
         
     protected:
         struct IntPair
@@ -159,14 +160,14 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
             }
         };
         
-        static QList< IntPair > parseArgument(const QString &exp);
+        static QList<IntPair> parseArgument(const QString &exp);
         
     private:
         QStringList envVarDirectories(const QString &varName) const;
         static message_callback s_msgcallback;
         
         static KDevelop::ReferencedTopDUContext
-            createContext(const KUrl& path, KDevelop::ReferencedTopDUContext aux, int endl ,int endc, bool isClean);
+            createContext(const KDevelop::IndexedString& path, KDevelop::ReferencedTopDUContext aux, int endl ,int endc, bool isClean);
         
         void macroDeclaration(const CMakeFunctionDesc& def, const CMakeFunctionDesc& end, const QStringList& args);
         CMakeFunctionDesc resolveVariables(const CMakeFunctionDesc &exp);
@@ -174,7 +175,6 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
         QStringList theValue(const QString& exp, const IntPair& p) const;
         
         void defineTarget(const QString& id, const QStringList& sources, Target::Type t);
-        int notImplemented(const QString& n) const;
         bool haveToFind(const QString &varName);
         void createDefinitions(const CMakeAst* ast);
         void createUses(const CMakeFunctionDesc& ast);
@@ -189,10 +189,9 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
         CMakeProperties m_props;
         QStringList m_modulePath;
         QString m_projectName;
-        QList<Subdirectory> m_subdirectories;
-        QStringList m_includeDirectories;
-        QMap<QString, QStringList> m_generatedFiles;
-        QMap<QString, Target> m_targetForId;
+        QVector<Subdirectory> m_subdirectories;
+        QHash<QString, QStringList> m_generatedFiles;
+        QHash<QString, Target> m_targetForId;
         
         QStack<VisitorState> m_backtrace;
         QString m_root;
@@ -203,9 +202,11 @@ class KDEVCMAKECOMMON_EXPORT CMakeProjectVisitor : CMakeAstVisitor
         KDevelop::ReferencedTopDUContext m_topctx;
         KDevelop::ReferencedTopDUContext m_parentCtx;
         bool m_hitBreak;
+        bool m_hitReturn;
         QMap<QString, QString> m_environmentProfile;
+        QHash<QString, QString> m_targetAlias;
 
-        QList<Test> m_testSuites;
+        QVector<Test> m_testSuites;
 };
 
 #endif
