@@ -30,69 +30,50 @@
 #include <interfaces/icore.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/iruncontroller.h>
+#include <interfaces/iprojectcontroller.h>
 #include <project/interfaces/ibuildsystemmanager.h>
 #include <project/projectmodel.h>
+#include <QFileInfo>
 
 using namespace KDevelop;
 
-void CTestUtils::createTestSuites(const QList< Test >& testSuites, ProjectFolderItem* folder)
+void CTestUtils::createTestSuites(const QVector<Test>& testSuites, ProjectFolderItem* folder)
 {
-    QString binDir = folder->project()->buildSystemManager()->buildDirectory(folder->project()->projectItem()).toLocalFile();
-    KUrl currentBinDir = folder->project()->buildSystemManager()->buildDirectory(folder);
-    KUrl currentSourceDir = folder->url();
-    
+    IProject* project = folder->project();
+    IBuildSystemManager* bsm = project->buildSystemManager();
+    const QString binDir = bsm->buildDirectory(folder->project()->projectItem()).toLocalFile();
+    const Path currentBinDir = bsm->buildDirectory(folder);
+    const Path currentSourceDir = folder->path();
+
     foreach (const Test& test, testSuites)
     {
+        KUrl::List files;
         QString exe = test.executable;
-        if (test.isTarget)
+        QString targetName = QFileInfo(exe).fileName();
+        QList<ProjectTargetItem*> items = bsm->targets(folder);
+        foreach (ProjectTargetItem* item, items)
         {
-            QList<ProjectTargetItem*> items = folder->project()->buildSystemManager()->targets(folder);
-            foreach (ProjectTargetItem* item, items)
+            ProjectExecutableTargetItem * exeTgt = item->executable();
+            if (exeTgt && (exeTgt->text() == test.executable || exeTgt->text()==targetName))
             {
-                ProjectExecutableTargetItem * exeTgt = item->executable();
-                if (exeTgt == 0)
-                {
-                    continue;
-                }
-                if (exeTgt->text() == test.executable)
-                {
-                    exe = exeTgt->builtUrl().toLocalFile();
-                    kDebug(9042) << "Found proper test target path" << test.executable << "->" << exe;
-                    break;
+                exe = exeTgt->builtUrl().toLocalFile();
+                kDebug(9042) << "Found proper test target path" << test.executable << "->" << exe;
+                foreach(ProjectFileItem* file, exeTgt->fileList()) {
+                    files += file->url();
                 }
+                break;
             }
         }
         exe.replace("#[bin_dir]", binDir);
-        KUrl exeUrl = KUrl(exe);
-        if (exeUrl.isRelative())
-        {
-            exeUrl = currentBinDir;
-            exeUrl.addPath(test.executable);
-        }
-        
-        QStringList files;
-        foreach (const QString& file, test.files)
-        {
-            KUrl fileUrl(file);
-            if (fileUrl.isRelative())
-            {
-                fileUrl = currentSourceDir;
-                fileUrl.addPath(file);
-            }
+        const Path exePath(currentBinDir, exe);
 
-            QString localFile = fileUrl.toLocalFile();
-            localFile.replace("#[bin_dir]", binDir);
-            files << localFile;
-        }
-        
-        
         QStringList args = test.arguments;
-        for (QStringList::iterator it = args.begin(); it != args.end(); ++it)
+        for (QStringList::iterator it = args.begin(), end = args.end(); it != end; ++it)
         {
-            (*it).replace("#[bin_dir]", binDir);
+            it->replace("#[bin_dir]", binDir);
         }
-        
-        CTestSuite* suite = new CTestSuite(test.name, exeUrl, files, folder->project(), args);
+
+        CTestSuite* suite = new CTestSuite(test.name, exePath.toUrl(), files, project, args, test.properties.value("WILL_FAIL", "FALSE") == "TRUE");
         ICore::self()->runController()->registerJob(new CTestFindJob(suite));
     }
 }
