@@ -21,10 +21,12 @@
 #include "cmakeloadprojecttest.h"
 #include "cmake-test-paths.h"
 
+#include <qtest_kde.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/topducontext.h>
 
+#include "cmake-test-paths.h"
 
 #include "cmListFileLexer.h"
 #include "cmakelistsparser.h"
@@ -32,21 +34,25 @@
 #include "cmakeast.h"
 #include <cmakeparserutils.h>
 #include <tests/autotestshell.h>
+#include <tests/testcore.h>
 #include <cmakeprojectdata.h>
 
-QTEST_MAIN( CMakeLoadProjectTest )
+inline QDebug &operator<<(QDebug debug, const Target &target)
+{ debug << target.name; return debug.maybeSpace(); }
+
+QTEST_KDEMAIN( CMakeLoadProjectTest, GUI )
 
 using namespace KDevelop;
 
-CMakeLoadProjectTest::CMakeLoadProjectTest()
+void CMakeLoadProjectTest::initTestCase()
 {
     AutoTestShell::init();
-    KDevelop::Core::initialize(0, KDevelop::Core::NoUi);
+    TestCore::initialize(Core::NoUi);
 }
 
-CMakeLoadProjectTest::~CMakeLoadProjectTest()
+void CMakeLoadProjectTest::cleanupTestCase()
 {
-    KDevelop::Core::self()->shutdown();
+    TestCore::shutdown();
 }
 
 void CMakeLoadProjectTest::testTinyCMakeProject()
@@ -58,6 +64,26 @@ void CMakeLoadProjectTest::testTinyCMakeProject()
     QCOMPARE(v.vm.value("CMAKE_INCLUDE_CURRENT_DIR"), QStringList("OFF"));
 }
 
+void CMakeLoadProjectTest::testBug335803()
+{
+    CMakeProjectData v = parseProject(CMAKE_TESTS_PROJECTS_DIR "/bug335803");
+    QCOMPARE(v.projectName, QString("bug335803"));
+    QStringList names;
+    foreach(const Target& t, v.targets) {
+        names << t.name;
+    }
+    QCOMPARE(v.targets.count(), 8);
+    names.sort();
+    QCOMPARE(names[0], QLatin1String("echo-a"));
+    QCOMPARE(names[1], QLatin1String("echo-b"));
+    QCOMPARE(names[2], QLatin1String("echo-custom_name"));
+    QCOMPARE(names[3], QLatin1String("echo-d"));
+    QCOMPARE(names[4], QLatin1String("echo2-a"));
+    QCOMPARE(names[5], QLatin1String("echo2-b"));
+    QCOMPARE(names[6], QLatin1String("echo2-custom_name"));
+    QCOMPARE(names[7], QLatin1String("echo2-d")); // This one was missing before bug #335803 got fixed
+}
+
 void CMakeLoadProjectTest::testSmallQt4Project()
 {
     CMakeProjectData v = parseProject(CMAKE_TESTS_PROJECTS_DIR "/qt4app");
@@ -67,22 +93,37 @@ void CMakeLoadProjectTest::testSmallQt4Project()
     QCOMPARE(v.targets.at( 0 ).files, QStringList() << "qt4app.cpp" << "main.cpp" );
 }
 
+int findTarget(const QVector<Target>& targets, const QString& name)
+{
+    for(int i=0, count=targets.count(); i<count; ++i) {
+        if(targets[i].name==name)
+            return i;
+    }
+    return -1;
+}
 
 void CMakeLoadProjectTest::testSmallKDE4Project()
 {
     CMakeProjectData v = parseProject(CMAKE_TESTS_PROJECTS_DIR "/kde4app");
     QCOMPARE(v.targets.count(), 3);
     QCOMPARE(v.projectName, QString("kde4app"));
-    QCOMPARE(v.targets.at( 0 ).name, QString("kde4app") );
-    QCOMPARE(v.targets.at( 0).files, QStringList() << "kde4app.cpp" << "main.cpp" << "kde4appview.cpp" 
-                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/ui_kde4appview_base.h" 
-                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/ui_prefs_base.h" 
-                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/settings.cpp" 
+
+    int idx = findTarget(v.targets, "kde4app");
+    QVERIFY(idx>=0);
+    QCOMPARE(v.targets.at( idx ).name, QString("kde4app") );
+    QCOMPARE(v.targets.at( idx ).files, QStringList() << "kde4app.cpp" << "main.cpp" << "kde4appview.cpp"
+                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/ui_kde4appview_base.h"
+                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/ui_prefs_base.h"
+                                                      << CMAKE_TESTS_PROJECTS_DIR "/kde4app/settings.cpp"
                                                       << CMAKE_TESTS_PROJECTS_DIR "/kde4app/settings.h" );
-    QCOMPARE(v.targets.at( 1 ).name, QString("testkde4app") );
-    
-    QCOMPARE(v.targets.at( 1 ).files, QStringList() << "kde4app.cpp");
-    QCOMPARE(v.targets.at( 2 ).name, QString("uninstall") );
+    int testIdx = findTarget(v.targets, "testkde4app");
+    QVERIFY(testIdx>=0);
+    QCOMPARE(v.targets.at( testIdx ).name, QString("testkde4app") );
+    QCOMPARE(v.targets.at( testIdx ).files, QStringList() << "kde4app.cpp");
+
+    int uninstallIdx = findTarget(v.targets, "uninstall");
+    QVERIFY(uninstallIdx>=0);
+    QCOMPARE(v.targets.at( uninstallIdx ).name, QString("uninstall") );
     QCOMPARE(v.vm.value("CMAKE_INCLUDE_CURRENT_DIR"), QStringList("ON"));
 }
 
@@ -92,11 +133,9 @@ void CMakeLoadProjectTest::testSmallProjectWithTests()
     QCOMPARE(v.testSuites.count(), 5);
     QCOMPARE(v.projectName, QString("unittests"));
     
-    QCOMPARE(v.testSuites.at(0).files, QStringList() << "success.cpp");
     QCOMPARE(v.testSuites.at(0).name, QString("success"));
     QCOMPARE(v.testSuites.at(0).arguments.count(), 0);
     
-    QCOMPARE(v.testSuites.at(3).files, QStringList() << "math_test.cpp");
     QCOMPARE(v.testSuites.at(3).name, QString("test_four"));
     QCOMPARE(v.testSuites.at(3).arguments.count(), 1);
     QCOMPARE(v.testSuites.at(3).arguments.at(0), QString("4"));
@@ -105,13 +144,11 @@ void CMakeLoadProjectTest::testSmallProjectWithTests()
 void CMakeLoadProjectTest::testKDE4ProjectWithTests()
 {
     CMakeProjectData v = parseProject(CMAKE_TESTS_PROJECTS_DIR "/unit_tests_kde");
-    QCOMPARE(v.testSuites.count(), 1);
+    QCOMPARE(v.testSuites.count(), 1); //cmake-test-unittestskde
     QCOMPARE(v.projectName, QString("unittestskde"));
     
-    QCOMPARE(v.testSuites.at(0).files, QStringList() << "test.cpp");
     QCOMPARE(v.testSuites.at(0).name, QString("cmake-test-unittestskde"));
     QCOMPARE(v.testSuites.at(0).arguments.count(), 0);
-    QVERIFY(!KUrl(v.testSuites.at(0).executable).isRelative());
 }
 
 CMakeProjectData CMakeLoadProjectTest::parseProject( const QString& sourcedir )
@@ -123,6 +160,7 @@ CMakeProjectData CMakeLoadProjectTest::parseProject( const QString& sourcedir )
     CMakeProjectData data;
     data.vm = initials.first;
     data.vm.insert("CMAKE_SOURCE_DIR", QStringList(sourcedir));
+    data.vm.insert("CMAKE_PREFIX_PATH", QString::fromLatin1(TEST_PREFIX_PATH).split(';', QString::SkipEmptyParts));
     
     KDevelop::ReferencedTopDUContext buildstrapContext=new TopDUContext(IndexedString("buildstrap"), RangeInRevision(0,0, 0,0));
     DUChain::self()->addDocumentChain(buildstrapContext);
@@ -143,12 +181,16 @@ CMakeProjectData CMakeLoadProjectTest::parseProject( const QString& sourcedir )
     v.setMacroMap(&data.mm);
     v.setCacheValues(&data.cache);
     v.setModulePath(modulesPath);
+    v.setProperties(data.properties);
+    QMap<QString, QString> env;
+    env["CMAKE_PREFIX_PATH"] = QString::fromLatin1(TEST_ENV_PREFIX_PATH);
+    env["CMAKE_INCLUDE_PATH"] = QString::fromLatin1(TEST_ENV_INCLUDE_PATH);
+    env["CMAKE_LIBRARY_PATH"] = QString::fromLatin1(TEST_ENV_LIBRARY_PATH);
+    v.setEnvironmentProfile( env );
     v.walk(code, 0);
     
     data.projectName=v.projectName();
     data.subdirectories=v.subdirectories();
-    data.definitions=v.definitions();
-    data.includeDirectories=v.includeDirectories();
     data.targets=v.targets();
     data.properties=v.properties();
     data.testSuites=v.testSuites();
