@@ -48,17 +48,26 @@ void CTestFindSuitesTest::initTestCase()
 {
     AutoTestShell::init();
     TestCore::initialize();
-    DUChain::self()->disablePersistentStorage();
+
+    cleanup();
+}
+
+void CTestFindSuitesTest::cleanup()
+{
+    foreach(IProject* p, ICore::self()->projectController()->projects()) {
+        ICore::self()->projectController()->closeProject(p);
+    }
+    QVERIFY(ICore::self()->projectController()->projects().isEmpty());
 }
 
 void CTestFindSuitesTest::cleanupTestCase()
 {
-    Core::self()->shutdown();
+    TestCore::shutdown();
 }
 
 void CTestFindSuitesTest::testCTestSuite()
 {
-    IProject* project = parseProject( "unit_tests" );
+    IProject* project = loadProject( "unit_tests" );
     QVERIFY2(project, "Project was not opened");
     WAIT_FOR_SUITES(5, 10)
     QList<ITestSuite*> suites = ICore::self()->testController()->testSuitesForProject(project);
@@ -79,50 +88,24 @@ void CTestFindSuitesTest::testCTestSuite()
 
 void CTestFindSuitesTest::testQtTestSuite()
 {
-    IProject* project = parseProject( "unit_tests_kde" );
+    IProject* project = loadProject( "unit_tests_kde" );
     QVERIFY2(project, "Project was not opened");
-    WAIT_FOR_SUITES(2, 10)
+    WAIT_FOR_SUITES(1, 10)
     QList<ITestSuite*> suites = ICore::self()->testController()->testSuitesForProject(project);
     
-    QCOMPARE(suites.size(), 2);
-    ITestSuite* suite = suites.first();
+    QCOMPARE(suites.size(), 1);
+    CTestSuite* suite = static_cast<CTestSuite*>(suites.first());
+    QVERIFY(suite);
     QCOMPARE(suite->cases().size(), 5);
 
     DUChainReadLocker locker(DUChain::lock());
     QVERIFY(suite->declaration().isValid());
 
-    foreach (ITestSuite* suite, suites)
-    {
-        CTestSuite* ctest = (CTestSuite*)(suite);
-        QString exeSubdir = KUrl::relativeUrl(project->folder(), ctest->executable().directory());
-        QCOMPARE(exeSubdir, ctest->name() == "unittestskde-nonstd-location" ? QString("build/bin") : QString("build") );
-    }
+    QString exeSubdir = KUrl::relativeUrl(project->folder(), suite->executable().directory());
+    QCOMPARE(exeSubdir, QString("build") );
 
     foreach (const QString& testCase, suite->cases())
     {
         QVERIFY(suite->caseDeclaration(testCase).isValid());
     }
 }
-
-IProject* CTestFindSuitesTest::parseProject( const QString& name)
-{
-    const TestProjectPaths paths = projectPaths(name);
-    defaultConfigure(paths);
-    
-    ICore::self()->projectController()->openProject(paths.projectFile);
-    
-    IProject* project = ICore::self()->projectController()->findProjectByName(name);
-    int t = 0;
-    const int timeout = 100;
-    while (!project && t < 30000)
-    {
-        t += timeout;
-        QTest::kWaitForSignal(ICore::self()->projectController(), SIGNAL(projectOpened(KDevelop::IProject*)), timeout);
-        project = ICore::self()->projectController()->findProjectByName(name);
-    }
-        
-    return project;
-}
-
-#include "ctestfindsuitestest.moc"
-
