@@ -14,22 +14,8 @@
 
 #include <project/abstractfilemanagerplugin.h>
 #include <project/interfaces/ibuildsystemmanager.h>
-#include <interfaces/iplugin.h>
-#include <QtCore/QVariant>
 
-template <typename T> class QList;
-class QString;
-
-namespace KDevelop
-{
-    class IProject;
-    class ProjectItem;
-    class ProjectFolderItem;
-    class ProjectBaseItem;
-    class ProjectFileItem;
-    class ProjectTargetItem;
-    class IProjectBuilder;
-}
+class IMakeBuilder;
 
 class CustomMakeManager : public KDevelop::AbstractFileManagerPlugin,
                           public KDevelop::IBuildSystemManager
@@ -53,7 +39,7 @@ public:
     /**
      * Provide a list of include directories.
      */
-    virtual KUrl::List includeDirectories(KDevelop::ProjectBaseItem*) const;
+    virtual KDevelop::Path::List includeDirectories(KDevelop::ProjectBaseItem*) const;
 
     /**
      * Provide a list of files that contain the preprocessor defines for the
@@ -98,7 +84,7 @@ public:
     /**
      * Get the toplevel build directory for the project
      */
-    virtual KUrl buildDirectory(KDevelop::ProjectBaseItem*) const;
+    virtual KDevelop::Path buildDirectory(KDevelop::ProjectBaseItem*) const;
 
     /**
      * Get a list of all the targets in this project
@@ -112,22 +98,26 @@ public:
     virtual QList<KDevelop::ProjectTargetItem*> targets(KDevelop::ProjectFolderItem*) const;
 
 protected:
-    virtual bool isValid(const KUrl& url, const bool isFolder, KDevelop::IProject* project) const;
-    virtual KDevelop::ProjectFileItem* createFileItem(KDevelop::IProject* project, const KUrl& url, KDevelop::ProjectBaseItem* parent);
-    virtual KDevelop::ProjectFolderItem* createFolderItem(KDevelop::IProject* project, const KUrl& url, KDevelop::ProjectBaseItem* parent = 0);
+    virtual KDevelop::ProjectFileItem* createFileItem(KDevelop::IProject* project,
+                                                      const KDevelop::Path& path,
+                                                      KDevelop::ProjectBaseItem* parent);
+    virtual KDevelop::ProjectFolderItem* createFolderItem(KDevelop::IProject* project,
+                                                          const KDevelop::Path& path,
+                                                          KDevelop::ProjectBaseItem* parent = 0);
+
+private slots:
+    void reloadMakefile(KDevelop::ProjectFileItem *item);
 
 private:
     /**
      * Initialize targets by reading Makefile in @arg dir
      * @return Target lists in Makefile at @arg dir.
      */
-    QStringList parseCustomMakeFile( const KUrl &makefile );
+    QStringList parseCustomMakeFile( const KDevelop::Path &makefile );
 
-private slots:
-    void slotDirty(const QString& path);
+    void createTargetItems(KDevelop::IProject* project, const KDevelop::Path& path, KDevelop::ProjectBaseItem* parent);
 
 private:
-    class Private;
-    Private *d;
+    IMakeBuilder *m_builder;
 };
 #endif
